@@ -71,8 +71,12 @@ public final class ConfigKeys {
 		public static final String ICFG_NETWORK_TCP_WRITE_SELECT_MIN_TIME = "network.tcp.write.select.min.time";
 		public static final String BCFG_IPV_6_ENABLE_SUPPORT = "IPV6 Enable Support";
 		public static final String BCFG_IPV_6_CHECK_MULTIPLE_ADDRESS_CHECKS = "IPV6 Enable Multiple Address Checks";
+		public static final String SCFG_IPV_6_EXTRA_GLOBALS = "IPV6 Extra Globals";
 		public static final String BCFG_IPV_6_PREFER_ADDRESSES = "IPV6 Prefer Addresses";
 		public static final String BCFG_IPV_4_PREFER_STACK = "IPV4 Prefer Stack";
+		public static final String BCFG_IPV_4_IGNORE_NI_ADDRESSES = "IPV4 Ignore Network Addresses";
+		public static final String BCFG_IPV_6_IGNORE_NI_ADDRESSES = "IPV6 Ignore Network Addresses";
+
 		public static final String SCFG_DNS_ALT_SERVERS = "DNS Alt Servers";
 		public static final String BCFG_DNS_ALT_SERVERS_SOCKS_ENABLE = "DNS Alt Servers SOCKS Enable";
 		public static final String BCFG_NETWORK_TRANSPORT_ENCRYPTED_REQUIRE = "network.transport.encrypted.require";
@@ -126,6 +130,8 @@ public final class ConfigKeys {
 		public static final String ICFG_MAX_FILE_LINKS_SUPPORTED = "Max File Links Supported";
 		public static final String BCFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART = "Insufficient Space Download Restart Enable";
 		public static final String ICFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART_MINS = "Insufficient Space Download Restart Period";
+		public static final String BCFG_MISSING_FILE_DOWNLOAD_RESTART = "Missing File Download Restart Enable";
+		public static final String ICFG_MISSING_FILE_DOWNLOAD_RESTART_MINS = "Missing File Download Restart Period";
 		public static final String BCFG_USE_RESUME = "Use Resume";
 		public static final String ICFG_SAVE_RESUME_INTERVAL = "Save Resume Interval";
 		public static final String BCFG_ON_RESUME_RECHECK_ALL = "On Resume Recheck All";
@@ -233,6 +239,7 @@ public final class ConfigKeys {
 		public static final String SCFG_LOGGING_DIR = "Logging Dir";
 		public static final String ICFG_LOGGING_MAX_SIZE = "Logging Max Size";
 		public static final String SCFG_LOGGING_TIMESTAMP = "Logging Timestamp";
+		public static final String BCFG_LOGGER_DEBUG_FILES_FORCE = "Logger.DebugFiles.Enabled.Force";
 		public static final String ICFG_LOGGER_DEBUG_FILES_SIZE_KB = "Logger.DebugFiles.SizeKB";
 	}
 
@@ -312,6 +319,8 @@ public final class ConfigKeys {
 		public static final String ICFG_TRACKER_CLIENT_MIN_ANNOUNCE_INTERVAL = "Tracker Client Min Announce Interval";
 		public static final String ICFG_TRACKER_CLIENT_CONNECT_TIMEOUT = "Tracker Client Connect Timeout";
 		public static final String ICFG_TRACKER_CLIENT_READ_TIMEOUT = "Tracker Client Read Timeout";
+		public static final String ICFG_TRACKER_CLIENT_CLOSEDOWN_TIMEOUT = "Tracker Client Closedown Timeout";
+		public static final String ICFG_TRACKER_CLIENT_CONCURRENT_ANNOUNCE = "Tracker Client Concurrent Announce";
 		public static final String BCFG_TRACKER_KEY_ENABLE_CLIENT = "Tracker Key Enable Client";
 		public static final String BCFG_TRACKER_SEPARATE_PEER_I_DS = "Tracker Separate Peer IDs";
 		public static final String BCFG_TRACKER_PORT_ENABLE = "Tracker Port Enable";
@@ -325,6 +334,7 @@ public final class ConfigKeys {
 		public static final String BCFG_TRACKER_TOR_ENABLE = "Tracker Tor Enable";
 		public static final String SCFG_TRACKER_TOR_HOST_PORT = "Tracker Tor Host Port";
 		public static final String BCFG_TRACKER_PUBLIC_ENABLE = "Tracker Public Enable";
+		public static final String BCFG_TRACKER_PUBLIC_ENABLE_KNOWN_ONLY = "Tracker Public Enable Known Only";
 		public static final String BCFG_TRACKER_PORT_FORCE_EXTERNAL = "Tracker Port Force External";
 		public static final String BCFG_TRACKER_HOST_ADD_OUR_ANNOUNCE_URLS = "Tracker Host Add Our Announce URLs";
 		public static final String BCFG_TRACKER_PASSWORD_ENABLE_WEB = "Tracker Password Enable Web";
