@@ -160,6 +160,8 @@ public class ConfigurationDefaults {
     def.put(ConfigKeys.Connection.BCFG_IPV_6_CHECK_MULTIPLE_ADDRESS_CHECKS, TRUE );
     def.put("IPV6 Prefer Addresses",FALSE );
     def.put("IPV4 Prefer Stack", FALSE );
+    def.put(ConfigKeys.Connection.BCFG_IPV_4_IGNORE_NI_ADDRESSES, FALSE );
+    def.put(ConfigKeys.Connection.BCFG_IPV_6_IGNORE_NI_ADDRESSES, FALSE );
 
     def.put("max active torrents", new Long(4));
     def.put("max downloads", new Long(4));
@@ -427,8 +429,10 @@ public class ConfigurationDefaults {
     def.put( "Files Auto Tag Best Size", FALSE);
     def.put( ConfigKeys.File.BCFG_FILES_AUTO_TAG_ALLOW_MOD, FALSE );
     
-    def.put( "Insufficient Space Download Restart Enable", FALSE );
-    def.put( "Insufficient Space Download Restart Period", 10 );
+    def.put( ConfigKeys.File.BCFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART, FALSE );
+    def.put( ConfigKeys.File.ICFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART_MINS, 10 );
+    def.put( ConfigKeys.File.BCFG_MISSING_FILE_DOWNLOAD_RESTART, FALSE );
+    def.put( ConfigKeys.File.ICFG_MISSING_FILE_DOWNLOAD_RESTART_MINS, 30 );
 
     def.put( "Play Download Error", FALSE );
     def.put( "Play Download Error File", "" );
@@ -482,6 +486,8 @@ public class ConfigurationDefaults {
     def.put( "Tracker Separate Peer IDs", FALSE);
     def.put( "Tracker Client Connect Timeout", new Long(120));
     def.put( "Tracker Client Read Timeout", SIXTY );
+    def.put( ConfigKeys.Tracker.ICFG_TRACKER_CLIENT_CLOSEDOWN_TIMEOUT, 20L );
+    def.put( ConfigKeys.Tracker.ICFG_TRACKER_CLIENT_CONCURRENT_ANNOUNCE, 32 );
 	def.put( "Tracker Client Send OS and Java Version", FALSE);
 	def.put( "Tracker Client Show Warnings", TRUE);
 	def.put( "Tracker Client Min Announce Interval", ZERO);
@@ -490,7 +496,8 @@ public class ConfigurationDefaults {
 	def.put( ConfigKeys.Tracker.BCFG_TRACKER_CLIENT_SMART_ACTIVATION, TRUE);
 	def.put( "Tracker Client Exclude LAN", TRUE);
 
-    def.put( "Tracker Public Enable", FALSE );
+	def.put( ConfigKeys.Tracker.BCFG_TRACKER_PUBLIC_ENABLE, FALSE );
+	def.put( ConfigKeys.Tracker.BCFG_TRACKER_PUBLIC_ENABLE_KNOWN_ONLY, FALSE );
     def.put( "Tracker Log Enable", FALSE );
     def.put( "Tracker Port Enable", FALSE );
     def.put( "Tracker Port", new Long( TRHost.DEFAULT_PORT ) );
@@ -735,7 +742,8 @@ public class ConfigurationDefaults {
     def.put( "tb.confirm.delete.content", Long.valueOf(0) ); // 0=ask; 1=delete content; 2=delete torrent
 
     def.put( "br.backup.auto.enable", FALSE );
-    def.put( "br.backup.auto.everydays", 1L );
+    def.put( "br.backup.auto.everydays", ONE );
+    def.put( "br.backup.auto.everyhours", ZERO );
     def.put( "br.backup.auto.retain", 5L );
 
     //temp section for SpeedManagerAlgorithmProviderV2
