@@ -25,6 +25,7 @@ import java.io.File;
 
 import com.biglybt.core.disk.impl.piecemapper.DMPieceList;
 import com.biglybt.core.disk.impl.piecemapper.DMPieceMap;
+import com.biglybt.core.peermanager.piecepicker.util.BitFlags;
 import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.util.DirectByteBuffer;
 import com.biglybt.core.util.IndentWriter;
@@ -43,10 +44,15 @@ DiskManager
 	public static final int READY = 4;
 	public static final int FAULTY = 10;
 
+		// don't change these, they are persisted when closing
+	
 	public static final int ET_NONE						= 0;
 	public static final int ET_OTHER					= 1;
 	public static final int ET_INSUFFICIENT_SPACE		= 2;
 	public static final int ET_STOP_DURING_INIT			= 3;
+	public static final int ET_FILE_MISSING				= 4;
+	public static final int ET_READ_ERROR				= 5;
+	public static final int ET_WRITE_ERROR				= 6;
 
 		// CHANGE THIS AND YOU MUST CHANGE NORMAL_REQUEST_SIZE in PeerReadRequest (plugin interface)
 
@@ -188,6 +194,9 @@ DiskManager
 	public int
 	getNbPieces();
 
+	public BitFlags
+	getAvailability();
+	
 	public DiskManagerFileInfo[] getFiles();
 	public DiskManagerFileInfoSet getFileSet();
 	public DiskManagerPiece getPiece(int PieceNumber);
@@ -338,7 +347,7 @@ DiskManager
    */
 
 	public void
-	saveState();
+	saveState( boolean interim );
 
 	/**
 	 * @param pieceNumber
