@@ -56,6 +56,7 @@ import com.biglybt.core.logging.LogAlert;
 import com.biglybt.core.logging.LogEvent;
 import com.biglybt.core.logging.LogIDs;
 import com.biglybt.core.logging.Logger;
+import com.biglybt.core.peermanager.piecepicker.util.BitFlags;
 import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.torrent.TOTorrentFile;
@@ -341,7 +342,7 @@ DiskManagerUtil
 
 	    state.setFileLink( file_info.getIndex(), from_file, to_link );
 
-	    state.save();
+	    state.save(false);
 
 	    return( null );
 	}
@@ -663,13 +664,13 @@ DiskManagerUtil
 								}
 							}
 
-							DiskManagerImpl.storeFileDownloaded( download_manager, res, true );
+							DiskManagerImpl.storeFileDownloaded( download_manager, res, true, false );
 
 							doFileExistenceChecks(this, toChange, download_manager, newStorageType == FileSkeleton.ST_LINEAR || newStorageType == FileSkeleton.ST_REORDER );
 
 						} finally {
 							dmState.suppressStateSave(false);
-							dmState.save();
+							dmState.save(false);
 						}
 
 						return modified;
@@ -857,9 +858,9 @@ DiskManagerUtil
 	    											file_name = prefix + file_name;
 	    										}
 
-			            						File new_parent = new File( parent, dnd_sf );
+			            						File new_parent = FileUtil.newFile( parent, dnd_sf );
 
-			            						File new_file = new File( new_parent, file_name );
+			            						File new_file = FileUtil.newFile( new_parent, file_name );
 
 			            						if ( !new_file.exists()){
 
@@ -897,11 +898,11 @@ DiskManagerUtil
 
 			            					if ( parent != null && parent.canWrite()){
 
-			            						File new_parent = parent.getName().equals( dnd_sf )?parent:new File( parent, dnd_sf );
+			            						File new_parent = parent.getName().equals( dnd_sf )?parent:FileUtil.newFile( parent, dnd_sf );
 
 			            							// use link name to handle incomplete file suffix if set
 
-			            						File new_file = new File( new_parent, link.getName());
+			            						File new_file = FileUtil.newFile( new_parent, link.getName());
 
 			            						if ( new_file.equals( link )){
 
@@ -932,11 +933,11 @@ DiskManagerUtil
 			    											prefix = "";
 			    										}
 
-			    										file = new File( file.getParentFile(), prefix + file_name + incomp_ext );
+			    										file = FileUtil.newFile( file.getParentFile(), prefix + file_name + incomp_ext );
 
 			    									}else if ( prefix_removed ){
 
-			    										file = new File( file.getParentFile(), file_name);
+			    										file = FileUtil.newFile( file.getParentFile(), file_name);
 			    									}
 
 			            							if ( new_file.exists()){
@@ -1047,7 +1048,7 @@ DiskManagerUtil
 		            			}
 		            		}
 
-		            		dataFile = new WeakReference(toReturn = simpleFile != null ? simpleFile : new File( path_str ));
+		            		dataFile = new WeakReference(toReturn = simpleFile != null ? simpleFile : FileUtil.newFile( path_str ));
 
 		            		//System.out.println("new file:"+toReturn);
 		            		return toReturn;
@@ -1661,7 +1662,7 @@ DiskManagerUtil
 					}
 					
 					@Override
-					public void saveState(){
+					public void saveState( boolean interim ){
 					}
 					
 					@Override
@@ -1766,9 +1767,14 @@ DiskManagerUtil
 						return piece_map;
 					}
 					
+					@Override
+					public BitFlags getAvailability(){
+						return null;
+					}
+					
 					@Override
 					public int getPieceLength(int piece_number){
-						return pieces[piece_number].getLength();
+						return( piece_number==nbPieces-1?lastPieceLength:pieceLength );
 					}
 					
 					@Override
@@ -1925,7 +1931,7 @@ DiskManagerUtil
 					}
 					
 					@Override
-					public void setFailed(String reason, Throwable cause){
+					public void setFailed(int type, String reason, Throwable cause){
 					}
 										
 					@Override
