@@ -88,7 +88,10 @@ DownloadManager
 	public static final int ET_OTHER					= DiskManager.ET_OTHER;
 	public static final int ET_INSUFFICIENT_SPACE		= DiskManager.ET_INSUFFICIENT_SPACE;
 	public static final int ET_STOP_DURING_INIT			= DiskManager.ET_STOP_DURING_INIT;
+	public static final int ET_FILE_MISSING				= DiskManager.ET_FILE_MISSING;
 
+	public static final int EF_WAS_FORCE_START			= 0x01;
+	
 	public static final Object UD_KEY_STOP_REASON = Download.UD_KEY_STOP_REASON;
 	
     public void
@@ -391,12 +394,27 @@ DownloadManager
     filesExist(
     	boolean	expected_to_be_allocated );
 
+    /**
+     * Download must be stopped - used at start-of-day to recover error state
+     * @param errorType
+     * @param errorDetails
+     */
+    
+    public void
+    setErrorState(
+    	int		errorType,
+    	String	errorDetails,
+    	int		errorFlags );
+    
     public String
     getErrorDetails();
 
     public int
     getErrorType();
 
+    public int
+    getErrorFlags();
+    
     public DownloadManagerStats
     getStats();
 
@@ -461,7 +479,7 @@ DownloadManager
          */
 
     public void
-    saveDownload();
+    saveDownload( boolean interim );
 
 	/**
 	 * To retreive arbitrary objects against this object.
@@ -491,7 +509,9 @@ DownloadManager
 
     public int getSeedingRank();
 
-    public String isSwarmMerging();
+    public boolean isSwarmMerging();
+    
+    public String getSwarmMergingInfo();
 
     public void setMaxUploads( int max_slots );
 
