@@ -41,7 +41,6 @@ import com.biglybt.core.logging.Logger;
 import com.biglybt.core.networkmanager.LimitedRateGroup;
 import com.biglybt.core.networkmanager.NetworkConnection;
 import com.biglybt.core.networkmanager.NetworkManager;
-import com.biglybt.core.networkmanager.impl.tcp.TCPNetworkManager;
 import com.biglybt.core.peer.*;
 import com.biglybt.core.peermanager.PeerManager;
 import com.biglybt.core.peermanager.PeerManagerRegistration;
@@ -202,7 +201,8 @@ DownloadManagerController
 
 	private String 	errorDetail;
 	private int		errorType	= DownloadManager.ET_NONE;
-
+	private int		errorFlags	= 0;
+	
 	final GlobalManagerStats		global_stats;
 
 	boolean bInitialized = false;
@@ -777,7 +777,8 @@ DownloadManagerController
 
 			errorDetail	= "";
 			errorType	= DownloadManager.ET_NONE;
-
+			errorFlags	= 0;
+			
 			setState( initialising_state, false );
 
 		  	DiskManager dm = DiskManagerFactory.create( download_manager.getTorrent(), download_manager);
@@ -896,6 +897,15 @@ DownloadManagerController
 
 				//already in stopped state, just do removals if necessary
 
+				if ( stateAfterStopping !=  DownloadManager.STATE_ERROR && state == DownloadManager.STATE_ERROR ){
+				
+						// explicitly stopping an error download clears this stuff out
+					
+					errorType	= DownloadManager.ET_NONE;
+					errorDetail	= "";
+					errorFlags	= 0;
+				}
+				
 				if( remove_data ){
 
 					download_manager.deleteDataFiles();
@@ -1008,7 +1018,7 @@ DownloadManagerController
 								// we don't want to update the torrent if we're seeding
 
 								if ( !download_manager.getAssumedComplete()){
-									download_manager.getDownloadState().save();
+									download_manager.getDownloadState().save(false);
 								}
 
 								setDiskManager( null, null );
@@ -1079,6 +1089,8 @@ DownloadManagerController
 
 			control_mon.exit();
 
+			Logger.log(new LogEvent(this, LogIDs.CORE, "Stopped - state=" + getState() + ",error=" + getErrorType() + "/" + getErrorDetail() + "/" + getErrorFlags()));
+
 			download_manager.informStateChanged();
 		}
 	}
@@ -1836,7 +1848,8 @@ DownloadManagerController
 				int	state = getState();
 
 				if (	force_start &&
-						(	state == DownloadManager.STATE_STOPPED ||
+						(	state == DownloadManager.STATE_ERROR ||
+							state == DownloadManager.STATE_STOPPED ||
 							state == DownloadManager.STATE_QUEUED )) {
 
 						// Start it!  (Which will cause a stateChanged to trigger)
@@ -1884,23 +1897,62 @@ DownloadManagerController
 		}
 	}
 	
-	private void
+	protected void
+	setErrorState(
+		int			type,
+		String		reason,
+		int			flags )
+	{
+			// recovering download state on restart
+
+		errorFlags = flags;
+		
+		setFailed( type, reason, true );
+	}
+	
+	protected void
 	setFailed(
 		int			type,
 		String		reason )
 	{
-		if( type == DownloadManager.ET_STOP_DURING_INIT ){
+		setFailed( type, reason, false );
+	}
+	
+	private void
+	setFailed(
+		int			type,
+		String		reason,
+		boolean		recovering )
+	{
+			// all download errors come through here
+		
+		if ( type == DownloadManager.ET_STOP_DURING_INIT ){
 			
 			stopIt( DownloadManager.STATE_STOPPED, false, false, false );
 			
 		}else{
+			
 			if ( reason != null ){
 	
 				errorDetail = reason;
 			}
 	
 			errorType	= type;
-	
+			
+			if ( !recovering ){
+				
+				if ( force_start ){
+					
+						// stopIt will clear down force-start so grab the state before
+					
+					errorFlags = DownloadManager.EF_WAS_FORCE_START;
+					
+				}else{
+					
+					errorFlags = 0;
+				}
+			}
+			
 			stopIt( DownloadManager.STATE_ERROR, false, false, false );
 		}
 	}
@@ -1953,8 +2005,8 @@ DownloadManagerController
 							}
 						}
 
-						setFailed(MessageText.getString("DownloadManager.error.datamissing")
-								+ " " + file);
+						setFailed( DownloadManager.ET_FILE_MISSING, MessageText.getString("DownloadManager.error.datamissing") + ": " + file.getAbsolutePath());
+						
 						return false;
 
 					} else if (fileInfo.getLength() < file.length()) {
@@ -2148,6 +2200,12 @@ DownloadManagerController
 	{
 		return( errorType );
 	}
+	
+	protected int
+	getErrorFlags()
+	{
+		return( errorFlags );
+	}
 
  	protected void
   	setDiskManager(
@@ -2252,17 +2310,38 @@ DownloadManagerController
 
 	@Override
 	public int
-	getUploadRateLimitBytesPerSecond()
+	getEffectiveUploadRateLimitBytesPerSecond()
 	{
 		return( download_manager.getEffectiveUploadRateLimitBytesPerSecond());
 	}
 
+	@Override
+	public int
+	getUploadRateLimitBytesPerSecond()
+	{
+		return( stats.getUploadRateLimitBytesPerSecond());
+	}
+
+	@Override
+	public void
+	setUploadRateLimitBytesPerSecond( int b )
+	{
+		stats.setUploadRateLimitBytesPerSecond( b );
+	}
+	
 	@Override
 	public int
 	getDownloadRateLimitBytesPerSecond()
 	{
 		return( stats.getDownloadRateLimitBytesPerSecond());
 	}
+	
+	@Override
+	public void
+	setDownloadRateLimitBytesPerSecond( int b )
+	{
+		stats.setDownloadRateLimitBytesPerSecond( b );
+	}
 
 		// these per-download rates are not easy to implement as we either have per-peer limits or global limits, with the download-limits being implemented
 		// by adding them to all peers as peer-limits. So for the moment we stick with global (non-lan) limits
