@@ -249,7 +249,8 @@ DownloadManagerStateImpl
 
 	private final TorrentUtils.ExtendedTorrent	torrent;
 
-	private boolean						write_required;
+	private boolean						write_required_soon;
+	private boolean						write_required_sometime;
 
 	private Category 	category;
 
@@ -922,7 +923,7 @@ DownloadManagerStateImpl
 
 			if ( changed ){
 
-				write_required	= true;
+				setDirty( false );
 
 				torrent.setAdditionalMapProperty( TRACKER_CACHE_KEY, value );
 			}
@@ -960,27 +961,40 @@ DownloadManagerStateImpl
 	setResumeData(
 		Map	data )
 	{
+		boolean changed = false;
+
 		try{
 			this_mon.enter();
 
 			// System.out.println( "setting download state/resume data for '" + new String(torrent.getName()));
-
+			
+			Map existing = torrent.getAdditionalMapProperty(RESUME_KEY  );
+			
 			if ( data == null ){
 
 				setLongAttribute( AT_RESUME_STATE, 1 );
+				
+				if ( existing != null ){
+				
+					torrent.removeAdditionalProperty( RESUME_KEY );
 
-				torrent.removeAdditionalProperty( RESUME_KEY );
-
+					changed = true;
+				}
 			}else{
 
-				torrent.setAdditionalMapProperty( RESUME_KEY, data );
+				changed = !BEncoder.mapsAreIdentical( existing, data );
 
+				torrent.setAdditionalMapProperty( RESUME_KEY, data );
+				
 				boolean complete = DiskManagerFactory.isTorrentResumeDataComplete( this );
 
 				setLongAttribute( AT_RESUME_STATE, complete?2:1 );
 			}
 
-			write_required	= true;
+			if ( changed ){
+			
+				setDirty( false );
+			}
 
 		}finally{
 
@@ -989,7 +1003,7 @@ DownloadManagerStateImpl
 
 			// we need to ensure this is persisted now as it has implications regarding crash restarts etc
 
-		save();
+		saveSupport(false,false);
 	}
 
 	@Override
@@ -1045,7 +1059,7 @@ DownloadManagerStateImpl
 		try{
 			this_mon.enter();
 
-			save( true );
+			saveSupport( false, true );
 
 			byte[]	hash = torrent.getHash();
 
@@ -1090,15 +1104,35 @@ DownloadManagerStateImpl
 			supressWrites--;
 	}
 
+
+	private void
+	setDirty(
+		boolean		slightly )
+	{
+		//Debug.out( (slightly?"slightly":"dirty" )+ ": " + new String(torrent.getName()));
+		
+		if ( slightly ){
+			
+			write_required_sometime = true;
+			
+		}else{
+		
+			write_required_soon = true;
+		}
+	}
+	
 	@Override
 	public void
-	save()
+	save( 
+		boolean interim )
 	{
-		save( false );
+		saveSupport( interim, false );
 	}
 
 	protected void
-	save( boolean force )
+	saveSupport( 
+		boolean interim, 
+		boolean force )
 	{
 		if ( supressWrites > 0 && !force ){
 
@@ -1110,10 +1144,24 @@ DownloadManagerStateImpl
 		try{
 			this_mon.enter();
 
-			do_write = write_required;
-
-			write_required = false;
-
+			if ( write_required_soon ){
+				
+				do_write = true;
+				
+			}else if ( write_required_sometime ){
+								
+				do_write = !interim;
+				
+			}else{
+				
+				do_write = false;
+			}
+			
+			if ( do_write ){
+			
+				write_required_soon 	= false;
+				write_required_sometime	= false;
+			}
 		}finally{
 
 			this_mon.exit();
@@ -1121,7 +1169,8 @@ DownloadManagerStateImpl
 
 		if ( do_write ){
 
-			try {
+			try{
+				
 				// System.out.println( "writing download state for '" + new String(torrent.getName()));
 
 				if (Logger.isEnabled())
@@ -1132,10 +1181,11 @@ DownloadManagerStateImpl
 
 				TorrentUtils.writeToFile(torrent, true);
 
-			} catch (Throwable e) {
+			}catch ( Throwable e ){
+				
 				Logger.log(new LogEvent(torrent, LOGID, "Saving state", e));
 			}
-		} else {
+		}else{
 
 			// System.out.println( "not writing download state for '" + new String(torrent.getName()));
 		}
@@ -1197,7 +1247,7 @@ DownloadManagerStateImpl
 
 			if ( write ){
 
-				save();
+				saveSupport(false,false);
 
 				if ( download_manager != null ){
 
@@ -2337,7 +2387,9 @@ DownloadManagerStateImpl
 
 					attributes.remove( attribute_name );
 
-					write_required = changed = true;
+					changed = true;
+					
+					setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
 				}
 			}else{
 
@@ -2346,7 +2398,8 @@ DownloadManagerStateImpl
 
 				if (existing_bytes == null || !Arrays.equals(existing_bytes, new_bytes)) {
 					attributes.put(attribute_name, new_bytes);
-					write_required = changed = true;
+					changed = true;
+					setDirty( attribute_name == DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE );
 				}
 			}
 		}finally{
@@ -2398,7 +2451,7 @@ DownloadManagerStateImpl
 
 					attributes.put( attribute_name, new Long( res ));
 
-					write_required	= true;
+					setDirty( false );
 
 					return( res );
 				}
@@ -2432,7 +2485,9 @@ DownloadManagerStateImpl
 
 				attributes.put( attribute_name, new Long( attribute_value) );
 
-				write_required = changed = true;
+				changed = true;
+								
+				setDirty( attribute_name == DownloadManagerState.AT_SCRAPE_CACHE );
 			}
 		}finally{
 
@@ -2588,7 +2643,9 @@ DownloadManagerStateImpl
 
 					attributes.remove( attribute_name );
 
-					write_required = changed = true;
+					changed = true;
+					
+					setDirty( false );
 				}
 			}else{
 
@@ -2598,7 +2655,9 @@ DownloadManagerStateImpl
 
 					attributes.put( attribute_name, attribute_value );
 
-					write_required = changed = true;
+					changed = true;
+					
+					setDirty( false );
 
 				}else{
 
@@ -2611,7 +2670,7 @@ DownloadManagerStateImpl
 
 					if ( changed ){
 
-						write_required = true;
+						setDirty( false );
 
 						attributes.put( attribute_name, attribute_value );
 					}
@@ -2674,7 +2733,9 @@ DownloadManagerStateImpl
 
 					attributes.remove( attribute_name );
 
-					write_required = changed = true;
+					changed = true;
+					
+					setDirty( false );
 				}
 			}else{
 
@@ -2684,7 +2745,9 @@ DownloadManagerStateImpl
 
 					attributes.put( attribute_name, attribute_value );
 
-					write_required = changed = true;
+					changed = true;
+					
+					setDirty( false );
 
 				}else{
 
@@ -2697,7 +2760,7 @@ DownloadManagerStateImpl
 
 					if ( changed ){
 
-						write_required = true;
+						setDirty( false );
 
 						attributes.put( attribute_name, attribute_value );
 					}
@@ -3353,7 +3416,7 @@ DownloadManagerStateImpl
 
 		@Override
 		public void
-		save()
+		save( boolean interim )
 		{
 		}
 
@@ -3445,11 +3508,14 @@ DownloadManagerStateImpl
 
 		Boolean		simple_torrent;
 		long		size;
+		Boolean		is_private;
 		int			file_count;
 
 		URL								announce_url;
 		cacheGroup						announce_group;
 
+		List<TOTorrentListener>			pending_listeners;
+		
 		volatile boolean		discard_fluff;
 
 		protected
@@ -3504,6 +3570,13 @@ DownloadManagerStateImpl
 				size = l_size.longValue();
 			}
 
+			Long	l_priv = (Long)cache.get( "priv" );
+
+			if ( l_priv != null ){
+
+				is_private = l_priv.longValue() != 0;
+			}
+			
 			byte[]	au = (byte[])cache.get( "au" );
 
 			if ( au != null ){
@@ -3535,7 +3608,7 @@ DownloadManagerStateImpl
 
 			throws TOTorrentException
 		{
-			Map	cache = new HashMap();
+			Map<String,Object>	cache = new HashMap<>();
 
 			TOTorrent	state = dms.getTorrent();
 
@@ -3545,7 +3618,8 @@ DownloadManagerStateImpl
 			cache.put( "comment", state.getComment());
 			cache.put( "createdby", state.getCreatedBy());
 			cache.put( "size", new Long( state.getSize()));
-
+			cache.put( "priv", new Long( state.getPrivate()?1:0));
+			
 			cache.put( "encoding", state.getAdditionalStringProperty( "encoding" ));
 			cache.put( "torrent filename", state.getAdditionalStringProperty( "torrent filename" ));
 
@@ -3659,6 +3733,8 @@ DownloadManagerStateImpl
 		{
 			private TOTorrentAnnounceURLSet[]		sets;
 
+			private volatile long	uid = TorrentUtils.getAnnounceGroupUID();
+			
 			protected
 			cacheGroup(
 				List	group )
@@ -3681,6 +3757,20 @@ DownloadManagerStateImpl
 					sets[i] = new cacheSet(urls);
 				}
 			}
+			
+			@Override
+			public long 
+			getUID()
+			{
+				TorrentUtils.ExtendedTorrent	del = delegate;
+				
+				if ( del != null ){
+					
+					return( del.getAnnounceURLGroup().getUID());
+				}
+				
+				return( uid );
+			}
 
 			@Override
 			public TOTorrentAnnounceURLSet[]
@@ -3796,6 +3886,8 @@ DownloadManagerStateImpl
 		    		}else{
 
 		    			urls = toSet;
+		    			
+		    			uid = TorrentUtils.getAnnounceGroupUID();
 		    		}
 		    	}
            	}
@@ -3871,6 +3963,13 @@ DownloadManagerStateImpl
 
 								announce_group = null;
 							}
+							
+							if ( pending_listeners != null ){
+								for ( TOTorrentListener l: pending_listeners ){
+									delegate.addListener( l );
+								}
+								pending_listeners = null;
+							}
 						}
 					}
 				}
@@ -4388,12 +4487,19 @@ DownloadManagerStateImpl
 	    public boolean
     	getPrivate()
        	{
-	   		if ( fixup()){
-
-				return( delegate.getPrivate());
-			}
-
-	   		return( false );
+    		if ( is_private == null ){
+    			
+		   		if ( fixup()){
+	
+					return( delegate.getPrivate());
+				}
+	
+		   		return( false );
+		   		
+    		}else{
+    			
+    			return( is_private );
+    		}
     	}
 
     	@Override
@@ -4402,12 +4508,14 @@ DownloadManagerStateImpl
     		boolean	_private )
 
     		throws TOTorrentException
-    	   	{
-    	   		if ( fixup()){
+	   	{
+    		is_private = null;
+    		
+	   		if ( fixup()){
 
-    				delegate.setPrivate( _private );
-    			}
-        	}
+				delegate.setPrivate( _private );
+			}
+    	}
 
     	@Override
 	    public void
@@ -4678,9 +4786,15 @@ DownloadManagerStateImpl
        addListener(
     	  TOTorrentListener		l )
        {
-    	   if ( fixup()){
-
-    		   delegate.addListener( l );
+    	   synchronized( this ){
+    		   if ( delegate != null ){
+    			   delegate.addListener( l );
+    		   }else{
+    			   if ( pending_listeners == null ){
+    				   pending_listeners = new ArrayList<>(2);
+    			   }
+    			   pending_listeners.add( l );
+    		   }
     	   }
        }
 
@@ -4689,9 +4803,14 @@ DownloadManagerStateImpl
        removeListener(
     	  TOTorrentListener		l )
        {
-    	   if ( fixup()){
-
-    		   delegate.removeListener( l );
+    	   synchronized( this ){
+    		   if ( delegate != null ){
+    			   delegate.removeListener( l );
+    		   }else{
+    			   if ( pending_listeners != null ){
+    				   pending_listeners.remove( l );
+    			   }
+    		   }
     	   }
        }
 
