@@ -61,6 +61,7 @@ import com.biglybt.pifimpl.local.PluginInitializer;
 
 public class
 GlobalManagerFileMerger
+	implements AEDiagnosticsEvidenceGenerator
 {
 	private static final int HASH_FAILS_BEFORE_QUIT	= 3;
 
@@ -88,8 +89,9 @@ GlobalManagerFileMerger
 
 	final Map<HashWrapper,DownloadManager>		dm_map = new HashMap<>();
 
-	final List<SameSizeFiles>				sames = new ArrayList<>();
-
+	final List<SameSizeFiles>				sames 		= new ArrayList<>();
+	final Set<DownloadManager>				sames_dms	= new IdentityHashSet<>();
+	
 	final AsyncDispatcher		read_write_dispatcher = new AsyncDispatcher( "GMFM" );
 
 	private TimerEventPeriodic	timer_event;
@@ -115,6 +117,8 @@ GlobalManagerFileMerger
 	void
 	initialise()
 	{
+		AEDiagnostics.addEvidenceGenerator( this );
+		
 		String VIEW_NAME =  "TableColumn.header.mergeddata";
 		
 		PluginInterface plugin_interface = CoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface();
@@ -175,6 +179,8 @@ GlobalManagerFileMerger
 							
 							sames.clear();
 							
+							sames_dms.clear();
+							
 							syncFileSets( true );
 						}
 					}
@@ -266,17 +272,31 @@ GlobalManagerFileMerger
 		
 		if ( !b ){
 			
-			logCurrentState();
+			logCurrentState( null );
 		}
 	}
 	
 	void 
 	log(
-		String	str )
+		String			str )
 	{
-		if ( !logging_paused ){
+		log( null, str );
+	}
+	
+	void 
+	log(
+		IndentWriter	writer,
+		String			str )
+	{
+		if ( writer == null ){
 			
-			logSupport( str );
+			if ( !logging_paused ){
+				
+				logSupport( str );
+			}
+		}else{
+			
+			writer.println( str );
 		}
 	}
 	
@@ -287,8 +307,18 @@ GlobalManagerFileMerger
 		log.log( str );
 	}
 	
+	protected boolean
+	isSwarmMergingZ(
+		DownloadManager		dm )
+	{
+		synchronized( dm_map ){
+
+			return( sames_dms.contains( dm ));
+		}
+	}
+
 	protected String
-	isSwarmMerging(
+	getSwarmMergingInfo(
 		DownloadManager		dm )
 	{
 		synchronized( dm_map ){
@@ -322,7 +352,7 @@ GlobalManagerFileMerger
 
 		return( null );
 	}
-
+	
 	void
 	syncFileSets(
 		boolean		force )
@@ -337,6 +367,8 @@ GlobalManagerFileMerger
 
 			if ( enabled ){
 
+				log( "Scanning files" );
+				
 				for ( DownloadManager dm: dms ){
 
 					/* 
@@ -654,8 +686,20 @@ GlobalManagerFileMerger
 					sames.remove( dead );
 				}
 
+				if ( enabled ){
+					
+					log( "Scan result: dm_map=" + dm_map.size() + ", sames=" + sames.size());
+				}
+				
+				sames_dms.clear();
+				
 				if ( sames.size() > 0 ){
 
+					for ( SameSizeFiles same: sames ){
+						
+						sames_dms.addAll( same.getDownloadManagers());
+					}
+					
 					if ( timer_event == null ){
 
 						timer_event =
@@ -684,7 +728,7 @@ GlobalManagerFileMerger
 								});
 					}
 				}else{
-
+					
 					if ( timer_event != null ){
 
 						timer_event.cancel();
@@ -948,7 +992,7 @@ GlobalManagerFileMerger
 						}
 					};
 
-				dm.setUserData( this, dmpl );
+				dm.setUserData( SameSizeFiles.this, dmpl );
 
 				dm.addPeerListener( dmpl );
 			}
@@ -970,6 +1014,12 @@ GlobalManagerFileMerger
 			return( dm_set.contains( dm ));
 		}
 
+		Set<DownloadManager>
+		getDownloadManagers()
+		{
+			return( dm_set );
+		}
+		
 		void
 		sync(
 			int		tick_count )
@@ -1030,7 +1080,7 @@ GlobalManagerFileMerger
 
 					if ( do_sync ){
 
-						DownloadManagerPeerListenerEx dmpl = (DownloadManagerPeerListenerEx)dm.getUserData( this );
+						DownloadManagerPeerListenerEx dmpl = (DownloadManagerPeerListenerEx)dm.getUserData( SameSizeFiles.this );
 
 						if ( dmpl != null ){
 
@@ -1069,7 +1119,7 @@ GlobalManagerFileMerger
 								DiskManager 	dm = ss_file.getDiskManager();
 								PEPeerManager 	pm = ss_file.getPeerManager();
 
-								if ( dm == null ){
+								if ( dm == null || pm == null ){
 
 									continue;
 								}
@@ -1283,10 +1333,12 @@ GlobalManagerFileMerger
 
 				DownloadManager	dm = file.getDownloadManager();
 
-				DownloadManagerPeerListenerEx dmpl = (DownloadManagerPeerListenerEx)dm.getUserData( this );
+				DownloadManagerPeerListenerEx dmpl = (DownloadManagerPeerListenerEx)dm.getUserData( SameSizeFiles.this );
 
 				if ( dmpl != null ){
 
+					dm.setUserData( SameSizeFiles.this, null );
+					
 					dm.removePeerListener( dmpl );
 				}
 			}
@@ -1294,6 +1346,13 @@ GlobalManagerFileMerger
 
 		private void
 		logCurrentState()
+		{
+			logCurrentState( null );
+		}
+		
+		private void
+		logCurrentState(
+			IndentWriter	writer )
 		{
 			long	size_min = -1;
 			long	size_max = -1;
@@ -1324,25 +1383,24 @@ GlobalManagerFileMerger
 				size_str += " (+" + (size_max-size_min ) + ")";
 			}
 			
-			log( "  " + size_str + ", files=" + file_wrappers.size());
+			log( writer, "  " + size_str + ", files=" + file_wrappers.size());
 			
 			if ( completion_logged ){
 				
-				log( "    Completed" );
+				log( writer, "    Completed" );
 				
 			}else if ( abandon_reason != null ){
 				
-				log( "    " + abandon_reason );
+				log( writer, "    Abandoned: " + abandon_reason );
 				
 			}else{
 				
 				for ( SameSizeFileWrapper file: file_wrappers ){
 					
-					file.logCurrentState();
+					file.logCurrentState( writer );
 				}
 			}
 		}
-		
 		private String
 		getString()
 		{
@@ -1448,10 +1506,10 @@ GlobalManagerFileMerger
 				int	end_rem = (int)( file_end_offset%piece_length);
 
 				if ( end_rem == 0 ){
-					end_rem = (int)(piece_length-1);	// exact piece so last block is last block
+					end_rem = (int)(piece_length);	// exact piece so last block is last block
 				}
 				
-				last_piece_block_number	= end_rem/DiskManager.BLOCK_SIZE;
+				last_piece_block_number	= (end_rem-1)/DiskManager.BLOCK_SIZE;
 			}
 
 			DiskManagerFileInfo
@@ -1823,6 +1881,15 @@ GlobalManagerFileMerger
 										
 										consecutive_skips = 0;
 										
+										if ( write_block != null ){
+											
+											Debug.out( "eh?" );
+											
+											write_block.returnToPool();
+											
+											write_block = null;
+										}
+										
 										write_block = DirectByteBufferPool.getBuffer( DirectByteBuffer.AL_EXTERNAL, write_piece.getBlockSize( write_block_num ) );
 	
 										int block_offset = write_block_offset;
@@ -2297,27 +2364,47 @@ GlobalManagerFileMerger
 			}
 			
 			private void
-			logCurrentState()
+			logCurrentState(
+				IndentWriter	writer )
 			{
-				log( "      " + getName() + ": " + getInfo());
+				log( writer, "      " + (writer==null?getName():Debug.secretFileName(getName())) + ": " + getInfo());
 			}
 		}
 	}
 
 	private void
-	logCurrentState()
+	logCurrentState(
+		IndentWriter	write )
 	{
 		synchronized( dm_map ){
 
-			log( "Same size file sets=" + sames.size());
+			log( write, "Same size file sets=" + sames.size() + ", dm_map=" + dm_map.size());
 			
 			for ( SameSizeFiles same: sames ){
 				
-				same.logCurrentState();
+				same.logCurrentState( write );
 			}
 		}
 	}
 	
+	@Override
+	public void
+	generate(
+		IndentWriter writer )
+	{
+		writer.println( "Swarm Merge" );
+
+		try{
+			writer.indent();
+
+			logCurrentState( writer );
+			
+		}finally{
+
+			writer.exdent();
+		}
+	}
+	
 	private interface
 	DownloadManagerPeerListenerEx
 		extends DownloadManagerPeerListener
