@@ -30,6 +30,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import com.biglybt.core.*;
 import com.biglybt.core.backup.BackupManagerFactory;
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ConfigKeys;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.config.impl.TransferSpeedValidator;
 import com.biglybt.core.custom.CustomizationManagerFactory;
@@ -43,6 +44,7 @@ import com.biglybt.core.global.GlobalManager;
 import com.biglybt.core.global.GlobalManagerAdapter;
 import com.biglybt.core.global.GlobalManagerFactory;
 import com.biglybt.core.global.GlobalManagerStats;
+import com.biglybt.core.global.GlobalMangerProgressListener;
 import com.biglybt.core.instancemanager.ClientInstanceManager;
 import com.biglybt.core.instancemanager.ClientInstanceManagerAdapter;
 import com.biglybt.core.instancemanager.ClientInstanceManagerFactory;
@@ -81,6 +83,8 @@ import com.biglybt.core.tag.Tag;
 import com.biglybt.core.tag.TagManagerFactory;
 import com.biglybt.core.tag.TagType;
 import com.biglybt.core.torrent.TOTorrent;
+import com.biglybt.core.tracker.AllTrackersManager;
+import com.biglybt.core.tracker.AllTrackersManager.AllTrackers;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncer;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncerResponse;
 import com.biglybt.core.tracker.host.TRHost;
@@ -1539,10 +1543,19 @@ CoreImpl
 			}
 		}
 	}
-
+	
 	@Override
 	public void
 	stop()
+		throws CoreException
+	{
+		stop( new CoreOperationTask.ProgressCallbackAdapter());
+	}
+
+	@Override
+	public void
+	stop(
+		CoreOperationTask.ProgressCallback	callback )
 
 		throws CoreException
 	{
@@ -1552,15 +1565,16 @@ CoreImpl
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(LOGID, "Stop operation starts"));
 
-				stopSupport( false, true );
+				stopSupport( false, true, callback );
 			}
 		});
 	}
 
 	void
 	stopSupport(
-		final boolean		for_restart,
-		final boolean		apply_updates )
+		final boolean							for_restart,
+		final boolean							apply_updates,
+		CoreOperationTask.ProgressCallback		callback )
 
 		throws CoreException
 	{
@@ -1716,6 +1730,11 @@ CoreImpl
 			}
 		}
 
+		int	progress = 125;
+				
+		callback.setSubTaskName( MessageText.getString( "label.starting.closedown" ));
+		callback.setProgress( progress );
+		
 		try{
 			if (Logger.isEnabled())
 				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopping' listeners"));
@@ -1757,25 +1776,133 @@ CoreImpl
 					},
 					10*1000 );
 
-			if (Logger.isEnabled())
+			if (Logger.isEnabled()){
 				Logger.log(new LogEvent(LOGID, "Waiting for quiescence pre gm stop"));
-
+			}
+			
 			NonDaemonTaskRunner.waitUntilIdle();
 
 			last_progress.set( SystemTime.getMonotonousTime());
 			
-			if (Logger.isEnabled())
+			if (Logger.isEnabled()){
 				Logger.log(new LogEvent(LOGID, "Stopping global manager"));
-
-			if (global_manager != null) {
-				global_manager.stopGlobalManager();
+			}
+			
+			progress = 250;
+						
+			callback.setSubTaskName( MessageText.getString( "label.stopping.downloads" ));
+			callback.setProgress( progress );
+			
+			if ( global_manager != null ){
+				
+				int p_start = 250;
+				int p_end	= 699;
+				
+				global_manager.stopGlobalManager( 
+					new GlobalMangerProgressListener(){
+						
+						@Override
+						public void 
+						reportPercent(
+							int percent)
+						{
+							callback.setProgress( p_start + ((p_end-p_start)*percent)/100 );
+						}
+						
+						@Override
+						public void 
+						reportCurrentTask(
+							String currentTask )
+						{
+							callback.setSubTaskName( currentTask );
+						}
+					});
 			}
 
 			last_progress.set( SystemTime.getMonotonousTime());
 			
-			if (Logger.isEnabled())
-				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopped' listeners"));
+			AllTrackers at = AllTrackersManager.getAllTrackers();
+			
+			progress = 750;
+				
+			int wait_secs = COConfigurationManager.getIntParameter( ConfigKeys.Tracker.ICFG_TRACKER_CLIENT_CLOSEDOWN_TIMEOUT );
 
+			callback.setSubTaskName( MessageText.getString( "label.waiting.tracker.updates", new String[]{ String.valueOf( wait_secs )}));
+			callback.setProgress( progress );
+
+			int active_req = at.getActiveRequestCount();
+			
+			AllTrackersManager.AnnounceStats announce_stats = at.getAnnounceStats();
+
+			int scheduled_req = announce_stats.getPrivateScheduledCount() + announce_stats.getPublicScheduledCount();
+			
+			int total_req = active_req + scheduled_req;
+			
+			if ( total_req > 0 ){				
+				
+				if (Logger.isEnabled()){
+					Logger.log(new LogEvent(LOGID, "Waiting for tracker updates, " + active_req + "/" + scheduled_req + " outstanding"));
+				}
+				
+				long at_start = SystemTime.getMonotonousTime();
+								
+				int current_req = total_req;
+				
+				int p_start = 750;
+				int p_end	= 899;
+				
+				if ( wait_secs > 0 ){
+					
+					while( true ){
+						
+						if ( SystemTime.getMonotonousTime() - at_start > wait_secs*1000 ){
+							
+							break;
+						}
+						
+						try{
+							Thread.sleep(500);
+							
+						}catch( Throwable e ){
+							
+						}
+						
+						int latest_active = at.getActiveRequestCount();
+						
+						announce_stats = at.getAnnounceStats();
+
+						int latest_scheduled = announce_stats.getPrivateScheduledCount() + announce_stats.getPublicScheduledCount();
+
+						int latest_req = latest_active + latest_scheduled;
+						
+						if ( latest_req == 0 ){
+							
+							break;
+						}
+												
+						if ( latest_req < current_req ){
+							
+							current_req = latest_req;
+							
+							int percent = ((total_req-current_req)*100)/total_req;
+							
+							callback.setProgress( p_start + ((p_end-p_start)*percent)/100 );
+							
+							last_progress.set( SystemTime.getMonotonousTime());
+						}
+					}
+				}
+			}
+			
+			if (Logger.isEnabled()){
+				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopped' listeners"));
+			}
+			
+			progress = 900;
+			
+			callback.setSubTaskName( MessageText.getString( "label.finalising.closedown" ));
+			callback.setProgress( progress );
+			
 			for (int i=0;i<sync_listeners.size();i++){
 				try{
 					((CoreLifecycleListener)sync_listeners.get(i)).stopped( this );
@@ -1788,9 +1915,10 @@ CoreImpl
 				last_progress.set( SystemTime.getMonotonousTime());
 			}
 
-			if (Logger.isEnabled())
+			if (Logger.isEnabled()){
 				Logger.log(new LogEvent(LOGID, "Invoking asynchronous 'stopped' listeners"));
-
+			}
+			
 			ListenerManager.dispatchWithTimeout(
 					async_listeners,
 					new ListenerManagerDispatcher()
@@ -1990,19 +2118,24 @@ CoreImpl
 
 		throws CoreException
 	{
-		if (stopped)
+		if ( stopped ){
+			
 			return;
-
+		}
+		
 		runNonDaemon(new AERunnable() {
 			@Override
 			public void runSupport() {
 
-				Iterator it = lifecycle_listeners.iterator();
+				boolean	in_progress = false;
 
-				while( it.hasNext()){
+				for ( CoreLifecycleListener l: lifecycle_listeners ){
+					
+					if ( l.stopRequested(CoreImpl.this)){
 
-					if (!((CoreLifecycleListener)it.next())
-							.stopRequested(CoreImpl.this)) {
+						in_progress = true;
+						
+					}else{
 						if (Logger.isEnabled())
 							Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
 									"Request to stop the core has been denied"));
@@ -2011,7 +2144,10 @@ CoreImpl
 					}
 				}
 
-				stop();
+				if ( !in_progress ){
+				
+					stop();
+				}
 			}
 		});
 	}
@@ -2019,6 +2155,16 @@ CoreImpl
 	@Override
 	public void
 	restart()
+	
+		throws CoreException
+	{
+		restart( new CoreOperationTask.ProgressCallbackAdapter());
+	}
+	
+	@Override
+	public void
+	restart(
+		CoreOperationTask.ProgressCallback		callback )
 
 		throws CoreException
 	{
@@ -2032,7 +2178,7 @@ CoreImpl
 
 				restarting = true;
 
-				stopSupport( true, false);
+				stopSupport( true, false, callback );
 
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(LOGID, "Restart operation: stop complete,"
@@ -2049,18 +2195,26 @@ CoreImpl
 
 		throws CoreException
 	{
+		if ( stopped ){
+			
+			return;
+		}
+
 		runNonDaemon(new AERunnable() {
             @Override
-            public void runSupport() {
+            public void runSupport(){
                 checkRestartSupported();
 
-                Iterator it = lifecycle_listeners.iterator();
-
-                while( it.hasNext()){
-                    CoreLifecycleListener l = (CoreLifecycleListener)it.next();
-
-                    if (!l.restartRequested(CoreImpl.this)) {
+                boolean	in_progress = false;
+                
+                for ( CoreLifecycleListener l: lifecycle_listeners ){
+                	
+                    if ( l.restartRequested(CoreImpl.this)){
 
+                    	in_progress = true;
+                    	
+                    }else{
+                    	
                         if (Logger.isEnabled())
                             Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
                                     "Request to restart the core"
@@ -2070,7 +2224,10 @@ CoreImpl
                     }
                 }
 
-                restart();
+                if (!in_progress ){
+                
+                	restart();
+                }
             }
         });
 	}
