@@ -48,11 +48,13 @@ public class MessageText {
 
 	public static final Locale LOCALE_DEFAULT = Locale.ROOT; // == english
 
+	private static final boolean LOG_MISSING_MESSAGES = System.getProperty("log.missing.messages", "0").equals("1");
+
 	private static Locale LOCALE_CURRENT = LOCALE_DEFAULT;
 
 	static final String BUNDLE_NAME;
 
-	private static final Map<String, Object> CONSTANTS = new HashMap<>();
+	public static final Map<String, String> CONSTANTS = new HashMap<>();
 
 	public static final String DEFAULT_BUNDLE_NAME = "com.biglybt.internat.MessagesBundle";
 
@@ -93,7 +95,12 @@ public class MessageText {
 			PLATFORM_SUFFIX = "._unknown";
 		}
 
-	  setResourceBundle( new IntegratedResourceBundle( getResourceBundle( BUNDLE_NAME, LOCALE_DEFAULT, MessageText.class.getClassLoader()), pluginLocalizationPaths, null, 4000, true ));
+		if (System.getProperty("SKIP_SETRB", "0").equals("0")) {
+			setResourceBundle(new IntegratedResourceBundle(
+					getResourceBundle(BUNDLE_NAME, LOCALE_DEFAULT,
+							MessageText.class.getClassLoader()),
+					pluginLocalizationPaths, null, 4000, true));
+		}
   }
 
   	// grab a reference to the default bundle
@@ -358,6 +365,9 @@ public class MessageText {
       if ( key.startsWith("!") && key.endsWith( "!" )){
     	  return( expandValue(key.substring(1,key.length()-1 )));
       }
+			if (LOG_MISSING_MESSAGES) {
+				System.err.println("Missing message key '" + key + "' via " + Debug.getCompressedStackTraceSkipFrames(2));
+			}
       return '!' + key + '!';
     }
   }
@@ -429,7 +439,18 @@ public class MessageText {
   		String		key,
 		String[] params )
   {
-  	String	res = getString(key);
+  	String	res = getString(key, (String) null);
+
+		if (res == null) {
+			if (LOG_MISSING_MESSAGES) {
+				System.err.println("Missing message key '" + key + "', params " + Arrays.toString(params));
+			}
+
+			if (params == null) {
+				return  "!" + key + "!";
+			}
+			return "!" + key + "(" + Arrays.toString(params) + ")" + "!";
+		}
 
   	if (params == null) {
   		return res;
@@ -440,10 +461,12 @@ public class MessageText {
   		String	from_str 	= "%" + (i+1);
   		String	to_str		= params[i];
 
+  		to_str = to_str.replace( '%', '\uFDE5' );	// invalid char to avoid %n in to_str being expanded
+  		
   		res = replaceStrings( res, from_str, to_str );
   	}
 
-  	return( res );
+  	return( res.replace( '\uFDE5', '%' ));
   }
 
   protected static String
