@@ -34,6 +34,7 @@ import com.biglybt.core.diskmanager.cache.CacheFileOwner;
 import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.internat.LocaleTorrentUtil;
 import com.biglybt.core.internat.LocaleUtilDecoder;
+import com.biglybt.core.peermanager.piecepicker.util.BitFlags;
 import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.util.ByteFormatter;
@@ -48,6 +49,7 @@ LWSDiskManager
 {
 	private static final sePiece	piece = new sePiece();
 
+	
 	private final LightWeightSeed			lws;
 	private final DiskAccessController	disk_access_controller;
 	private final File					save_file;
@@ -57,6 +59,8 @@ LWSDiskManager
 	private DMPieceMap				piece_map_use_accessor;
 
 	private final sePiece[]					pieces;
+	private final BitFlags					availability;
+
 	private DiskManagerFileInfoImpl[]	files;
 	private String						internal_name;
 	private final DownloadManagerState		download_state;
@@ -86,6 +90,10 @@ LWSDiskManager
 
 			pieces[i] = piece;
 		}
+		
+		availability = new BitFlags( pieces.length );
+		
+		availability.setAll();
 	}
 
 	public String
@@ -143,7 +151,7 @@ LWSDiskManager
 			}
 		}catch( Throwable e ){
 
-			setFailed( "Start failed", e );
+			setFailed( DiskManager.ET_OTHER, "Start failed", e );
 		}
 	}
 
@@ -207,7 +215,7 @@ LWSDiskManager
 
 		}catch( Throwable e ){
 
-			setFailed( "getFiles failed", e );
+			setFailed( DiskManager.ET_READ_ERROR, "getFiles failed", e );
 
 			return( null );
 
@@ -405,6 +413,12 @@ LWSDiskManager
 		return( pieces );
 	}
 
+	@Override
+	public BitFlags getAvailability()
+	{
+		return( availability );
+	}
+	
 	@Override
 	public DiskManagerPiece
 	getPiece(
@@ -647,7 +661,7 @@ LWSDiskManager
 
 	@Override
 	public void
-	saveState()
+	saveState( boolean interim )
 	{
 	}
 
@@ -722,6 +736,7 @@ LWSDiskManager
 	@Override
 	public void
 	setFailed(
+		int				type,
 		String			reason,
 		Throwable		cause )
 	{
@@ -730,7 +745,7 @@ LWSDiskManager
 		state	= FAULTY;
 
 		error_message	= reason;
-		error_type		= ET_OTHER;
+		error_type		= type;
 	}
 
 	@Override
