@@ -20,10 +20,7 @@
 
 package com.biglybt.core.pairing.impl;
 
-import java.io.BufferedInputStream;
-import java.io.File;
-import java.io.IOException;
-import java.io.InputStream;
+import java.io.*;
 import java.net.*;
 import java.text.SimpleDateFormat;
 import java.util.*;
@@ -42,24 +39,23 @@ import com.biglybt.core.security.CryptoManagerFactory;
 import com.biglybt.core.util.*;
 import com.biglybt.core.versioncheck.VersionCheckClient;
 import com.biglybt.core.versioncheck.VersionCheckClientListener;
-import com.biglybt.net.upnp.UPnPRootDevice;
+import com.biglybt.pifimpl.local.PluginInitializer;
+import com.biglybt.pifimpl.local.clientid.ClientIDManagerImpl;
+import com.biglybt.pifimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
+import com.biglybt.util.JSONUtils;
+
 import com.biglybt.pif.PluginInterface;
 import com.biglybt.pif.clientid.ClientIDException;
 import com.biglybt.pif.clientid.ClientIDGenerator;
 import com.biglybt.pif.tracker.web.TrackerWebPageRequest;
 import com.biglybt.pif.tracker.web.TrackerWebPageResponse;
+import com.biglybt.pif.ui.UIInstance;
 import com.biglybt.pif.ui.UIManager;
 import com.biglybt.pif.ui.UIManagerEvent;
 import com.biglybt.pif.ui.config.*;
 import com.biglybt.pif.ui.model.BasicPluginConfigModel;
 import com.biglybt.pif.utils.DelayedTask;
 import com.biglybt.pif.utils.StaticUtilities;
-import com.biglybt.pifimpl.local.PluginInitializer;
-import com.biglybt.pifimpl.local.clientid.ClientIDManagerImpl;
-import com.biglybt.pifimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
-import com.biglybt.plugin.upnp.UPnPPlugin;
-import com.biglybt.plugin.upnp.UPnPPluginService;
-import com.biglybt.util.JSONUtils;
 
 public class
 PairingManagerImpl
@@ -155,6 +151,9 @@ PairingManagerImpl
 	private String			local_v4	= "";
 	private String			local_v6	= "";
 
+	private Set<InetAddress>	ignored_v4 = new HashSet<>();
+	private Set<InetAddress>	ignored_v6 = new HashSet<>();
+	
 	private PairingManagerTunnelHandler	tunnel_handler;
 
 	private boolean	update_outstanding;
@@ -436,20 +435,14 @@ PairingManagerImpl
 		param_e_enable.addEnabledOnSelection( param_local_ipv6 );
 		param_e_enable.addEnabledOnSelection( param_host );
 
-		configModel.createGroup(
-			"pairing.group.explicit",
-			new Parameter[]{
-				param_e_info,
-				param_e_enable,
-				param_public_ipv4,
-				param_public_ipv6,
-				param_host,
-				spacer,
-				param_local_ipv4,
-				param_local_ipv6,
-			});
+		ParameterGroup groupExplicit = configModel.createGroup(
+				"pairing.group.explicit", param_e_info, param_e_enable,
+				param_public_ipv4, param_public_ipv6, param_host, spacer,
+				param_local_ipv4, param_local_ipv6);
+		groupExplicit.setMinimumRequiredUserMode(1);
 
 		param_icon_enable = configModel.addBooleanParameter2( "pairing.config.icon.show", "pairing.config.icon.show", true );
+		param_icon_enable.setAllowedUiTypes(UIInstance.UIT_SWT);
 
 		CoreFactory.addCoreRunningListener(
 			new CoreRunningListener()
@@ -795,7 +788,8 @@ PairingManagerImpl
 
 	protected void
 	setLastServerError(
-		String	error )
+		String					error,
+		Map<String, Object>		payload )
 	{
 		String last_error = param_last_error.getValue();
 
@@ -804,6 +798,14 @@ PairingManagerImpl
 			error = "";
 		}
 
+		try{
+			COConfigurationManager.setParameter( "Plugin.default.pairing.last.error.payload", error.isEmpty()?"":Base32.encode(BEncoder.encode( payload )));
+			
+		}catch( Throwable e ){
+			
+			Debug.out( e );
+		}
+		
 		if ( !last_error.equals( error )){
 
 			param_last_error.setValue( error );
@@ -1187,12 +1189,21 @@ PairingManagerImpl
 				existing_checked = new HashSet<>(local_address_checks.keySet());
 			}
 
+				// Some users have huge numbers of these that breaks the server table limit (seen a linux user with 50 of them)
+				// To retain determinism in the computation of address equivalence over updates we maintain a list of ignored addresses
+			
+			int	num_v4_accepted = 0;
+			int	num_v6_accepted = 0;
+			
 			for ( NetworkAdminNetworkInterface intf: interfaces ){
 
 				NetworkAdminNetworkInterfaceAddress[] addresses = intf.getAddresses();
 
+				int num_v4_from_this_intf = 0;
+				int num_v6_from_this_intf = 0;
+				
 				for ( NetworkAdminNetworkInterfaceAddress address: addresses ){
-
+					
 					final InetAddress ia = address.getAddress();
 
 					if ( ia.isLoopbackAddress()){
@@ -1202,8 +1213,32 @@ PairingManagerImpl
 
 					if ( ia.isLinkLocalAddress() || ia.isSiteLocalAddress()){
 
-						final String a_str = ia.getHostAddress();
+						boolean is_v4 = ia instanceof Inet4Address;
 
+						if ( is_v4 ){
+							if ( ignored_v4.contains( ia )){
+								continue;
+							}
+							if ( num_v4_from_this_intf == 3 || num_v4_accepted == 10 ){
+								ignored_v4.add( ia );
+								continue;
+							}
+							num_v4_from_this_intf++;
+							num_v4_accepted++;
+						}else{
+							if ( ignored_v6.contains( ia )){
+								continue;
+							}
+							if ( num_v6_from_this_intf == 3 || num_v6_accepted == 10 ){
+								ignored_v6.add( ia );
+								continue;
+							}
+							num_v6_from_this_intf++;
+							num_v6_accepted++;
+						}							
+						
+						final String a_str = ia.getHostAddress();
+						
 						existing_checked.remove( a_str );
 
 						Object[] check;
@@ -1249,7 +1284,7 @@ PairingManagerImpl
 
 											local_address_checks.put( a_str, new Object[]{ new Long(now), result });
 
-											if ( ia instanceof Inet4Address ){
+											if ( is_v4 ){
 
 												latest_v4_locals.add( result );
 
@@ -1263,7 +1298,7 @@ PairingManagerImpl
 
 						}else{
 
-							if ( ia instanceof Inet4Address ){
+							if ( is_v4 ){
 
 								latest_v4_locals.add((String)check[1]);
 
@@ -1595,7 +1630,7 @@ PairingManagerImpl
 				}
 
 					// grab some UPnP info for diagnostics
-
+				/* removed this to reduce message size - diags not used at the moment anyway
 				try{
 				    PluginInterface pi_upnp = core.getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
 
@@ -1639,7 +1674,8 @@ PairingManagerImpl
 				    }
 				}catch( Throwable e ){
 				}
-
+				*/
+				
 				try{
 					NetworkAdmin admin = NetworkAdmin.getSingleton();
 
@@ -1904,7 +1940,7 @@ PairingManagerImpl
 				throw( new PairingException( error ));
 			}
 
-			setLastServerError( null );
+			setLastServerError( null, payload );
 
 			Map<String,Object> reply = (Map<String,Object>)response.get( "rep" );
 
@@ -1923,7 +1959,7 @@ PairingManagerImpl
 
 		}catch( Throwable e ){
 
-			setLastServerError( Debug.getNestedExceptionMessage( e ));
+			setLastServerError( Debug.getNestedExceptionMessage( e ), payload );
 
 			if ( e instanceof PairingException ){
 
