@@ -1631,7 +1631,16 @@ TagPropertyConstraintHandler
 						
 					}else if ( value.startsWith( "{" )){
 						
-						return( new Object[]{ dereference( value )});
+						Object temp = dereference( value );
+						
+						if ( temp instanceof Object[]){
+							
+							return((Object[])temp);
+							
+						}else{
+							
+							return( new Object[]{ temp });
+						}
 						
 					}else if ( value.contains( "(" )){
 						
@@ -1704,30 +1713,23 @@ TagPropertyConstraintHandler
 			private Object
 			dereference(
 				String	key )
+		
 			{
-				Object obj = context.get( key );
+				ConstraintExpr expr = context.get( key );
 				
-				if ( obj == null ){
+				if ( expr == null ){
 					
 					throw( new RuntimeException( "Reference " + key + " not found" ));
 				}
 			
-				if ( obj instanceof ConstraintExprParams ){
+				if ( expr instanceof ConstraintExprParams ){
 					
-					ConstraintExprParams params = (ConstraintExprParams)obj;
-					
-					Object[] args = params.getValues();
-					
-					if ( args.length != 1 ){
-						
-						throw( new RuntimeException( "Reference " + key + " resolved incorrectly" ));
-					}
-					
-					return( args[0] );
+					ConstraintExprParams params = (ConstraintExprParams)expr;
 					
+					return( params.getValues());
 				}
 				
-				return( obj );
+				return( expr );
 			}
 			
 			@Override
@@ -2047,6 +2049,8 @@ TagPropertyConstraintHandler
 		private static final int FT_SET_COLOURS		= 30;		
 		private static final int FT_IS_NEW			= 31;		
 		private static final int FT_IS_SUPER_SEEDING	= 32;		
+		private static final int FT_IS_SEQUENTIAL		= 33;		
+		private static final int FT_TAG_POSITION		= 34;		
 		
 		private static final int	DEP_STATIC		= 0;
 		private static final int	DEP_RUNNING		= 1;
@@ -2201,7 +2205,7 @@ TagPropertyConstraintHandler
 				
 				boolean	params_ok = false;
 
-				if ( func_name.equals( "hasTag" ) || func_name.equals( "hasTagAge" ) || func_name.equals( "countTag" )){
+				if ( func_name.equals( "hasTag" ) || func_name.equals( "hasTagAge" ) || func_name.equals( "countTag" ) || func_name.equals( "tagPosition" )){
 
 					if ( func_name.equals( "hasTag" )){
 						
@@ -2211,13 +2215,31 @@ TagPropertyConstraintHandler
 						
 						fn_type = FT_HAS_TAG_AGE;
 						
-					}else{
+					}else if ( func_name.equals( "countTag" )){
 						
 						fn_type = FT_COUNT_TAG;
+						
+					}else{
+						
+						fn_type = FT_TAG_POSITION;
 					}
 					
-					params_ok = num_params == 1 && getStringLiteral( params, 0 );
+					params_ok = num_params >= 1 && getStringLiteral( params, 0 );
 
+					if ( params_ok ){
+						
+						if ( fn_type == FT_TAG_POSITION ){
+							
+							if ( num_params > 1 ){
+								
+								params_ok = num_params == 2 && getNumericLiteral( params, 1 );
+							}
+						}else{
+							
+							params_ok = num_params == 1;
+						}
+					}
+						
 					if ( params_ok ){
 						
 						String tag_name = (String)params[0];
@@ -2500,6 +2522,12 @@ TagPropertyConstraintHandler
 							}
 						}
 					}
+				}else if ( func_name.equals( "isSequential" )){
+
+					fn_type = FT_IS_SEQUENTIAL;
+
+					params_ok = num_params == 0;
+
 				}else{
 
 					throw( new RuntimeException( "Unsupported function '" + func_name + "'" ));
@@ -2650,6 +2678,139 @@ TagPropertyConstraintHandler
 							return( target.getTaggedCount());
 						}
 					}
+					case FT_TAG_POSITION:{
+						
+						String tag_name = (String)params[0];
+						
+						Tag target = null;
+						
+						if ( dependent_on_tags != null ){
+							
+							for ( Tag t: dependent_on_tags ){
+	
+								if ( t.getTagName( true ).equals( tag_name )){
+	
+									target = t;
+									
+									break;
+								}
+							}
+						}	
+						
+						if ( target == null ){
+							
+							tag.setTransientProperty( Tag.TP_CONSTRAINT_ERROR, "Tag '" + tag_name + "' not found" );
+							
+							return( -1 );
+							
+						}else{
+							
+							int sort;
+							
+							if ( params.length == 1 ){
+							
+								sort = 1;
+								
+							}else{
+								
+								sort = ((Number)params[1]).intValue();
+							}
+							
+							Set<Taggable> dms = target.getTagged();
+							
+							if ( !dms.contains( dm )){
+								
+								return( -1 );	// not present
+							}
+							
+							if ( sort == 1 ){
+								
+								int my_position = dm.getPosition();
+								
+								int tag_position = 0;
+								
+								boolean found = false;
+								
+								for ( Taggable t: dms ){
+									
+									DownloadManager this_dm = (DownloadManager)t;
+									
+									if ( t == dm ){
+										
+										found = true;
+										
+									}else{
+										
+										int this_pos = this_dm.getPosition();
+										
+										if ( this_pos < my_position ){
+											
+											tag_position++;
+										}
+									}
+								}
+								
+								if ( found ){
+									
+									return( tag_position );
+									
+								}else{
+									
+									return( -1 );
+								}
+							}else{
+								
+								long my_time = target.getTaggableAddedTime( dm );
+								
+								if ( my_time == -1 ){
+									
+									return( -1 );	// not in tag
+								}
+								
+									// need to mix in position as if a number of downloads are added to a tag at the same time (drag+drop a bunch)
+									// then they most likely end up with identical times
+								
+								my_time += dm.getPosition();
+								
+								int tag_position = 0;
+								
+								boolean found = false;
+								
+								for ( Taggable t: dms ){
+																		
+									if ( t == dm ){
+											
+										found = true;
+									
+									}else{
+										
+										long this_time = target.getTaggableAddedTime( t );
+										
+										if ( this_time != -1 ){
+											
+											DownloadManager this_dm = (DownloadManager)t;
+											
+											this_time += this_dm.getPosition();
+											
+											if ( this_time < my_time ){
+												
+												tag_position++;
+											}
+										}
+									}
+								}
+								
+								if ( found ){
+									
+									return( tag_position );
+									
+								}else{
+									
+									return( -1 );
+								}
+							}
+						}
+					}
 					case FT_HAS_NET:{
 
 						String net_name = (String)params[0];
@@ -3014,6 +3175,11 @@ TagPropertyConstraintHandler
 						
 						return( true );
 					}
+					case FT_IS_SEQUENTIAL:{
+
+						return( dm.getDownloadState().getFlag(DownloadManagerState.FLAG_SEQUENTIAL_DOWNLOAD ));
+					}
+
 				}
 
 				return( false );
