@@ -433,7 +433,7 @@ public class TorrentOpenOptions
 	public String getDataDir() {
 		if (torrent.isSimpleTorrent())
 			return sDestDir;
-		return new File(sDestDir, sDestSubDir == null
+		return FileUtil.newFile(sDestDir, sDestSubDir == null
 				? FileUtil.convertOSSpecificChars(getTorrentName(), true) : sDestSubDir).getPath();
 	}
 
@@ -686,7 +686,7 @@ public class TorrentOpenOptions
 
 			if (files[i] != null) {
 				files[i].orgFullName = torrentFile.getRelativePath(); // translated to locale
-				files[i].orgFileName = new File(files[i].orgFullName).getName();
+				files[i].setOriginalFileName( new File(files[i].orgFullName).getName());
 			}
 		}
 
@@ -774,7 +774,7 @@ public class TorrentOpenOptions
 				continue;
 			}
 			
-			String name = file.getDestFileName();
+			String name = file.getOriginalFileName();	// important to use original, especially for simple torrents
 			
 			int pos = name.lastIndexOf( '.' );
 			
@@ -1147,6 +1147,11 @@ public class TorrentOpenOptions
 					networks.add( network );
 				}
 			}
+			
+				// use actually trackers to check for I2P enable - cache might well have I2P set if it was potentially
+				// available for magnet download
+			
+			boolean	enable_i2p = networks.contains( AENetworkClassifier.AT_I2P );
 
 			List<String> network_cache = TorrentUtils.getNetworkCache( torrent );
 
@@ -1157,22 +1162,34 @@ public class TorrentOpenOptions
 					// the resulting torrent file happens to have public trackers - don't surprise the user
 					// by leaving 'Public' enabled
 
+					// actually we only really want to apply this logic to the 'public' case, no harm in leaving
+					// the others as they are
+				
+				String to_remove = null;
+				
 				for ( String net: enabledNetworks.keySet()){
 
 					boolean enabled = network_cache.contains( net );
 
 					if ( !enabled ){
 
-						enabledNetworks.put( net, false );
+						if ( net == AENetworkClassifier.AT_PUBLIC ){
+							
+							to_remove = net;
+						}
 					}
 				}
+				
+				if ( to_remove != null ){
+					
+					enabledNetworks.put( to_remove, false );
+				}
 			}
 
 			networks.addAll( network_cache );
 
 				// could do something here if multiple networks to get user to decide what to do...
-
-			boolean	enable_i2p = networks.contains( AENetworkClassifier.AT_I2P );
+			
 			String enable_i2p_reason = null;
 
 			if ( enable_i2p ){
@@ -1326,80 +1343,90 @@ public class TorrentOpenOptions
 				
 				if ( COConfigurationManager.getBooleanParameter( "Plugin.Magnet URI Handler.MagnetPlugin.rename.using.dn" )){
 					
-					boolean do_it = true;
-					
-					if ( torrent.isSimpleTorrent()){
-					
-						if ( COConfigurationManager.getBooleanParameter( "Plugin.Magnet URI Handler.MagnetPlugin.rename.using.dn.only.with.ext" )){
-							
-							do_it = false;
-							
-							int pos = display_name.lastIndexOf( "." );
-							
-							if ( pos != -1 ){
+					if ( display_name.startsWith( "urlencode(" )){
+						
+						// see quite a large number of downloads out there where the title has been set to "urlencode(<title>)" - must be a bug
+						// in some software somewhere
+						
+						// ignore it
+						
+					}else{
+						
+						boolean do_it = true;
+						
+						if ( torrent.isSimpleTorrent()){
+						
+							if ( COConfigurationManager.getBooleanParameter( "Plugin.Magnet URI Handler.MagnetPlugin.rename.using.dn.only.with.ext" )){
 								
-								String ext = display_name.substring( pos+1 ).trim();
+								do_it = false;
 								
-								if ( !ext.isEmpty()){
+								int pos = display_name.lastIndexOf( "." );
+								
+								if ( pos != -1 ){
 									
-									do_it = true;
+									String ext = display_name.substring( pos+1 ).trim();
+									
+									if ( !ext.isEmpty()){
+										
+										do_it = true;
+									}
 								}
 							}
+							
+							if ( do_it ){
+							
+								getFiles()[0].setDestFileName( FileUtil.convertOSSpecificChars( display_name, false ), true );
+							}
+						}else{
+							
+							setExplicitDataDir( new File( getDataDir()).getParentFile().getAbsolutePath(), FileUtil.convertOSSpecificChars( display_name, true ), false );
 						}
 						
 						if ( do_it ){
-						
-							getFiles()[0].setDestFileName( FileUtil.convertOSSpecificChars( display_name, false ), true );
-						}
-					}else{
-						
-						setExplicitDataDir( new File( getDataDir()).getParentFile().getAbsolutePath(), FileUtil.convertOSSpecificChars( display_name, true ), false );
-					}
-					
-					if ( do_it ){
-						
-						try{
-							File existing = new File( TorrentUtils.getTorrentFileName(torrent));
 							
-							File folder = existing.getParentFile();
-							
-							String old_name = existing.getName();
-							
-							String new_name = FileUtil.convertOSSpecificChars( display_name, false ) + ".torrent";
-							
-							if ( !new_name.equals( old_name )){
+							try{
+								File existing = new File( TorrentUtils.getTorrentFileName(torrent));
 								
-								String prefix = "";
+								File folder = existing.getParentFile();
 								
-								for ( int i=0;i<16;i++){
+								String old_name = existing.getName();
+								
+								String new_name = FileUtil.convertOSSpecificChars( display_name, false ) + ".torrent";
+								
+								if ( !new_name.equals( old_name )){
 									
-									File new_file = new File( folder, prefix + new_name );
+									String prefix = "";
 									
-									if ( !new_file.exists()){
+									for ( int i=0;i<16;i++){
 										
-										try{
+										File new_file = new File( folder, prefix + new_name );
 										
-											TorrentUtils.writeToFile( torrent, new_file, false );
+										if ( !new_file.exists()){
 											
-											sFileName = new_file.getAbsolutePath();
+											try{
 											
-											TorrentUtils.delete( existing, true );
-											
-											break;
-											
-										}catch( Throwable e ){
-											
-											Debug.out( e );
+												TorrentUtils.writeToFile( torrent, new_file, false );
+												
+												sFileName = new_file.getAbsolutePath();
+												
+												TorrentUtils.delete( existing, true );
+												
+												break;
+												
+											}catch( Throwable e ){
+												
+												Debug.out( e );
+											}
 										}
+										
+										prefix += "_";
 									}
-									
-									prefix += "_";
 								}
+								
+							}catch( Throwable e ){
+								
+								Debug.out( e );
 							}
-							
-						}catch( Throwable e ){
-							
-							Debug.out( e );
 						}
 					}
 				}
