@@ -24,6 +24,7 @@ package com.biglybt.core.torrentdownloader.impl;
 import java.io.*;
 import java.net.*;
 import java.util.*;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.zip.GZIPInputStream;
@@ -52,6 +53,8 @@ import com.biglybt.pifimpl.local.utils.xml.rss.RSSUtils;
  */
 public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader {
 
+  private static final AtomicLong uid = new AtomicLong( SystemTime.getCurrentTime());
+	
   private String	original_url;
   private String 	url_str;
   private Proxy		proxy;
@@ -605,7 +608,10 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
         		filename = "Torrent" + (long)(Math.random()*Long.MAX_VALUE);
         	}
 
-
+    			// in case we end up downloading the same magnet more than once concurrently add in some spice
+    		
+    		filename += "_" + uid.incrementAndGet();
+    		
     		filename += ".tmp";
 
         }else{
@@ -791,8 +797,12 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 	                						}
 
 	                						try{
-	                							percentDone = Integer.parseInt( s.substring( i, pos ).trim());
 
+	                								// keep < 100 until all done as there's some crappy code out there that treats
+	                								// 100% as done prematurely 
+	                							
+	                							percentDone = Math.min( 99, Integer.parseInt( s.substring( i, pos ).trim()));	
+	                							
 	                							progress_update = true;
 
 	                						}catch( Throwable e ){
@@ -931,7 +941,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 		            this.readTotal += bufBytes;
 
 		            if (size > 0){
-		              this.percentDone = (100 * this.readTotal) / size;
+		              this.percentDone = Math.min( 99,  (100 * this.readTotal) / size );
 		            }
 
 		            notifyListener();
@@ -1039,6 +1049,8 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 		        	  TorrentUtils.setObtainedFrom( file, original_url );
 		          }
 
+		          percentDone = 100;
+		          
 		          this.state = STATE_FINISHED;
 		        }
 		        this.notifyListener();
