@@ -28,6 +28,7 @@ import javax.net.ssl.*;
 
 import org.gudy.bouncycastle.util.encoders.Base64;
 
+import com.biglybt.core.CoreFactory;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.networkmanager.admin.NetworkAdmin;
@@ -41,6 +42,7 @@ import com.biglybt.pif.torrent.TorrentAnnounceURLListSet;
 import com.biglybt.pif.utils.resourcedownloader.ResourceDownloader;
 import com.biglybt.pif.utils.resourceuploader.ResourceUploader;
 import com.biglybt.pifimpl.local.PluginCoreUtils;
+import com.biglybt.plugin.magnet.MagnetPlugin;
 
 
 /**
@@ -55,6 +57,7 @@ public class UrlUtils
 	private static final String[] prefixes = new String[] {
 			"http://",
 			"https://",
+			"udp://",			// for trackers
 			"biglybt://",
 			"ftp://",
 			"dht://",
@@ -62,7 +65,7 @@ public class UrlUtils
 			"magnet://?",
 			"maggot://" };
 
-	private static final int MAGNETURL_STARTS_AT = 3;	// dht:// is a form of magnet URL
+	private static final int MAGNETURL_STARTS_AT = 5;	// dht:// is a form of magnet URL
 
 	private static final Object[] XMLescapes = new Object[] {
 		new String[] { "&", "&amp;" },
@@ -108,6 +111,13 @@ public class UrlUtils
 		return( "magnet:?xt=urn:btih:" + Base32.encode( hash ));
 	}
 
+	public static String
+	getURLForm(
+		InetSocketAddress	address )
+	{
+		return( AddressUtils.getHostAddressForURL(address) + ":" + address.getPort());
+	}
+	
 	public static String
 	getURLForm(
 		InetAddress		address,
@@ -318,6 +328,53 @@ public class UrlUtils
 		return( getMagnetURI( dm, Integer.MAX_VALUE ));
 	}
 
+	public static String
+	truncateForURI(
+		String		str,
+		int			max_utf8_bytes )
+	{	
+		if ( max_utf8_bytes == Integer.MAX_VALUE ){
+			
+			return( str );
+		}
+		
+		boolean truncated = false;
+		
+			// assume min size for simplicity, only an idiot would truncate below this...
+		
+		while( str.length() > 4 ){
+			
+			int enc_len = UrlUtils.encode( str ).getBytes( Constants.UTF_8 ).length;
+		
+			if ( enc_len > max_utf8_bytes ){
+				
+				if ( !truncated ){
+				
+					truncated = true;
+					
+					max_utf8_bytes -= 3;	// room for the ...
+				}
+				
+				str = str.substring( 0, str.length() - 1 );
+				
+				if ( Character.isHighSurrogate( str.charAt( str.length()-1 ))){
+					
+					str = str.substring( 0, str.length() - 1 );
+				}
+			}else{
+				
+				break;
+			}
+		}
+		
+		if ( truncated ){
+			
+			str = str + "...";
+		}
+		
+		return( str );
+	}
+	
 	public static String
 	getMagnetURI(
 		DownloadManager		dm,
@@ -335,12 +392,7 @@ public class UrlUtils
 			return( null );
 		}
 
-		String name = dm.getDisplayName();
-
-		if ( name.length() > max_name_len ){
-
-			name = name.substring( 0, max_name_len-3) + "...";
-		}
+		String name = truncateForURI( dm.getDisplayName(), max_name_len );
 
 		String[]	networks = dm.getDownloadState().getNetworks();
 
@@ -452,6 +504,26 @@ public class UrlUtils
 
 		return( magnet_str );
 	}
+	
+	public static String
+	addSource(
+		Download			download,
+		String				magnet,
+		InetSocketAddress	address )
+	{
+		if ( address != null ){
+
+			MagnetPlugin magnet_plugin = (MagnetPlugin)CoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( MagnetPlugin.class ).getPlugin();
+
+			if ( magnet_plugin != null ){
+				
+				magnet = magnet_plugin.addSource( download, magnet, address );
+			}
+		}
+		
+		return( magnet );
+	}
+	
 		/**
 		 * returns magnet uri if input is base 32 or base 16 encoded sha1 hash, null otherwise
 		 * @param base_hash
