@@ -157,6 +157,13 @@ PeerForeignDelegate
 			});
 	}
 
+	@Override
+	public boolean 
+	isMyPeer()
+	{
+		return( false );
+	}
+	
 	@Override
 	public void
 	start()
@@ -1156,14 +1163,6 @@ PeerForeignDelegate
 		return( foreign.getOutgoingRequestedPieceNumbers());
 	}
 
-	@Override
-	public int getOutboundDataQueueSize()
-	{
-			// don't know, assume all requests are queued and block size
-
-		return( getOutgoingRequestCount() * DiskManager.BLOCK_SIZE );
-	}
-
 	@Override
 	public int getLastPiece()
 	{
