@@ -21,6 +21,8 @@
 package com.biglybt.plugin.net.buddy;
 
 import java.io.File;
+import java.net.Inet4Address;
+import java.net.Inet6Address;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.URL;
@@ -69,6 +71,7 @@ BuddyPluginBuddy
 	private boolean			authorised;
 	private String			public_key;
 	private String			nick_name;
+	private String			my_name;
 	private List<Long>		recent_ygm;
 	private boolean			is_transient;
 	
@@ -77,6 +80,9 @@ BuddyPluginBuddy
 	private long				post_time;
 	
 	private InetSocketAddress	current_ip;
+	private InetSocketAddress	latest_ipv4;
+	private InetSocketAddress	latest_ipv6;
+	
 	private int					tcp_port;
 	private int					udp_port;
 	
@@ -147,6 +153,7 @@ BuddyPluginBuddy
 		boolean				_authorised,
 		String				_pk,
 		String				_nick_name,
+		String				_my_name,
 		int					_version,
 		String				_rss_local_cats,
 		String				_rss_remote_cats,
@@ -161,6 +168,7 @@ BuddyPluginBuddy
 		authorised			= _authorised;
 		public_key 			= _pk;
 		nick_name			= _nick_name;
+		my_name				= _my_name;
 		version				= Math.max( version, _version );
 		rss_local_cats		= stringToCats( _rss_local_cats );
 		rss_remote_cats		= stringToCats( _rss_remote_cats );
@@ -734,12 +742,50 @@ BuddyPluginBuddy
 	public String
 	getName()
 	{
+		String res;
+		
 		if ( nick_name != null ){
 
-			return( nick_name );
-		}
+			res = nick_name;
+			
+		}else{
 
-		return( getShortString());
+			res = getShortString();
+		}
+		
+		if ( my_name == null ){
+			
+			return( res );
+			
+		}else{
+			
+			return( my_name + " (" + res + ")" );
+		}
+	}
+	
+	public String
+	getMyName()
+	{
+		return( my_name );
+	}
+	
+	public void
+	setMyName(
+		String		_my_name )
+	{
+		if ( _my_name != null ){
+			
+			_my_name = _my_name.trim();
+			
+			if ( _my_name.isEmpty()){
+				
+				_my_name = null;
+			}
+		}
+		
+		my_name = _my_name;
+		
+		plugin_network.fireDetailsChanged( this );
 	}
 
 	public void
@@ -756,6 +802,13 @@ BuddyPluginBuddy
 		return( current_ip );
 	}
 
+	public InetSocketAddress
+	getLatestIP(
+		boolean	v4 )
+	{
+		return( v4?latest_ipv4:latest_ipv6 );
+	}
+	
 	public InetSocketAddress
 	getAdjustedIP()
 	{
@@ -841,6 +894,11 @@ BuddyPluginBuddy
 	isOnline(
 		boolean	is_connected )
 	{
+		if ( destroyed ){
+			
+			return( false );
+		}
+		
 		boolean	connected = isConnected();
 
 			// if we're connected then we're online whatever
@@ -2019,6 +2077,19 @@ BuddyPluginBuddy
 			if ( current_ip == null ){
 
 				current_ip	= _ip;
+				
+				if ( current_ip != null && !current_ip.isUnresolved()){
+				
+					if ( current_ip.getAddress() instanceof Inet4Address ){
+					
+						latest_ipv4 = current_ip;
+						
+					}else if ( current_ip.getAddress() instanceof Inet6Address ){
+						
+						latest_ipv6 = current_ip;
+					}
+				}
+				
 				tcp_port	= _tcp_port;
 				udp_port	= _udp_port;
 			}
@@ -2127,6 +2198,19 @@ BuddyPluginBuddy
 					setAddress( _ias );
 
 					current_ip		= _ias;
+					
+					if ( current_ip != null && !current_ip.isUnresolved()){
+						
+						if ( current_ip.getAddress() instanceof Inet4Address ){
+						
+							latest_ipv4 = current_ip;
+							
+						}else if ( current_ip.getAddress() instanceof Inet6Address ){
+							
+							latest_ipv6 = current_ip;
+						}
+					}
+					
 					tcp_port		= _tcp_port;
 					udp_port		= _udp_port;
 
