@@ -57,6 +57,7 @@ import com.biglybt.plugin.dht.DHTPlugin;
 import com.biglybt.plugin.dht.DHTPluginContact;
 import com.biglybt.plugin.dht.DHTPluginOperationListener;
 import com.biglybt.plugin.dht.DHTPluginValue;
+import com.biglybt.util.StringCompareUtils;
 
 /**
  * @author parg
@@ -3938,7 +3939,23 @@ DHTTrackerPlugin
 
 							status = TrackerPeerSource.ST_DISABLED;
 							
-							new_status_str = MessageText.getString( "label.peer.source.disabled" );
+							boolean done = false;
+							
+							try{
+								if ( download.getTorrent().isPrivate()){
+												
+									new_status_str = MessageText.getString( "label.private" );
+									
+									done = true;
+								}
+							}catch( Throwable e ){
+								
+							}
+					
+							if ( !done ){
+								
+								new_status_str = MessageText.getString( "label.peer.source.disabled" );
+							}
 						}
 
 						status_str = new_status_str;
@@ -4099,6 +4116,13 @@ DHTTrackerPlugin
 				{
 					announce( download );
 				}
+				
+				@Override
+				public long[]
+				getReportedStats()
+				{
+					return( null );
+				}
 			});
 	}
 
@@ -4519,7 +4543,19 @@ DHTTrackerPlugin
 		sameAs(
 			putDetails		other )
 		{
-			return( getEncoded().equals( other.getEncoded()));
+				// ignore i2p setting as this can flip about with network mixing
+			
+			if ( !StringCompareUtils.equals( ip_override, other.ip_override )){
+				
+				return( false );
+			}
+			
+			if ( tcp_port == other.tcp_port && udp_port == other.udp_port ){
+				
+				return( true );
+			}
+			
+			return( false );
 		}
 	}
 
