@@ -31,6 +31,7 @@ import com.biglybt.platform.PlatformManager;
 import com.biglybt.platform.PlatformManagerCapabilities;
 import com.biglybt.platform.PlatformManagerFactory;
 
+import com.biglybt.pif.ui.UIInstance;
 import com.biglybt.pif.ui.config.ConfigSection;
 import com.biglybt.pif.ui.config.Parameter;
 import com.biglybt.pif.ui.config.ParameterListener;
@@ -79,13 +80,17 @@ public class ConfigSectionFile
 			MessageText.getString("OpenTorrentOptions.show.always"),
 			MessageText.getString("OpenTorrentOptions.show.many"),
 		};
-		add(new StringListParameterImpl(
-				ConfigurationDefaults.CFG_TORRENTADD_OPENOPTIONS,
-				"ConfigView.section.file.showopentorrentoptions", openValues,
-				openLabels), listDefaultDir);
+		StringListParameterImpl paramShowOpenTorrentOptions = new StringListParameterImpl(
+			ConfigurationDefaults.CFG_TORRENTADD_OPENOPTIONS,
+			"ConfigView.section.file.showopentorrentoptions", openValues,
+			openLabels);
+		paramShowOpenTorrentOptions.setAllowedUiTypes(UIInstance.UIT_SWT);
+		add(paramShowOpenTorrentOptions, listDefaultDir);
 
-		add(new BooleanParameterImpl(BCFG_UI_ADDTORRENT_OPENOPTIONS_SEP,
-				"ConfigView.section.file.showopentorrentoptions.sep"), listDefaultDir);
+		BooleanParameterImpl paramShowSep = new BooleanParameterImpl(BCFG_UI_ADDTORRENT_OPENOPTIONS_SEP,
+			"ConfigView.section.file.showopentorrentoptions.sep");
+		paramShowSep.setAllowedUiTypes(UIInstance.UIT_SWT);
+		add(paramShowSep, listDefaultDir);
 
 		// def dir: autoSave -> auto-rename
 
@@ -270,21 +275,36 @@ public class ConfigSectionFile
 
 		// restart out of space downloads
 
-		// resume data
 		BooleanParameterImpl OOSDRE = new BooleanParameterImpl(
 				BCFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART,
 				"ConfigView.label.restart.no.space.dls");
 		add(OOSDRE, Parameter.MODE_INTERMEDIATE);
 
-		IntParameterImpl paramOOSDRInterval = new IntParameterImpl(
+		IntParameterImpl OOSDRInterval = new IntParameterImpl(
 				ICFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART_MINS,
 				"ConfigView.label.restart.no.space.dls.interval", 1, Integer.MAX_VALUE);
-		add(paramOOSDRInterval, Parameter.MODE_INTERMEDIATE);
-		paramOOSDRInterval.setIndent(1, true);
-		paramOOSDRInterval.setSuffixLabelKey("ConfigView.text.minutes");
+		add(OOSDRInterval, Parameter.MODE_INTERMEDIATE);
+		OOSDRInterval.setIndent(1, true);
+		OOSDRInterval.setSuffixLabelKey("ConfigView.text.minutes");
 
-		OOSDRE.addEnabledOnSelection(paramOOSDRInterval);
+		OOSDRE.addEnabledOnSelection(OOSDRInterval);
 
+		// restart missing file downloads
+
+		BooleanParameterImpl MFDRE = new BooleanParameterImpl(
+				BCFG_MISSING_FILE_DOWNLOAD_RESTART,
+				"ConfigView.label.restart.missing.file.dls");
+		add(MFDRE, Parameter.MODE_INTERMEDIATE);
+
+		IntParameterImpl MFDRInterval = new IntParameterImpl(
+				ICFG_MISSING_FILE_DOWNLOAD_RESTART_MINS,
+				"ConfigView.label.restart.no.space.dls.interval", 1, Integer.MAX_VALUE);
+		add(MFDRInterval, Parameter.MODE_INTERMEDIATE);
+		MFDRInterval.setIndent(1, true);
+		MFDRInterval.setSuffixLabelKey("ConfigView.text.minutes");
+
+		MFDRE.addEnabledOnSelection(MFDRInterval);
+	
 		// use resume
 
 		// resume data
@@ -498,10 +518,12 @@ public class ConfigSectionFile
 		IntListParameterImpl paramConfirmDelete = new IntListParameterImpl(
 				ICFG_TB_CONFIRM_DELETE_CONTENT, "ConfigView.section.file.tb.delete",
 				values, labels);
+		paramConfirmDelete.setAllowedUiTypes(UIInstance.UIT_SWT);
 		add(paramConfirmDelete, Parameter.MODE_INTERMEDIATE, listDeletion);
 
 		BooleanParameterImpl paramDeleteTorrentFile = new BooleanParameterImpl(
 				BCFG_DEF_DELETETORRENT, "ConfigView.section.file.delete.torrent");
+		paramDeleteTorrentFile.setAllowedUiTypes(UIInstance.UIT_SWT);
 		add(paramDeleteTorrentFile, Parameter.MODE_INTERMEDIATE, listDeletion);
 
 		try {
