@@ -56,7 +56,6 @@ import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.mdi.MultipleDocumentInterface;
 import com.biglybt.ui.swt.MenuBuildUtils;
-import com.biglybt.ui.swt.SimpleTextEntryWindow;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.imageloader.ImageLoader;
 import com.biglybt.ui.swt.minibar.AllTransfersBar;
@@ -66,8 +65,6 @@ import com.biglybt.ui.swt.views.utils.TagUIUtils;
 
 import com.biglybt.pif.disk.DiskManagerFileInfo;
 import com.biglybt.pif.download.Download;
-import com.biglybt.pif.ui.UIInputReceiver;
-import com.biglybt.pif.ui.UIInputReceiverListener;
 import com.biglybt.pif.ui.UIInstance;
 import com.biglybt.pif.ui.menus.MenuItem;
 import com.biglybt.pif.ui.menus.*;
@@ -86,6 +83,9 @@ BuddyPluginView
 	implements BuddyPluginViewInterface
 {
 	public static final String VIEWID_CHAT = "azbuddy.ui.menu.chat";
+	
+	private static final Object TT_KEY = new Object();
+	
 	private  TimerEvent 			buddyStatusInit;
 	private BuddyPluginAZ2Listener buddyPluginAZ2Listener;
 	
@@ -403,7 +403,7 @@ BuddyPluginView
 					
 					tick_event = SimpleTimer.addPeriodicEvent(
 							"Buddy:GuiUpdater2",
-							20*1000,
+							10*1000,
 							new TimerEventPerformer()
 							{
 								@Override
@@ -464,17 +464,7 @@ BuddyPluginView
 						if ( rates <= 0 ){
 
 								// defer switch until we're actually transferring
-
-							SimpleTimer.addEvent(
-								"BP:backoff",
-								SystemTime.getOffsetTime( 1000 ),
-								new TimerEventPerformer() {
-
-									@Override
-									public void perform(TimerEvent event) {
-										updateStatus();
-									}
-								});
+								// we'll come back through here soon due to the updater timer
 
 							return;
 						}
@@ -634,6 +624,11 @@ BuddyPluginView
 					chat_item,
 					new MenuBuildUtils.ChatKeyResolver()
 					{
+						@Override
+						public String getResourceKey(){
+							return( "menu.discuss.download" );
+						}
+						
 						@Override
 						public String
 						getChatKey(
@@ -641,6 +636,36 @@ BuddyPluginView
 						{
 							return( BuddyPluginUtils.getChatKey((Download)object ));
 						}
+						
+						@Override
+						public boolean 
+						canShareMessage()
+						{
+							return( true );
+						}
+						
+						@Override
+						public void
+						shareMessage(
+							Object				target,
+							ChatInstance		chat )
+						{
+							if ( target instanceof Download ){
+								
+								Download dl = (Download)target;
+							
+								chat.sendMessage( dl );
+								
+							}else{
+								
+								Download[] dls = (Download[])target;
+								
+								for ( Download dl: dls ){
+									
+									chat.sendMessage( dl );
+								}
+							}
+						}
 					});
 
 			addBetaSubviews( true );
@@ -966,6 +991,8 @@ BuddyPluginView
 							boolean	is_pending 	= false;
 							boolean	is_sm		= false;
 							
+							String msg_history = "";
+							
 							for ( ChatInstance instance: instances ){
 								
 								if ( instance.getMessageOutstanding()){
@@ -985,6 +1012,72 @@ BuddyPluginView
 										}
 									}
 								}
+								
+								if ( plugin.getBeta().getFTUXAccepted()){
+																		
+									Object[] entry = (Object[])dl.getUserData( TT_KEY );
+									
+									long now = SystemTime.getCurrentTime();
+									
+									if ( 	entry == null ||
+											now - (Long)entry[0] > 30*1000 ){
+										
+										List<ChatMessage> msgs = instance.getMessages();
+										
+										int num = msgs.size();
+										
+										if ( num > 0 ){
+																				
+											String msgs_str = "";
+										
+											int added = 0;
+											
+											for ( int i=msgs.size()-1;i>=0;i--){
+												
+												ChatMessage msg = msgs.get(i);
+												
+												if ( msg.isIgnored()){
+													
+													continue;
+													
+												}else if ( msg.getMessageType() == ChatMessage.MT_NORMAL ){
+													
+													String str = msg.getMessage();
+															
+													if ( str.length() > 50 ){
+														
+														str = str.substring( 0,  47 ) + "...";
+													}
+													
+													msgs_str = str + "\n" + msgs_str;
+													
+													added++;
+													
+													if ( added == 5 ){
+														
+														break;
+													}
+												}
+											}
+											
+											if ( !msgs_str.isEmpty()){
+												
+												if ( !msg_history.isEmpty()){
+													
+													msg_history += "\n\n";
+												}
+		
+												msg_history += msgs_str;
+											}
+										}
+										
+										dl.setUserData( TT_KEY, new Object[]{ now, msg_history });
+										
+									}else{
+										
+										msg_history = (String)entry[1];	
+									}
+								}
 							}
 
 							Image	graphic;
@@ -1012,6 +1105,11 @@ BuddyPluginView
 								tooltip		= MessageText.getString( "label.no.messages" );
 								sort_order	= 0;
 							}
+							
+							if ( !msg_history.isEmpty()){
+								
+								tooltip += "\n\n---- " + MessageText.getString( "label.history" ) + " ----\n" + msg_history.trim();
+							}
 
 							cell.setMarginHeight(0);
 							cell.setGraphic( graphic );
@@ -1055,6 +1153,29 @@ BuddyPluginView
 										}
 									}
 								}
+							}else if ( event.eventType == TableCellMouseEvent.EVENT_MOUSEDOUBLECLICK ){
+								
+								TableCell cell = event.cell;
+
+								Download	dl = (Download)cell.getDataSource();
+
+								if ( dl != null ){
+									
+									List<ChatInstance> instances = BuddyPluginUtils.peekChatInstances( dl );
+									
+									for ( ChatInstance instance: instances ){
+										
+										try{
+											BuddyPluginUtils.getBetaPlugin().showChat( instance  );
+												
+										}catch( Throwable e ){
+												
+											Debug.out( e );
+										}
+									}
+								}
+								
+								event.skipCoreFunctionality = true;
 							}
 						}
 					};
@@ -1708,88 +1829,16 @@ BuddyPluginView
 
 			mi.setStyle( MenuItem.STYLE_MENU );
 
-			menu_items.add( mi );
-
-			mi.addFillListener(new com.biglybt.pif.ui.menus.MenuItemFillListener() {
+			BuddyUIUtils.createChat( menu_manager, mi, false, new BuddyUIUtils.ChatCreationListener(){
+				
 				@Override
-				public void menuWillBeShown(MenuItem menu, Object data){
-
-					menu.removeAllChildItems();
-
-					MenuItem mi = menu_manager.addMenuItem( menu, "!" + MessageText.getString( "label.public" ) + "...!" );
-
-					mi.addListener(
-						new MenuItemListener()
-						{
-							@Override
-							public void
-							selected(
-								MenuItem			menu,
-								Object 				target )
-							{
-								SimpleTextEntryWindow entryWindow = new SimpleTextEntryWindow(
-										"chat.view.enter.key.title", "chat.view.enter.key.msg");
-
-								entryWindow.prompt(new UIInputReceiverListener() {
-									@Override
-									public void UIInputReceiverClosed(UIInputReceiver receiver) {
-										if (!receiver.hasSubmittedInput()) {
-											return;
-										}
-
-										String key = receiver.getSubmittedInput().trim();
-
-										BuddyPluginUtils.createBetaChat( AENetworkClassifier.AT_PUBLIC, key, null );
-									}
-								});
-
-							}
-						});
-
-					mi = menu_manager.addMenuItem( menu, "!" + MessageText.getString( "label.anon" ) + "...!" );
-
-					mi.addListener(
-							new MenuItemListener()
-							{
-								@Override
-								public void
-								selected(
-									MenuItem			menu,
-									Object 				target )
-								{
-									if ( plugin.getBeta().isI2PAvailable()){
-
-										SimpleTextEntryWindow entryWindow = new SimpleTextEntryWindow(
-												"chat.view.enter.key.title", "chat.view.enter.key.msg");
-
-										entryWindow.prompt(new UIInputReceiverListener() {
-											@Override
-											public void UIInputReceiverClosed(UIInputReceiver receiver) {
-												if (!receiver.hasSubmittedInput()) {
-													return;
-												}
-
-												String key = receiver.getSubmittedInput().trim();
-
-												BuddyPluginUtils.createBetaChat( AENetworkClassifier.AT_I2P, key, null );
-											}
-										});
-
-									}else{
-
-										I2PHelpers.installI2PHelper( null, null, null );
-									}
-
-								}
-							});
-
-					if ( I2PHelpers.isInstallingI2PHelper()){
-
-						mi.setEnabled( false );
-						mi.setText(  mi.getText() + " (" + MessageText.getString( "PeersView.state.pending" ) + ")" );
-					}
-				}});
-
+				public void chatCreated(Object target, String name){
+				}
+				
+				@Override
+				public void chatAvailable(Object target,ChatInstance chat){
+				}
+			});
 
 				// chat overview
 
