@@ -21,6 +21,7 @@ package com.biglybt.ui.swt.mainwindow;
 import java.io.*;
 import java.net.URL;
 import java.util.*;
+import java.util.List;
 
 import com.biglybt.core.Core;
 import com.biglybt.core.CoreFactory;
@@ -93,6 +94,7 @@ import com.biglybt.pif.ui.tables.*;
 import com.biglybt.pif.update.Update;
 import com.biglybt.pif.update.UpdateCheckInstance;
 import com.biglybt.pif.update.UpdateCheckInstanceListener;
+import com.biglybt.pif.update.UpdateChecker;
 
 public class MenuFactory
 	implements IMenuConstants
@@ -583,7 +585,7 @@ public class MenuFactory
 
 			@Override
 			public void handleEvent(Event event) {
-				UIFunctionsManagerSWT.getUIFunctionsSWT().dispose(true, false);
+				UIFunctionsManagerSWT.getUIFunctionsSWT().dispose(true);
 			}
 		});
 		return file_restart;
@@ -600,7 +602,7 @@ public class MenuFactory
 		file_exit.addListener(SWT.Selection, new Listener() {
 			@Override
 			public void handleEvent(Event e) {
-				UIFunctionsManagerSWT.getUIFunctionsSWT().dispose(false, false);
+				UIFunctionsManagerSWT.getUIFunctionsSWT().dispose(false);
 			}
 		});
 
@@ -1681,7 +1683,7 @@ public class MenuFactory
 				}
 
 				new TextViewerWindow(
-						MessageText.getString( "MainWindow.menu.quick_view" ) + ": " + file.getName(),
+						MessageText.getString( "torrent.view.info.title" ) + ": " + file.getName(),
 						null, content.toString(), false  );
 
 			}
@@ -2364,30 +2366,63 @@ public class MenuFactory
 											}
 											if (!hasUpdates) {
 
-												int build = Constants.getIncrementalBuild();
-
-												if ( COConfigurationManager.getBooleanParameter( "Beta Programme Enabled" ) && build > 0 ){
-
-													String build_str = "" + build;
-
-													if ( build_str.length() == 1 ){
-
-														build_str = "0" + build_str;
+												UpdateChecker[] checkers = instance.getCheckers();
+												
+												List<String> errors = new ArrayList<>();
+												
+												for ( UpdateChecker checker: checkers ){
+													
+													if ( checker.getFailed()){
+														
+														String name = checker.getComponent().getName();
+														
+														Throwable reason = checker.getFailureReason();
+														
+														errors.add( name + ", " + Debug.getNestedExceptionMessage( reason ));
+													}
+												}
+												
+												if ( errors.isEmpty()){
+													
+													int build = Constants.getIncrementalBuild();
+	
+													if ( COConfigurationManager.getBooleanParameter( "Beta Programme Enabled" ) && build > 0 ){
+	
+														String build_str = "" + build;
+	
+														if ( build_str.length() == 1 ){
+	
+															build_str = "0" + build_str;
+														}
+	
+														MessageBoxShell mb = new MessageBoxShell(
+																SWT.ICON_INFORMATION | SWT.OK,
+																"window.update.noupdates.beta", new String[]{ "B" + build_str });
+	
+														mb.open(null);
+	
+													}else{
+	
+														MessageBoxShell mb = new MessageBoxShell(
+																SWT.ICON_INFORMATION | SWT.OK,
+																"window.update.noupdates", (String[]) null);
+	
+														mb.open(null);
 													}
-
-													MessageBoxShell mb = new MessageBoxShell(
-															SWT.ICON_INFORMATION | SWT.OK,
-															"window.update.noupdates.beta", new String[]{ "B" + build_str });
-
-													mb.open(null);
-
 												}else{
-
+													
+													String str = "";
+													
+													for ( String e: errors ){
+														
+														str += (str.isEmpty()?"":"\n\n") + e;
+													}
+														
 													MessageBoxShell mb = new MessageBoxShell(
-															SWT.ICON_INFORMATION | SWT.OK,
-															"window.update.noupdates", (String[]) null);
+															SWT.ICON_ERROR | SWT.OK,
+															"window.update.failed", new String[]{ str });
 
-													mb.open(null);
+													mb.open(null);												
 												}
 											}
 										}
@@ -2754,17 +2789,8 @@ public class MenuFactory
 		return menuItem;
 	}
 
-	public static final MenuItem addMenuItem(Menu menu, int style,
-			String localizationKey, Listener selListener) {
-		return addMenuItem(menu, style, -1, localizationKey, selListener);
-	}
-
-	public static final MenuItem addMenuItem(Menu menu, int style, int index,
-			String localizationKey, Listener selListener) {
-		if (index < 0 || index > menu.getItemCount()) {
-			index = menu.getItemCount();
-		}
-		MenuItem menuItem = new MenuItem(menu, style, index);
+	public static final MenuItem addMenuItem(Menu menu, int style, String localizationKey, Listener selListener) {
+		MenuItem menuItem = new MenuItem(menu, style);
 		Messages.setLanguageText(menuItem, localizationKey);
 		KeyBindings.setAccelerator(menuItem, localizationKey);
 		menuItem.addListener(SWT.Selection, selListener);
@@ -3109,7 +3135,7 @@ public class MenuFactory
 				public void widgetSelected(SelectionEvent e) {
 					UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 					if (uiFunctions != null) {
-						uiFunctions.dispose(true, false);
+						uiFunctions.dispose(true);
 					}
 				}
 			});
