@@ -23,6 +23,15 @@ import java.util.*;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.*;
+import org.eclipse.swt.dnd.DND;
+import org.eclipse.swt.dnd.DragSource;
+import org.eclipse.swt.dnd.DragSourceAdapter;
+import org.eclipse.swt.dnd.DragSourceEvent;
+import org.eclipse.swt.dnd.DropTarget;
+import org.eclipse.swt.dnd.DropTargetAdapter;
+import org.eclipse.swt.dnd.DropTargetEvent;
+import org.eclipse.swt.dnd.TextTransfer;
+import org.eclipse.swt.dnd.Transfer;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.events.MenuEvent;
@@ -76,6 +85,8 @@ public class TabbedMDI
 	implements TabbedMdiInterface, AEDiagnosticsEvidenceGenerator,
 	ParameterListener, ObfuscateImage
 {
+	private static final String KEY_AUTO_CLOSE = "TabbedMDI:autoclose";
+	
 	private CTabFolder tabFolder;
 
 	private LinkedList<MdiEntry>	select_history = new LinkedList<>();
@@ -143,6 +154,12 @@ public class TabbedMDI
 
 		mapUserClosedTabs = COConfigurationManager.getMapParameter(key, new HashMap());
 		COConfigurationManager.addWeakParameterListener(this, false, key);
+		
+		String key2 = props_prefix + ".tabOrder";
+
+		List<String> pref = BDecoder.decodeStrings(COConfigurationManager.getListParameter( key2, new ArrayList<>()));
+		
+		setPreferredOrder( pref.toArray( new String[0] ));
 	}
 	
 	@Override
@@ -252,7 +269,18 @@ public class TabbedMDI
 		tabFolder.addListener(SWT.Selection, new Listener() {
 			@Override
 			public void handleEvent(Event event) {
-				TabbedEntry entry = (TabbedEntry) event.item.getData("TabbedEntry");
+				CTabItem item = (CTabItem)event.item;
+				
+				TabbedEntry entry = (TabbedEntry)item.getData("TabbedEntry");
+				
+				if ( !isMainMDI && !soMain.isDisposed() && item.getData( KEY_AUTO_CLOSE ) == null ){
+					String id = entry.getViewID();
+				
+					String key = props_prefix + ".selectedTab";
+	
+					COConfigurationManager.setParameter( key, id );
+				}
+				
 				showEntry(entry);
 			}
 		});
@@ -329,7 +357,7 @@ public class TabbedMDI
 		}
 
 		tabFolder.addListener(SWT.MouseDoubleClick, e -> {
-			if (!tabFolder.getMinimized() && tabFolder.getMaximizeVisible()) {
+			if (!tabFolder.getMinimized() && tabFolder.getMinimizeVisible()) {
 				minimize();
 			}
 		});
@@ -361,6 +389,9 @@ public class TabbedMDI
 				UISWTViewBuilderCore builder = entry == null ? null
 						: entry.getEventListenerBuilder();
 
+				if ( entry != null ){
+					entry.setUserInitiatedClose();
+				}
 				if (select_history.remove(entry)) {
 
 					if (select_history.size() > 0) {
@@ -487,55 +518,110 @@ public class TabbedMDI
 				final CTabItem item = tabFolder.getItem(
 						tabFolder.toControl(cursorLocation.x, cursorLocation.y));
 
-				boolean need_sep = false;
 
-				if (item == null) {
+				if ( item == null ){
 
-					need_sep = mapUserClosedTabs.size() > 0;
-					if (need_sep) {
-  					for (Object id : mapUserClosedTabs.keySet()) {
-  						final String view_id = (String) id;
+					for (Object id : mapUserClosedTabs.keySet()) {
+						
+						final String view_id = (String) id;
 
-  						org.eclipse.swt.widgets.MenuItem mi = new org.eclipse.swt.widgets.MenuItem(menu, SWT.PUSH);
+						org.eclipse.swt.widgets.MenuItem mi = new org.eclipse.swt.widgets.MenuItem(menu, SWT.PUSH);
 
-  						String title;
+						String title;
 
-  						Object oTitle = mapUserClosedTabs.get(id);
-  						if (oTitle instanceof String && ((String) oTitle).length() > 0) {
-								title = (String) oTitle;
-							} else {
-								title = MessageText.getString(getViewTitleID(view_id));
+						Object oTitle = mapUserClosedTabs.get(id);
+						if (oTitle instanceof String && ((String) oTitle).length() > 0) {
+							title = (String) oTitle;
+						} else {
+							title = MessageText.getString(getViewTitleID(view_id));
+						}
+
+						// can still end up with a resource key here :( not sure why we don't always use the view-id
+
+						if ( title.contains( "." )){
+							String temp = MessageText.getString( title );
+							if ( temp != null && !temp.isEmpty()){
+								title = temp;
 							}
-  						mi.setText(title);
+						}
 
-  						mi.addListener(SWT.Selection, new Listener() {
-  							@Override
-							  public void handleEvent(Event event) {
-  								String key = props_prefix + ".closedtabs";
+						mi.setText(title);
 
-  								Map closedtabs = COConfigurationManager.getMapParameter(key,
-  										new HashMap());
+						mi.addListener(SWT.Selection, new Listener() {
+							@Override
+							public void handleEvent(Event event) {
+								String key = props_prefix + ".closedtabs";
 
-  								if (closedtabs.containsKey(view_id)) {
+								Map closedtabs = COConfigurationManager.getMapParameter(key,
+										new HashMap());
 
-  									closedtabs.remove(view_id);
+								if (closedtabs.containsKey(view_id)) {
 
-  									COConfigurationManager.setParameter(key, closedtabs);
-  								}
+									closedtabs.remove(view_id);
 
-  								showEntryByID(view_id);
-  							}
-  						});
+									COConfigurationManager.setParameter(key, closedtabs);
+								}
 
-  					}
+								if ( !isMainMDI ){
+
+									CTabItem[] items = tabFolder.getItems();
+
+									List<String>	ids = new ArrayList<>();
+
+									for ( CTabItem item: items ){
+
+										TabbedEntry e = getEntryFromTabItem( item );
+
+										if ( e != null ){
+											ids.add( e.getViewID());
+										}
+									}
+
+									if ( !ids.contains( view_id )){
+										ids.add( view_id );
+									}
+
+									setPreferredOrder( ids.toArray( new String[0] ));
+
+									String key2 = props_prefix + ".tabOrder";
+
+									COConfigurationManager.setParameter( key2, ids );
+
+								}
+								showEntryByID(view_id);
+							}
+						});
 					}
-				}
 
-				if (need_sep) {
+					if ( !mapUserClosedTabs.isEmpty()){
+						
+						new org.eclipse.swt.widgets.MenuItem(menu, SWT.SEPARATOR);
+					}
+
+					org.eclipse.swt.widgets.MenuItem miReset = new org.eclipse.swt.widgets.MenuItem(menu, SWT.PUSH);
+
+					miReset.setText( MessageText.getString( "menu.reset.tabs" ));
+					
+					miReset.addListener( SWT.Selection, (ev)->{
+												
+						mapUserClosedTabs.clear();
+						
+						String key = props_prefix + ".closedtabs";
+						
+						COConfigurationManager.setParameter(key, mapUserClosedTabs );
+						
+						setPreferredOrder( new String[0] );
+						
+						String key2 = props_prefix + ".tabOrder";
+
+						COConfigurationManager.setParameter( key2, new ArrayList<>());
+						
+						buildTabs();
+					});
+					
 					new org.eclipse.swt.widgets.MenuItem(menu, SWT.SEPARATOR);
 				}
 
-
 				TabbedEntry entry = null;
 				if (item != null) {
 					entry = getEntryFromTabItem(item);
@@ -558,19 +644,189 @@ public class TabbedMDI
 			setMinimized(toMinimize);
 		}
 
-		// Create views registered to this tab
-
-		ViewManagerSWT vm = ViewManagerSWT.getInstance();
-		List<UISWTViewBuilderCore> builders = vm.getBuilders(getViewID(),
-				getDataSourceType());
-		for (UISWTViewBuilderCore builder : builders) {
-			try {
-				createEntry(builder, true);
-			} catch (Exception ignore) {
-			}
+		if ( !isMainMDI ){
+			
+			addDragDropListeners();
 		}
+			// Create views registered to this tab
+
+		buildTabs();
 	}
 
+    private void addDragDropListeners() {
+  
+        DragSource dragSource = new DragSource( tabFolder, DND.DROP_MOVE );
+                
+        dragSource.setTransfer(new Transfer[] {TextTransfer.getInstance()});
+        
+        dragSource.addDragListener(     		
+    		new DragSourceAdapter() {
+    			 
+                private CTabItem item;
+     
+                @Override
+                public void dragStart(DragSourceEvent event) {
+                   
+                    Point cursorLocation = tabFolder.getDisplay().getCursorLocation();
+                    
+                    Point p = tabFolder.toControl(cursorLocation);
+                    
+                    item = tabFolder.getItem( p );
+                    
+                    event.doit = item != null;
+                }
+     
+                @Override
+                public void dragSetData(final DragSourceEvent event) {
+                	
+                	if ( item != null ){
+                		
+                		event.data = ((TabbedEntry)item.getData("TabbedEntry")).getViewID();
+
+                	}else{
+
+                		event.data 	= null;
+                		event.doit	= false;
+                	}
+                }
+            });
+        
+        DropTarget dropTarget = new DropTarget( tabFolder, DND.DROP_MOVE);
+        
+        dropTarget.setTransfer(new Transfer[] {TextTransfer.getInstance()});
+        
+        dropTarget.addDropListener(
+    		new DropTargetAdapter(){
+            	
+                private int pos;
+     
+                @Override
+                public void dragOver( DropTargetEvent event ){
+                	                    	                        
+                    Point rel = tabFolder.toControl( new Point(event.x, event.y ));
+                    
+                    if ( rel.y > tabFolder.getTabHeight()){
+                    	
+                    	 event.detail = DND.DROP_NONE;
+                    	 
+                    	 pos = -1;
+                    	 
+                    }else{
+                    	
+                        event.detail = DND.DROP_MOVE;
+
+                    	CTabItem item = tabFolder.getItem( rel );
+
+
+                    	if ( item == null ){
+
+                    		pos = tabFolder.getItemCount() - 1;
+
+                    	}else{
+
+                    		pos = tabFolder.indexOf( item );
+                    	}
+                    }
+                }
+     
+                @Override
+                public void drop(final DropTargetEvent event) {
+                    
+                	if ( pos < 0 ){
+                		
+                		return;
+                	}
+                	
+					CTabItem[] items = tabFolder.getItems();
+
+					List<String>	ids = new ArrayList<>();
+					
+					for ( CTabItem x: items ){
+						
+						ids.add( ((TabbedEntry)x.getData("TabbedEntry")).getViewID());
+					}
+					
+					String target_id = (String)event.data;
+					
+					if ( ids.indexOf( target_id ) == pos ){
+						
+						return;
+					}
+					
+					ids.remove( target_id );
+					
+					ids.add( pos, target_id );
+															
+					setPreferredOrder( ids.toArray( new String[0]));
+					
+					String key2 = props_prefix + ".tabOrder";
+
+					COConfigurationManager.setParameter( key2, ids );
+					
+					buildTabs();
+                }
+            });
+        
+        tabFolder.addListener( SWT.Dispose, (ev)->{ dragSource.dispose(); dropTarget.dispose();});
+    }
+    
+	private void
+	buildTabs()
+	{
+		CTabItem[] items = tabFolder.getItems();
+		
+		for ( CTabItem item: items ){
+			
+			item.setData( KEY_AUTO_CLOSE, true );
+		}
+		
+		for ( CTabItem item: items ){
+			
+			item.dispose();
+		}
+
+		ViewManagerSWT vm = ViewManagerSWT.getInstance();
+		
+		List<UISWTViewBuilderCore> builders = vm.getBuilders(getViewID(),getDataSourceType());
+		
+		for (UISWTViewBuilderCore builder : builders){
+			
+			try{
+				createEntry(builder, true);
+				
+			}catch (Exception ignore) {
+			}	
+		}
+		
+		if ( !isMainMDI ){
+		
+			Utils.execSWTThreadLater(0,()->{
+
+				String key = props_prefix + ".selectedTab";
+				
+				String id = COConfigurationManager.getStringParameter( key, null );
+				
+				if ( id != null ){
+				
+					int index = indexOf( id );
+					
+					if ( index != -1 ){
+						
+						tabFolder.setSelection( index );
+						
+						return;
+					}
+				}
+				
+				if ( tabFolder.getItemCount() > 0 ){
+					
+					tabFolder.setSelection( 0 );
+				}
+			});
+			
+		}
+	}
+	
 	private MdiEntryVitalityImageSWT getVitalityImageAtPos(int x, int y) {
 		CTabItem item = tabFolder.getItem(new Point(x, y));
 		if (item == null) {
@@ -665,7 +921,7 @@ public class TabbedMDI
 			}
 			selection.getControl().setVisible(true);
 			*/
-			tabbedEntry.updateUI();
+			tabbedEntry.updateUI( true );
 		}
 
 		if (tabFolder.getMaximizeVisible()) {
@@ -876,7 +1132,9 @@ public class TabbedMDI
 			return;
 		}
 
-		if ( getEntry( entry.getViewID()) != entry ){
+		String viewID = entry.getViewID();
+		
+		if ( getEntry( viewID ) != entry ){
 			
 			// entry has been deleted/replaced in the meantime
 		
@@ -902,6 +1160,46 @@ public class TabbedMDI
 			}
 		}
 
+		if ( index == -1 ){
+			
+			String[] order = getPreferredOrder();
+			for (int i = 0; i < order.length; i++) {
+				String orderID = order[i];
+				if (orderID.equals(viewID)) {
+					
+					CTabItem[] items = tabFolder.getItems();
+					Map<String,Integer>	map = new HashMap<>();
+					
+					for ( int j=0;j<items.length;j++){
+						TabbedEntry e = getEntryFromTabItem( items[j] );
+						if ( e != null ){
+							map.put( e.getViewID(), j );
+						}
+					}
+					
+					for ( int j=i-1;j>=0;j--){
+						Integer x = map.get( order[j] );
+						if ( x != null ){
+							index = x+1;
+							break;
+						}
+					}
+					
+					if ( index == -1 ){
+						for ( int j=i+1;j<order.length;j++){
+							Integer x = map.get( order[j] );
+							if ( x != null ){
+								index = x;
+								break;
+							}
+						}
+					}
+					
+					break;
+				}
+			}
+		}
+
 		if (index < 0 || index >= tabFolder.getItemCount()) {
 			index = tabFolder.getItemCount();
 		}
@@ -940,6 +1238,38 @@ public class TabbedMDI
 		}
 	}
 
+	private int
+	indexOf(
+		MdiEntry	entry )
+	{
+		CTabItem[] items = tabFolder.getItems();
+		
+		for ( int i=0;i<items.length;i++){
+			if ( getEntryFromTabItem( items[i] ) == entry ){
+				
+				return( i );
+			}
+		}
+		return( -1 );
+	}
+		
+	private int
+	indexOf(
+		String		viewID )
+	{
+		CTabItem[] items = tabFolder.getItems();
+		
+		for ( int i=0;i<items.length;i++){
+			TabbedEntry entry = getEntryFromTabItem( items[i] );
+			
+			if ( entry != null && entry.getViewID().equals( viewID )){
+				
+				return( i );
+			}
+		}
+		return( -1 );
+	}
+	
 	protected TabbedEntry getEntryFromTabItem(CTabItem item) {
 		if (item.isDisposed()) {
 			return null;
@@ -1126,6 +1456,12 @@ public class TabbedMDI
 		return allowSubViews;
 	}
 
+	public boolean
+	isEmpty()
+	{
+		return( getEntries().length == 0 && ( mapUserClosedTabs == null || mapUserClosedTabs.isEmpty()));
+	}
+	
 	// @see com.biglybt.ui.swt.debug.ObfuscateImage#obfuscatedImage(org.eclipse.swt.graphics.Image)
 	@Override
 	public Image obfuscatedImage(Image image) {
