@@ -26,15 +26,22 @@ import java.net.URL;
 import java.util.*;
 
 import com.biglybt.core.CoreFactory;
+import com.biglybt.core.category.Category;
+import com.biglybt.core.category.CategoryManager;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.history.DownloadHistoryEvent;
 import com.biglybt.core.history.DownloadHistoryListener;
 import com.biglybt.core.history.DownloadHistoryManager;
 import com.biglybt.core.internat.MessageText;
+import com.biglybt.core.peermanager.control.*;
 import com.biglybt.core.tag.Tag;
 import com.biglybt.core.tag.TagManager;
 import com.biglybt.core.tag.TagManagerFactory;
+import com.biglybt.core.tag.TagManagerListener;
+import com.biglybt.core.tag.TagType;
+import com.biglybt.core.tag.TagTypeListener;
+import com.biglybt.core.tag.TagTypeListener.TagEvent;
 import com.biglybt.core.tracker.AllTrackersManager;
 import com.biglybt.core.tracker.AllTrackersManager.AllTrackers;
 import com.biglybt.core.tracker.AllTrackersManager.AllTrackersEvent;
@@ -51,6 +58,7 @@ import com.biglybt.plugin.net.buddy.swt.SBC_ChatOverview;
 import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfo;
+import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfo2;
 import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfoManager;
 import com.biglybt.ui.mdi.*;
 import com.biglybt.ui.swt.Utils;
@@ -80,9 +88,11 @@ public class MainMDISetup
 
 	private static ParameterListener configBetaEnabledListener;
 	private static TRHostListener trackerHostListener;
+	private static TRHostListener trackerHostListener2;
 	private static SB_Dashboard	sb_dashboard;
 	private static SB_Transfers sb_transfers;
 	private static ShareManagerListener shareManagerListener;
+	private static ShareManagerListener shareManagerListener2;
 	private static SB_Vuze sb_vuze;
 
 	public static Set<String>	hiddenTopLevelIDs = new HashSet<>();
@@ -162,10 +172,20 @@ public class MainMDISetup
 		//	  	new UISWTViewBuilderCore(StatsView.VIEW_ID, null, StatsView.class)
 		//	  	  .setParentEntryID(SIDEBAR_HEADER_PLUGINS));
 
-		mdi.registerEntry(StatsView.VIEW_ID, id -> mdi.createEntry(
-				new UISWTViewBuilderCore(id, null, StatsView.class).setParentEntryID(
+		mdi.registerEntry(
+			StatsView.VIEW_ID, 
+			id -> {
+				MdiEntry entry = 
+					mdi.createEntry(
+						new UISWTViewBuilderCore(id, null, StatsView.class).setParentEntryID(
 						MultipleDocumentInterface.SIDEBAR_HEADER_PLUGINS),
-				true));
+					true);
+				
+				entry.setImageLeftID("image.sidebar.stats2");
+				
+				return( entry );
+			}
+			);
 
 		mdi.registerEntry(SIDEBAR_SECTION_ALLPEERS,
 				id -> {
@@ -176,8 +196,77 @@ public class MainMDISetup
 							SB_Transfers.getSectionPosition(mdi, SIDEBAR_SECTION_ALLPEERS));
 					MdiEntry entry = mdi.createEntry(builder, true);
 
+					PeerControlScheduler scheduler = PeerControlSchedulerFactory.getSingleton(0);
+
+					ViewTitleInfo title_info =
+							new ViewTitleInfo()
+							{
+								@Override
+								public Object
+								getTitleInfoProperty(
+									int propertyID)
+								{
+									if ( propertyID == TITLE_INDICATOR_TEXT ){
+										
+										int[] counts = scheduler.getPeerCount();
+										
+										return( counts[0] + " | " + counts[1] );
+									}
+									
+									return( null );
+								}
+							};
+							
+					entry.setViewTitleInfo( title_info );
+
 					entry.setImageLeftID("image.sidebar.allpeers");
-					return entry;
+					
+					final TimerEventPeriodic	timer =
+							SimpleTimer.addPeriodicEvent(
+									"sb:allpeers",
+									1*1000,
+									new TimerEventPerformer()
+									{
+										private int last_count1 = -1;
+										private int last_count2 = -1;
+										
+										@Override
+										public void
+										perform(
+											TimerEvent event)
+										{
+											int[] counts = scheduler.getPeerCount();
+											
+											int c1 = counts[0];
+											int c2 = counts[1];
+											
+											if ( c1 != last_count1 || c2 != last_count2 ){
+											
+												last_count1 = c1;
+												last_count2 = c2;
+														
+												entry.redraw();
+
+												ViewTitleInfoManager.refreshTitleInfo( title_info );
+												
+												entry.redraw();
+											}
+										}
+									});
+
+					entry.addListener(
+						new MdiCloseListener() {
+
+							@Override
+							public void
+							mdiEntryClosed(
+								MdiEntry entry, boolean userClosed)
+							{
+								timer.cancel();
+							}
+						});
+
+					return( entry );
 				});
 
 		mdi.registerEntry(MultipleDocumentInterface.SIDEBAR_SECTION_LOGGER,
@@ -195,8 +284,106 @@ public class MainMDISetup
 								SIDEBAR_HEADER_TRANSFERS, id,
 								"tagsview", "{tags.view.heading}", null, null, true, 
 								SB_Transfers.getSectionPosition(mdi, MultipleDocumentInterface.SIDEBAR_SECTION_TAGS));
+						
 						entry.setImageLeftID("image.sidebar.tag-overview");
 						entry.setDefaultExpanded(true);
+
+						TagManager tm = TagManagerFactory.getTagManager();
+
+						ViewTitleInfo2 title_info =
+							new ViewTitleInfo2()
+							{
+								@Override
+								public Object
+								getTitleInfoProperty(
+									int propertyID)
+								{
+									if ( propertyID == TITLE_INDICATOR_TEXT ){
+										
+										int num = 0;
+										
+										for ( TagType tt: tm.getTagTypes()){
+											
+											num += tt.getTagCount();
+										}
+										
+										return( String.valueOf( num ));
+									}
+									
+									return( null );
+								}
+								
+								public void titleInfoLinked(MultipleDocumentInterface mdi, MdiEntry mdiEntry){}
+								
+								public MdiEntry	getLinkedMdiEntry(){
+									return( entry );
+								}
+							};
+
+						entry.setViewTitleInfo( title_info );
+												
+						TagTypeListener ttl = 
+							new TagTypeListener()
+							{
+								public void
+								tagTypeChanged(
+									TagType		tag_type )
+								{
+								}
+	
+								public void
+								tagEventOccurred(
+									TagEvent			event )
+								{
+									int type = event.getEventType();
+									
+									if ( type == TagEvent.ET_TAG_ADDED || type == TagEvent.ET_TAG_REMOVED ){
+										
+										ViewTitleInfoManager.refreshTitleInfo( title_info );
+									}
+								}
+
+							};
+							
+						TagManagerListener tml = 
+							new TagManagerListener()
+							{
+								public void
+								tagTypeAdded(
+									TagManager		manager,
+									TagType			tag_type )
+								{
+									tag_type.addTagTypeListener( ttl, false );
+										
+								}
+	
+								public void
+								tagTypeRemoved(
+									TagManager		manager,
+									TagType			tag_type )
+								{
+								}
+							};
+							
+						tm.addTagManagerListener( tml, true);
+												
+						entry.addListener(
+							new MdiCloseListener() {
+
+								@Override
+								public void
+								mdiEntryClosed(
+									MdiEntry entry, boolean userClosed)
+								{
+									tm.removeTagManagerListener( tml );
+									
+									for ( TagType tt: tm.getTagTypes()){
+										
+										tt.removeTagTypeListener( ttl );
+									}
+								}
+							});
+						
 						return entry;
 					}
 				});
@@ -275,7 +462,7 @@ public class MainMDISetup
 						MdiEntry mdi_entry = mdi.createEntryFromSkinRef(
 								MultipleDocumentInterface.SIDEBAR_HEADER_DISCOVERY,
 								MultipleDocumentInterface.SIDEBAR_SECTION_CHAT, "chatsview",
-								"{mdi.entry.chatsoverview}", title_info, null, true, null);
+								"{mdi.entry.chatsoverview}", title_info, null, true, MultipleDocumentInterface.SIDEBAR_SECTION_SUBSCRIPTIONS );
 
 						mdi_entry.setImageLeftID("image.sidebar.chat-overview");
 
@@ -651,19 +838,98 @@ public class MainMDISetup
 
 		mdi.registerEntry(MultipleDocumentInterface.SIDEBAR_SECTION_MY_SHARES,
 				id -> {
+					final ShareManager sm;
+					
+					{
+						ShareManager temp;
+						
+						try{
+							temp = CoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface().getShareManager();
+							
+						}catch( Throwable e ){
+							
+							temp = null;
+						}
+						
+						sm = temp;
+					}
+	
+					ViewTitleInfo title_info =
+							new ViewTitleInfo()
+							{
+								
+								@Override
+								public Object
+								getTitleInfoProperty(
+									int propertyID)
+								{
+									if ( propertyID == TITLE_INDICATOR_TEXT ){
+
+										return( sm==null?null:String.valueOf( sm.getShareCount()));
+									}
+
+									return null;
+								}
+							};
+
 					UISWTViewBuilderCore builder = new UISWTViewBuilderCore(id, null,
 							MySharesView.class).setParentEntryID(
 									SIDEBAR_HEADER_TRANSFERS).setPreferredAfterID(
 											SB_Transfers.getSectionPosition(mdi, id));
 					MdiEntry entry = mdi.createEntry(builder, true);
 
+					entry.setViewTitleInfo( title_info );
+					
 					entry.setImageLeftID("image.sidebar.myshares");
-
+						
+					if ( sm != null ){
+						
+						shareManagerListener2 = 
+								new ShareManagerListener()
+								{
+									public void
+									resourceAdded(
+										ShareResource		resource )
+									{
+										ViewTitleInfoManager.refreshTitleInfo( title_info );
+										entry.redraw();
+									}
+			
+									public void
+									resourceModified(
+										ShareResource		old_resource,
+										ShareResource		new_resource )
+									{
+									}
+			
+									public void
+									resourceDeleted(
+										ShareResource		resource )
+									{
+										ViewTitleInfoManager.refreshTitleInfo( title_info );
+										entry.redraw();
+									}
+			
+									public void
+									reportProgress(
+										int		percent_complete ){}
+			
+									public void
+									reportCurrentTask(
+										String	task_description ){}						
+								};
+								
+						sm.addListener( shareManagerListener2 );
+					}
+					
 					return entry;
 				});
 
 		mdi.registerEntry(MultipleDocumentInterface.SIDEBAR_SECTION_MY_TRACKER,
 				id -> {
+					
+					TRHost trackerHost = CoreFactory.getSingleton().getTrackerHost();
+					
 					UISWTViewBuilderCore builder = new UISWTViewBuilderCore(id, null,
 							MyTrackerView.class).setParentEntryID(
 									SIDEBAR_HEADER_TRANSFERS).setPreferredAfterID(
@@ -672,6 +938,55 @@ public class MainMDISetup
 
 					entry.setImageLeftID("image.sidebar.mytracker");
 
+					ViewTitleInfo title_info =
+							new ViewTitleInfo()
+							{
+								
+								@Override
+								public Object
+								getTitleInfoProperty(
+									int propertyID)
+								{
+									if ( propertyID == TITLE_INDICATOR_TEXT ){
+
+										return( String.valueOf( trackerHost.getTorrentCount()));
+									}
+
+									return null;
+								}
+							};
+					
+					entry.setViewTitleInfo( title_info );
+
+					trackerHostListener2 = new TRHostListener() {
+			
+						@Override
+						public void torrentRemoved(TRHostTorrent t) {
+							ViewTitleInfoManager.refreshTitleInfo( title_info );
+							entry.redraw();
+						}
+			
+						@Override
+						public void torrentChanged(TRHostTorrent t) {
+						}
+			
+						@Override
+						public void torrentAdded(TRHostTorrent t) {
+							ViewTitleInfoManager.refreshTitleInfo( title_info );
+							entry.redraw();
+						}
+			
+						@Override
+						public boolean handleExternalRequest(InetSocketAddress client_address,
+						                                     String user, String url, URL absolute_url, String header, InputStream is,
+						                                     OutputStream os, AsyncController async)
+								throws IOException {
+							return false;
+						}
+					};
+					
+					trackerHost.addListener(trackerHostListener2);
+					
 					return entry;
 				});
 
@@ -867,6 +1182,16 @@ public class MainMDISetup
 				}
 			});
 
+			menuItem = uim.getMenuManager().addMenuItem(
+					MenuManager.MENU_MENUBAR, "MainWindow.menu.view.allpeers");
+			menuItem.setDisposeWithUIDetach(UIInstance.UIT_SWT);
+			menuItem.addListener(new MenuItemListener() {
+				@Override
+				public void selected(MenuItem menu, Object target) {
+					UIFunctionsManager.getUIFunctions().getMDI().showEntryByID(
+							MultipleDocumentInterface.SIDEBAR_SECTION_ALLPEERS );
+				}
+			});
 		}
 
 		//		System.out.println("Activate sidebar " + startTab + " took "
@@ -894,44 +1219,8 @@ public class MainMDISetup
 					}
 				});
 
-		mdi.registerEntry("Tag\\..*", new MdiEntryCreationListener2() {
-
-			@Override
-			public MdiEntry createMDiEntry(MultipleDocumentInterface mdi, String id,
-			                               Object datasource, Map params) {
-
-				if (datasource instanceof Tag) {
-					Tag tag = (Tag) datasource;
-
-					return sb_transfers.setupTag(tag);
-
-				}else{
-
-					try{
-							// id format is "Tag." + tag.getTagType().getTagType() + "." + tag.getTagID();
-
-						TagManager tm = TagManagerFactory.getTagManager();
-
-						String[] bits = id.split( "\\." );
-
-						int	tag_type = Integer.parseInt( bits[1] );
-						int	tag_id	 = Integer.parseInt( bits[2] );
-
-						Tag tag = tm.getTagType( tag_type ).getTag( tag_id );
-
-						if ( tag != null ){
-
-							return sb_transfers.setupTag(tag);
-						}
-					}catch( Throwable e ){
-
-					}
-				}
-
-				return null;
-			}
-		});
-
+		setupCatsTags( mdi );
+		
 		sb_transfers = new SB_Transfers(mdi, false);
 		
 		SBC_ActivityTableView.setupSidebarEntry(mdi);
@@ -943,6 +1232,8 @@ public class MainMDISetup
 		
 		mdi.setPreferredOrder(preferredOrder);
 
+		setupCatsTags( mdi );
+		
 		sb_dashboard = new SB_Dashboard(mdi);
 		
 		for (int i = 0; i < preferredOrder.length; i++) {
@@ -1028,6 +1319,84 @@ public class MainMDISetup
 		mdi.loadEntryByID(MultipleDocumentInterface.SIDEBAR_SECTION_ACTIVITIES, false);
 	}
 
+	private static void
+	setupCatsTags(MultipleDocumentInterfaceSWT mdi )
+	{
+		mdi.registerEntry("Tag\\..*", new MdiEntryCreationListener2() {
+
+			@Override
+			public MdiEntry createMDiEntry(MultipleDocumentInterface mdi, String id,
+			                               Object datasource, Map params) {
+
+				if (datasource instanceof Tag) {
+					Tag tag = (Tag) datasource;
+
+					return sb_transfers.setupTag(tag);
+
+				}else{
+
+					try{
+							// id format is "Tag." + tag.getTagType().getTagType() + "." + tag.getTagID();
+
+						TagManager tm = TagManagerFactory.getTagManager();
+
+						String[] bits = id.split( "\\." );
+
+						int	tag_type = Integer.parseInt( bits[1] );
+						int	tag_id	 = Integer.parseInt( bits[2] );
+
+						Tag tag = tm.getTagType( tag_type ).getTag( tag_id );
+
+						if ( tag != null ){
+
+							return sb_transfers.setupTag(tag);
+						}
+					}catch( Throwable e ){
+
+					}
+				}
+
+				return null;
+			}
+		});
+
+		mdi.registerEntry("Cat\\..*", new MdiEntryCreationListener2() {
+
+			@Override
+			public MdiEntry createMDiEntry(MultipleDocumentInterface mdi, String id,
+			                               Object datasource, Map params) {
+
+				if (datasource instanceof Category) {
+
+					return sb_transfers.setupCategory((Category)datasource);
+
+				}else{
+
+					try{
+							// id format is "Cat." + base32 encoded name.getBytes();
+
+						TagManager tm = TagManagerFactory.getTagManager();
+
+						String[] bits = id.split( "\\." );
+
+						String cat_name = new String( Base32.decode( bits[1] ));
+						
+						Category cat = CategoryManager.getCategory( cat_name );
+
+						if ( cat != null ){
+
+							return sb_transfers.setupCategory(cat);
+						}
+					}catch( Throwable e ){
+
+					}
+				}
+
+				return null;
+			}
+		});
+	}
+	
 	public static SB_Transfers getSb_transfers() {
 		return sb_transfers;
 	}
@@ -1059,11 +1428,14 @@ public class MainMDISetup
 
 		if (trackerHostListener != null) {
 			TRHost trackerHost = CoreFactory.getSingleton().getTrackerHost();
-			if (trackerHost != null) {
-				trackerHost.removeListener(trackerHostListener);
-			}
+			trackerHost.removeListener(trackerHostListener);
 			trackerHostListener = null;
 		}
+		if (trackerHostListener2 != null) {
+			TRHost trackerHost = CoreFactory.getSingleton().getTrackerHost();
+			trackerHost.removeListener(trackerHostListener2);
+			trackerHostListener2 = null;
+		}
 
 		if (shareManagerListener != null) {
 			try {
@@ -1075,7 +1447,16 @@ public class MainMDISetup
 			} catch (ShareException ignore) {
 			}
 		}
-
+		if (shareManagerListener2 != null) {
+			try {
+				ShareManager share_manager = PluginInitializer.getDefaultInterface().getShareManager();
+				if (share_manager != null) {
+					share_manager.removeListener(shareManagerListener2);
+				}
+				shareManagerListener2 = null;
+			} catch (ShareException ignore) {
+			}
+		}
 		if (configBetaEnabledListener != null) {
 			COConfigurationManager.removeParameterListener(
 					"Beta Programme Enabled", configBetaEnabledListener);
