@@ -28,6 +28,7 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.FormLayout;
+import org.eclipse.swt.layout.RowLayout;
 import org.eclipse.swt.widgets.*;
 
 import com.biglybt.activities.ActivitiesManager;
@@ -86,6 +87,7 @@ import com.biglybt.ui.swt.uiupdater.UIUpdaterSWT;
 import com.biglybt.ui.swt.utils.DragDropUtils;
 import com.biglybt.ui.swt.utils.FontUtils;
 import com.biglybt.ui.swt.utils.SWTRunnable;
+import com.biglybt.ui.swt.views.QuickLinksView;
 import com.biglybt.ui.swt.views.skin.WelcomeView;
 import com.biglybt.ui.swt.views.skin.sidebar.SideBar;
 import com.biglybt.ui.swt.views.utils.LocProvUtils;
@@ -585,8 +587,13 @@ public class MainWindowImpl
 								COConfigurationManager.getBooleanParameter("System Tray Disabled Override"))){						
 
 						minimizeToTray(event);
-					} else {
-						event.doit = dispose(false, false);
+						
+					}else{
+						
+						dispose(false);
+						
+						event.doit = false;		// don't allow shell to close at this point, the disposal code 
+												// will close it when ready to
 					}
 				}
 
@@ -681,11 +688,11 @@ public class MainWindowImpl
 			//System.out.println("hooks init took " + (SystemTime.getCurrentTime() - startTime) + "ms");
 			//startTime = SystemTime.getCurrentTime();
 
-			initMDI();
+			BaseMDI mdi = initMDI();
 			System.out.println("skin widgets (1/2) init took "
 					+ (SystemTime.getCurrentTime() - startTime) + "ms");
 			startTime = SystemTime.getCurrentTime();
-			initWidgets2();
+			initWidgets2( mdi );
 
 			increaseProgress(uiInitializer, "v3.splash.initSkin");
 			System.out.println("skin widgets (2/2) init took "
@@ -1052,8 +1059,7 @@ public class MainWindowImpl
 
 	@Override
 	@SuppressWarnings("deprecation")
-	public boolean dispose(final boolean for_restart,
-			final boolean close_already_in_progress) {
+	public boolean dispose(final boolean for_restart ){
 		if (disposedOrDisposing) {
 			return true;
 		}
@@ -1061,13 +1067,13 @@ public class MainWindowImpl
 				new AERunnableBoolean() {
 					@Override
 					public boolean runSupport() {
-						return _dispose(for_restart, close_already_in_progress);
+						return _dispose(for_restart);
 					}
 				}, 0);
 		return b == null || b;
 	}
 
-	boolean _dispose(final boolean bForRestart, boolean bCloseAlreadyInProgress) {
+	boolean _dispose(final boolean bForRestart ) {
 		if (disposedOrDisposing) {
 			return true;
 		}
@@ -1088,7 +1094,7 @@ public class MainWindowImpl
 				public void runSupport() {
 					SWTThread instance = SWTThread.getInstance();
 					if (instance != null && !instance.isTerminated()) {
-						instance.getInitializer().stopIt(bForRestart, false);
+						instance.getInitializer().stopIt(bForRestart);
 					}
 				}
 			});
@@ -1625,16 +1631,18 @@ public class MainWindowImpl
 				uiSkinnableSWTListener);
 	}
 
-	private void initMDI() {
+	private BaseMDI initMDI() {
 		SWTSkinObject skinObject = skin.getSkinObject(SkinConstants.VIEWID_MDI);
 		if (null != skinObject) {
 			BaseMDI mdi = Utils.isAZ2UI() ? new TabbedMDI() : new SideBar();
 			mdi.buildMDI(skinObject);
 			MainMDISetup.setupSideBar(mdi);
+			return( mdi );
 		}
+		return( null );
 	}
 
-	private void initWidgets2() {
+	private void initWidgets2(BaseMDI mdi ) {
 		SWTSkinObject skinObject = skin.getSkinObject("statusbar");
 		if (skinObject != null) {
 			final Composite cArea = (Composite) skinObject.getControl();
@@ -1663,16 +1671,23 @@ public class MainWindowImpl
 			});
 		}
 
+		skinObject = skin.getSkinObject("quick-links");
+		
+		if ( skinObject != null && mdi != null ){
+			
+			QuickLinksView.init( mdi, skinObject );
+		}
+		
 		skinObject = skin.getSkinObject(SkinConstants.VIEWID_PLUGINBAR);
 		if (skinObject != null) {
 			Menu topbarMenu = new Menu(shell, SWT.POP_UP);
 
 			if (COConfigurationManager.getIntParameter("User Mode") > 1) {
 				MenuItem mi =
-					MainMenuV3.createViewMenuItem(skin, topbarMenu,
+					MainMenuV3.createPluginBarMenuItem(skin, topbarMenu,
 						"v3.MainWindow.menu.view." + SkinConstants.VIEWID_PLUGINBAR,
 						SkinConstants.VIEWID_PLUGINBAR + ".visible",
-						SkinConstants.VIEWID_PLUGINBAR, true, -1);
+						SkinConstants.VIEWID_PLUGINBAR );
 
 				if ( Utils.isAZ2UI()){
 
@@ -2001,7 +2016,7 @@ public class MainWindowImpl
 			});
 		}
 	}
-
+	
 	private void
 	runTest(
 		String	cmd )
@@ -2069,6 +2084,13 @@ public class MainWindowImpl
 				}
 				break;
 			}
+			case WINDOW_ELEMENT_QUICK_LINKS: {
+				SWTSkinObject skinObject = skin.getSkinObject(SkinConstants.VIEWID_QUICK_LINKS);
+				if (skinObject != null) {
+					return skinObject.isVisible();
+				}
+				break;
+			}
 			case WINDOW_ELEMENT_STATUSBAR:
 				//TODO:
 				break;
@@ -2085,12 +2107,18 @@ public class MainWindowImpl
 		switch (windowElement) {
 			case WINDOW_ELEMENT_TOOLBAR:
 				SWTSkinUtils.setVisibility(skin, "IconBar.enabled",
-						SkinConstants.VIEWID_TOOLBAR, value, true, true);
+						SkinConstants.VIEWID_TOOLBAR, value, true);
 				break;
 			case WINDOW_ELEMENT_TOPBAR:
 
 				SWTSkinUtils.setVisibility(skin, SkinConstants.VIEWID_PLUGINBAR
-						+ ".visible", SkinConstants.VIEWID_PLUGINBAR, value, true, true);
+						+ ".visible", SkinConstants.VIEWID_PLUGINBAR, value, true);
+
+				break;
+			case WINDOW_ELEMENT_QUICK_LINKS:
+
+				SWTSkinUtils.setVisibilityRelaxed(skin, SkinConstants.VIEWID_QUICK_LINKS
+						+ ".visible", SkinConstants.VIEWID_QUICK_LINKS, value, true);
 
 				break;
 			case WINDOW_ELEMENT_STATUSBAR:
@@ -2115,6 +2143,8 @@ public class MainWindowImpl
 					return skinObject.getControl().getBounds();
 				}
 
+				break;
+			case WINDOW_ELEMENT_QUICK_LINKS:
 				break;
 			case WINDOW_ELEMENT_STATUSBAR:
 
