@@ -35,6 +35,7 @@ import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.config.impl.ConfigurationDefaults;
 import com.biglybt.core.download.DownloadManager;
+import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.global.GlobalManager;
 import com.biglybt.core.global.GlobalManagerEvent;
 import com.biglybt.core.history.DownloadHistoryManager;
@@ -238,12 +239,11 @@ public class UIFunctionsImpl
 		}
 
 	}
-
-	// @see UIFunctions#dispose(boolean, boolean)
+	
 	@Override
-	public boolean dispose(boolean for_restart, boolean close_already_in_progress) {
+	public boolean dispose(boolean for_restart) {
 		try {
-			return mainWindow.dispose(for_restart, close_already_in_progress);
+			return mainWindow.dispose(for_restart );
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "Disposing MainWindow", e));
 		}
@@ -1097,6 +1097,25 @@ public class UIFunctionsImpl
 
 			if ( existingDownload != null ){
 
+					// if existing is metadata download then we need to remove it before continuing (use case is that user has a long running metadata download and in
+					// the meantime manually obtains the torrent from elsewhere and adds it
+				
+				if ( existingDownload.getDownloadState().getFlag( DownloadManagerState.FLAG_METADATA_DOWNLOAD )){
+					
+					try{
+						gm.removeDownloadManager( existingDownload, true, true );
+						
+						existingDownload = null;
+						
+					}catch( Throwable e){
+						
+						Debug.out( e );
+					}
+				}
+			}
+				
+			if ( existingDownload != null ){
+				
 				boolean delete_delegated = false;
 				
 				if ( !is_silent ){
@@ -1609,6 +1628,13 @@ public class UIFunctionsImpl
 		Utils.execSWTThread(() -> ManagerUtils.open(f));
 	}
 
+	@Override
+	public void showText(String title, String content){
+		Utils.execSWTThread(() ->{
+			new TextViewerWindow( title, null, content, false  );
+		});	
+	}	
+	
 	public void dispose() {
 		COConfigurationManager.removeParameterListener(
 				"show_torrents_menu", paramShowTorrentsMenuListener);
