@@ -25,6 +25,7 @@ import java.util.function.Function;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.*;
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
 
@@ -64,14 +65,18 @@ import com.biglybt.ui.swt.columns.searchsubs.*;
 import com.biglybt.ui.swt.columns.subscriptions.ColumnSubResultNew;
 import com.biglybt.ui.swt.imageloader.ImageLoader;
 import com.biglybt.ui.swt.mainwindow.ClipboardCopy;
+import com.biglybt.ui.swt.mainwindow.Colors;
 import com.biglybt.ui.swt.mdi.MultipleDocumentInterfaceSWT;
 import com.biglybt.ui.swt.search.SBC_SearchResultsView;
 import com.biglybt.ui.swt.skin.*;
 import com.biglybt.ui.swt.utils.SearchSubsUtils;
 import com.biglybt.ui.swt.views.skin.SkinView;
+import com.biglybt.ui.swt.views.table.TableRowSWT;
 import com.biglybt.ui.swt.views.table.TableViewSWT;
 import com.biglybt.ui.swt.views.table.TableViewSWTMenuFillListener;
 import com.biglybt.ui.swt.views.table.impl.TableViewFactory;
+import com.biglybt.ui.swt.views.table.utils.TableColumnCreator;
+import com.biglybt.ui.swt.views.tableitems.ColumnDateSizer;
 
 public class
 SBC_SubscriptionResultsView
@@ -82,6 +87,8 @@ SBC_SubscriptionResultsView
 
 	private static boolean columnsAdded = false;
 
+	private static TableRowSWT.ColorRequester colour_requester = ()->1;
+	
 	private TableViewSWT<SubscriptionResultFilterable> tv_subs_results;
 
 	private MdiEntry			mdi_entry;
@@ -729,6 +736,24 @@ SBC_SubscriptionResultsView
 						new ColumnSearchSubResultAge(column);
 					}
 				});
+		
+		tableManager.registerColumn(
+			SubscriptionResultFilterable.class,
+			ColumnSearchSubResultAssetDate.COLUMN_ID,
+			new TableColumnCoreCreationListener() {
+				@Override
+				public TableColumnCore createTableColumnCore(
+						Class<?> forDataSourceType, String tableID, String columnID) {
+					return new ColumnDateSizer(SubscriptionResultFilterable.class, columnID,
+							TableColumnCreator.DATE_COLUMN_WIDTH, tableID) {
+					};
+				}
+
+				@Override
+				public void tableColumnCreated(TableColumn column) {
+					new ColumnSearchSubResultAssetDate(column);
+				}
+			});
 
 		tableManager.registerColumn(
 			SubscriptionResultFilterable.class,
@@ -861,7 +886,7 @@ SBC_SubscriptionResultsView
 
 			tv_subs_results.processDataSourceQueueSync();
 
-			List<SubscriptionResultFilterable> existing_results = tv_subs_results.getDataSources( true );
+			Collection<SubscriptionResultFilterable> existing_results = tv_subs_results.getDataSources( true );
 
 			Map<String,SubscriptionResultFilterable>	existing_map = new HashMap<>();
 
@@ -1012,9 +1037,9 @@ SBC_SubscriptionResultsView
 				ColumnSearchSubResultAge.COLUMN_ID,
 				SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL );
 
-		TableColumnManager tableManager = TableColumnManager.getInstance();
-
-		tableManager.setDefaultColumnNames( TABLE_SR,
+		TableColumnManager tcm = TableColumnManager.getInstance();
+			
+		tcm.setDefaultColumnNames( TABLE_SR,
 				new String[] {
 					ColumnSubResultNew.COLUMN_ID,
 					ColumnSearchSubResultType.COLUMN_ID,
@@ -1024,19 +1049,23 @@ SBC_SubscriptionResultsView
 					ColumnSearchSubResultSeedsPeers.COLUMN_ID,
 					ColumnSearchSubResultRatings.COLUMN_ID,
 					ColumnSearchSubResultAge.COLUMN_ID,
+					ColumnSearchSubResultAssetDate.COLUMN_ID,
 					ColumnSearchSubResultRank.COLUMN_ID,
 					ColumnSearchSubResultCategory.COLUMN_ID,
 				});
-
-		tableManager.setDefaultSortColumnName(TABLE_SR, ColumnSearchSubResultAge.COLUMN_ID);
-
-		TableColumnCore tcc = tableManager.getTableColumnCore( TABLE_SR, ColumnSearchSubResultAge.COLUMN_ID );
-
-		if ( tcc != null ){
-
-			tcc.setDefaultSortAscending( true );
+	
+		if ( !tcm.hasTableColumnSettings( TABLE_SR )){
+			
+			tcm.setDefaultSortColumnName(TABLE_SR, ColumnSearchSubResultAge.COLUMN_ID);
+	
+			TableColumnCore tcc = tcm.getTableColumnCore( TABLE_SR, ColumnSearchSubResultAge.COLUMN_ID );
+	
+			if ( tcc != null ){
+	
+				tcc.setDefaultSortAscending( true );
+			}
 		}
-
+		
 		if (txtFilter != null) {
 			tv_subs_results.enableFilterCheck(txtFilter, this);
 		}
@@ -1125,6 +1154,33 @@ SBC_SubscriptionResultsView
 					}
 				}
 
+				TableRowCore	hash_row 	= null;
+				byte[] 			hash 		= null;
+				
+				if ( rows.length == 1 ){
+					
+					hash_row = rows[0];
+							
+					SubscriptionResultFilterable rc = (SubscriptionResultFilterable)hash_row.getDataSource();
+					
+					hash = rc.getHash();
+				}
+				
+				for ( TableRowCore row: tv_subs_results.getRows()){
+						
+					Color target = null;
+					
+					if ( row != hash_row && hash != null ){
+							
+						if ( Arrays.equals(((SubscriptionResultFilterable)row.getDataSource()).getHash(), hash )){
+			
+							target = Colors.blues[ Colors.BLUES_MIDLIGHT ];
+						}
+					}
+					
+					((TableRowSWT)row).requestBackgroundColor( colour_requester, target );
+				}
+				
 				ISelectedContent[] sels = valid.toArray( new ISelectedContent[valid.size()] );
 
 				SelectedContentManager.changeCurrentlySelectedContent("IconBarEnabler",
