@@ -58,6 +58,7 @@ import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfoManager;
 import com.biglybt.ui.mdi.MultipleDocumentInterface;
 import com.biglybt.ui.selectedcontent.SelectedContent;
 import com.biglybt.ui.selectedcontent.SelectedContentManager;
+import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.components.BubbleTextBox;
@@ -353,6 +354,8 @@ public class FilesView
 			return;
 		}
 		
+		force_refresh = true;
+		
 		if (tv != null) {
 	
 			for (DownloadManager manager: managers ){
@@ -369,6 +372,7 @@ public class FilesView
 			}
 		}
 
+		DownloadManager[] oldManagers = managers;
 		managers = newManagers;
 		tags = newTags;
 		datasource = newDataSource;
@@ -425,6 +429,9 @@ public class FilesView
 				tv.removeAllTableRows();
 				current_root = null;
 			} else {
+				if (oldManagers.length == 1 && newManagers.length == 1) {
+					tv.removeAllTableRows();
+				}
 				updateTable();
 			}
 			if ( disableTableWhenEmpty ){
@@ -896,12 +903,18 @@ public class FilesView
 
 			List<FilesView.FilesViewNodeInner>	inners = new ArrayList<>();
 			
+			List<DiskManagerFileInfo> inner_files = new ArrayList<>();
+			
 			for ( int i=0;i<data_sources.length;i++ ){
 				DiskManagerFileInfo file = (DiskManagerFileInfo)data_sources[i];
 				
 				if ( file instanceof FilesView.FilesViewNodeInner ){
+
+					FilesView.FilesViewNodeInner inner = (FilesView.FilesViewNodeInner )file;
 					
-					inners.add((FilesView.FilesViewNodeInner )file);
+					inner_files.addAll( inner.getFiles( true ));
+					
+					inners.add( inner );
 					
 				}else{
 				
@@ -917,6 +930,7 @@ public class FilesView
 					menu,
 					new DownloadManager[]{ managers[0] },
 					new DiskManagerFileInfo[][]{ files.toArray(new DiskManagerFileInfo[files.size()]) },
+					false,
 					false );
 				
 			}else if ( !inners.isEmpty()){
@@ -946,6 +960,20 @@ public class FilesView
 				mi.setEnabled( doTreeAction( inners, 1, true ));
 				
 				new MenuItem( menu, SWT.SEPARATOR );
+				
+				if ( !inner_files.isEmpty()){
+					
+					FilesViewMenuUtil.fillMenu(
+							tv,
+							sColumnName,
+							menu,
+							new DownloadManager[]{ managers[0] },
+							new DiskManagerFileInfo[][]{ inner_files.toArray(new DiskManagerFileInfo[inner_files.size()]) },
+							false,
+							true );
+									
+					MenuBuildUtils.addSeparater( menu );
+				}
 			}
 		}else{
 
@@ -960,6 +988,7 @@ public class FilesView
 				DiskManagerFileInfo file = (DiskManagerFileInfo)ds;
 
 				if ( file instanceof FilesView.FilesViewNodeInner ){
+					
 					continue;
 				}
 				
@@ -991,7 +1020,8 @@ public class FilesView
 			}
 
 			if ( files_list.length > 0 ){
-				FilesViewMenuUtil.fillMenu(tv, sColumnName, menu, manager_list, files_list, true );
+				
+				FilesViewMenuUtil.fillMenu(tv, sColumnName, menu, manager_list, files_list, true, false );
 			}
 		}
 	}
@@ -1376,7 +1406,7 @@ public class FilesView
 			
 			to_select = null;
 		}
-		
+				
 		if ( !tree_view ){
 			
 			updateFlatView( sync );
@@ -1477,6 +1507,8 @@ public class FilesView
 	{
 		int	num_managers = managers.length;
 		
+		boolean	wait_until_idle = false;
+		
 		if ( num_managers == 0 ){
 			
 			if ( tv.getRowCount() > 0 ){
@@ -1497,6 +1529,8 @@ public class FilesView
 			
 		}else if ( num_managers == 1 ){
 			
+			wait_until_idle = force_refresh;
+			
 			DownloadManager dm =  managers[0];
 			
 			if ( force_refresh || current_root == null || current_root.dm != dm ||  tv.getRowCount() == 0 ){
@@ -1594,6 +1628,8 @@ public class FilesView
 		
 			if ( force_refresh || current_root == null || tv.getRowCount() == 0 ){
 				
+				wait_until_idle = force_refresh;
+				
 				current_root = new FilesViewNodeInner( null, MessageText.getString( "label.downloads" ), null );
 				
 				tree_file_map.clear();
@@ -1668,6 +1704,33 @@ public class FilesView
 				tv.processDataSourceQueueSync();
 			}
 		}
+		
+		if ( wait_until_idle ){
+			
+			AEThread2.createAndStartDaemon(
+					"TableSorter",
+					()->{
+						long	start = SystemTime.getMonotonousTime();
+						
+							// this has the added bonus of forcing a sort which we particularly need if the user
+							// has disabled automatic resorts - we want the initial view to at least be sorted
+						
+						while( tv.hasChangesPending()){
+						
+							if ( SystemTime.getMonotonousTime() - start > 5000 ){
+								
+								return;
+							}
+							
+							try{
+								Thread.sleep( 100 );
+								
+							}catch( Throwable e ){
+								
+							}
+						}
+					});	
+		}
 	}
 	
 	private static Comparator<String> tree_comp = new FormattersImpl().getAlphanumericComparator( true );
@@ -1789,6 +1852,38 @@ public class FilesView
 			kids.put( child.getName(), child );
 		}
 
+		protected List<DiskManagerFileInfo>
+		getFiles(
+			boolean	recursive )
+		{
+			List<DiskManagerFileInfo> result = new ArrayList<>();
+			
+			getFiles( result, recursive );
+			
+			return( result );
+		}
+		
+		private void
+		getFiles(
+			List<DiskManagerFileInfo>		files,
+			boolean							recursive )
+		{
+			for ( FilesViewTreeNode kid: kids.values()){
+				
+				if ( kid.isLeaf()){
+					
+					files.add(((FilesViewNodeLeaf)kid).getTarget());
+					
+				}else{
+					
+					if ( recursive ){
+						
+						((FilesViewNodeInner)kid).getFiles( files, recursive );
+					}
+				}
+			}
+		}
+		
 		@Override
 		public String
 		getName()
@@ -2485,35 +2580,72 @@ public class FilesView
 	updateFlatView(
 		boolean		sync )
 	{
-	    List<DiskManagerFileInfo> files = getFileInfo();
-	    
-	    if (files.size() == 0) {
-	    	tv.removeAllTableRows();
-	    } else {
-	    	this.force_refresh = false;
+		if (!force_refresh) {
+			return;
+		}
+		if (!sync && Utils.isSWTThread()) {
+			Utils.getOffOfSWTThread(() -> updateFlatView(false));
+			return;
+		}
+		force_refresh = false;
 
-		    List<DiskManagerFileInfo> toRemove = tv.getDataSources(true);
-		    List<DiskManagerFileInfo> toAdd = new ArrayList<>();
-		    for (DiskManagerFileInfo info : files) {
-		    	if (toRemove.contains(info)) {
-		    		toRemove.remove(info);
-			    } else {
-		    		toAdd.add(info);
-			    }
-		    }
-	      tv.removeDataSources(toRemove.toArray(new DiskManagerFileInfo[toRemove.size()]));
-	      tv.addDataSources(toAdd.toArray(new DiskManagerFileInfo[toAdd.size()]));
-	      tv.tableInvalidate();
+		List<DiskManagerFileInfo> files = getFileInfo();
 
-	    	if ( sync ){
-	    		
-	    		tv.processDataSourceQueueSync();
-	    		
-	    	}else{
-		    
-	    		tv.processDataSourceQueue();
-	    	}
-	    }
+		if (files.size() == 0) {
+			tv.removeAllTableRows();
+		} else {
+			Collection<DiskManagerFileInfo> toRemove = tv.getDataSources(true);
+			// If we are removing a lot of rows, it's faster to clear and add 
+			if (toRemove.size() - files.size() > 50000) {
+				// todo: restore selectedRows
+				tv.removeAllTableRows();
+				toRemove = new HashSet<>();
+			}
+			List<DiskManagerFileInfo> toAdd = new ArrayList<>();
+			for (DiskManagerFileInfo info : files) {
+				if (toRemove.contains(info)) {
+					toRemove.remove(info);
+				} else {
+					toAdd.add(info);
+				}
+			}
+			tv.removeDataSources(toRemove.toArray(new DiskManagerFileInfo[0]));
+			tv.addDataSources(toAdd.toArray(new DiskManagerFileInfo[0]));
+			tv.tableInvalidate();
+
+			if ( sync ){
+
+				tv.processDataSourceQueueSync();
+
+			}else{
+
+				tv.processDataSourceQueue();
+			}
+			
+			AEThread2.createAndStartDaemon(
+				"TableSorter",
+				()->{
+					long	start = SystemTime.getMonotonousTime();
+					
+						// this has the added bonus of forcing a sort which we particularly need if the user
+						// has disabled automatic resorts - we want the initial view to at least be sorted
+					
+					while( tv.hasChangesPending()){
+					
+						if ( SystemTime.getMonotonousTime() - start > 5000 ){
+							
+							return;
+						}
+						
+						try{
+							Thread.sleep( 100 );
+							
+						}catch( Throwable e ){
+							
+						}
+					}
+				});
+		}
 	}
 	
 	private List<DiskManagerFileInfo>
