@@ -64,12 +64,14 @@ import com.biglybt.ui.mdi.MultipleDocumentInterface;
 import com.biglybt.ui.selectedcontent.SelectedContent;
 import com.biglybt.ui.selectedcontent.SelectedContentManager;
 import com.biglybt.ui.swt.*;
+import com.biglybt.ui.swt.mainwindow.Colors;
 import com.biglybt.ui.swt.mainwindow.TorrentOpener;
 import com.biglybt.ui.swt.mdi.MdiEntrySWT;
 import com.biglybt.ui.swt.minibar.DownloadBar;
 import com.biglybt.ui.swt.pif.UISWTViewEvent;
 import com.biglybt.ui.swt.pifimpl.UISWTViewBuilderCore;
 import com.biglybt.ui.swt.pifimpl.UISWTViewCore;
+import com.biglybt.ui.swt.utils.ColorCache;
 import com.biglybt.ui.swt.utils.DragDropUtils;
 import com.biglybt.ui.swt.utils.FontUtils;
 import com.biglybt.ui.swt.utils.SWTRunnable;
@@ -500,7 +502,7 @@ public class MyTorrentsView
 							int active 	= 0;
 							int	queued	= 0;
 							
-							List<DownloadManager> dms = tv.getDataSources();
+							Collection<DownloadManager> dms = tv.getDataSources();
 							
 							for ( DownloadManager dm: dms ){
 								
@@ -539,6 +541,7 @@ public class MyTorrentsView
 						"Library.ShowTagButtons", 
 						"Library.ShowTagButtons.CompOnly",
 						"Library.ShowTagButtons.FiltersOnly",
+						"Library.ShowTagButtons.ImageOverride",
 						"Library.ShowTagButtons.Inclusive",
 					}, this);
 	
@@ -586,9 +589,15 @@ public class MyTorrentsView
 
 	  tableHeaderMenu = new Menu(control.getShell(), SWT.POP_UP );
 
+	  MenuItem	showItem = new MenuItem( tableHeaderMenu, SWT.CASCADE );
+	  Messages.setLanguageText( showItem, "ConfigView.label.ui_switcher_button" );
+	  Menu showMenu = new Menu( tableHeaderMenu );
+	  
+	  showItem.setMenu( showMenu );
+	  
 	  // show uptime
 
-	  final MenuItem menuItemShowUptime = new MenuItem(tableHeaderMenu, SWT.CHECK);
+	  final MenuItem menuItemShowUptime = new MenuItem(showMenu, SWT.CHECK);
 	  Messages.setLanguageText( menuItemShowUptime, "ConfigView.label.showuptime" );
 
 	  menuItemShowUptime.addSelectionListener(new SelectionAdapter() {
@@ -601,7 +610,7 @@ public class MyTorrentsView
 
 	  // selected download rates
 
-	  final MenuItem menuItemShowRates = new MenuItem(tableHeaderMenu, SWT.CHECK);
+	  final MenuItem menuItemShowRates = new MenuItem(showMenu, SWT.CHECK);
 	  Messages.setLanguageText( menuItemShowRates, "label.show.selected.rates" );
 
 	  menuItemShowRates.addSelectionListener(new SelectionAdapter() {
@@ -613,7 +622,7 @@ public class MyTorrentsView
 	  });
 	  // show category buttons
 
-	  final MenuItem menuItemShowCatBut = new MenuItem(tableHeaderMenu, SWT.CHECK);
+	  final MenuItem menuItemShowCatBut = new MenuItem(showMenu, SWT.CHECK);
 	  Messages.setLanguageText( menuItemShowCatBut, "ConfigView.label.show.cat.but" );
 
 	  menuItemShowCatBut.addSelectionListener(new SelectionAdapter() {
@@ -627,7 +636,7 @@ public class MyTorrentsView
 
 	  // show tag buttons
 
-	  final MenuItem menuItemShowTagBut = new MenuItem(tableHeaderMenu, SWT.CHECK);
+	  final MenuItem menuItemShowTagBut = new MenuItem(showMenu, SWT.CHECK);
 	  Messages.setLanguageText( menuItemShowTagBut, "ConfigView.label.show.tag.but" );
 
 	  menuItemShowTagBut.addSelectionListener(new SelectionAdapter() {
@@ -638,6 +647,40 @@ public class MyTorrentsView
 		  }
 	  });
 
+	  
+	  // search
+	  
+	  MenuItem	searchItem = new MenuItem( tableHeaderMenu, SWT.CASCADE );
+	  Messages.setLanguageText( searchItem, "LoggerView.filter" );
+	  Menu searchMenu = new Menu( tableHeaderMenu );
+	  
+	  searchItem.setMenu( searchMenu );
+	  
+	  MenuItem searchActive = new MenuItem( searchMenu, SWT.PUSH );
+	  Messages.setLanguageText(searchActive, "dialog.active.color" );
+	  searchActive.addListener( SWT.Selection, (ev)->{
+		  RGB res = Utils.showColorDialog( control, Utils.getConfigColor( "table.filter.active.colour", Colors.fadedBlue ).getRGB());
+
+		  if ( res != null ){
+
+			  Utils.setConfigColor( "table.filter.active.colour", ColorCache.getColor( control.getShell().getDisplay(), res ) );
+		  }
+	  });
+	  
+
+	  final MenuItem searchHistoryEnable = new MenuItem(searchMenu, SWT.CHECK);
+	  Messages.setLanguageText( searchHistoryEnable, "label.enable.history" );
+
+	  searchHistoryEnable.addSelectionListener(new SelectionAdapter() {
+		  @Override
+		  public void widgetSelected(SelectionEvent e) {
+			  COConfigurationManager.setParameter(
+					  "table.filter.history.enabled", searchHistoryEnable.getSelection());
+		  }
+	  });
+	  
+	  
+	  
 	  new MenuItem( tableHeaderMenu, SWT.SEPARATOR );
 
 	  	// enable simple views
@@ -670,6 +713,8 @@ public class MyTorrentsView
 			  menuItemShowCatBut.setEnabled( !neverShowCatButtons );
 			  menuItemShowTagBut.setEnabled( !neverShowTagButtons );
 
+			  searchHistoryEnable.setSelection(COConfigurationManager.getBooleanParameter( "table.filter.history.enabled", true ));
+
 			  menuEnableSimple.setSelection(COConfigurationManager.getBooleanParameter( "Library.EnableSimpleView" ));
 
 		  }
@@ -719,12 +764,13 @@ public class MyTorrentsView
 			    TorrentUtils.removePotentialTorrentDeletionListener( this );
 			    globalManager.removeListener(this);
 			    globalManager.removeEventListener( gm_event_listener );
-				  COConfigurationManager.removeParameterListener("DND Always In Incomplete", this);
-				  COConfigurationManager.removeParameterListener("User Mode", this);
+				COConfigurationManager.removeParameterListener("DND Always In Incomplete", this);
+				COConfigurationManager.removeParameterListener("User Mode", this);
 			    COConfigurationManager.removeParameterListener("Library.ShowCatButtons", this);
 			    COConfigurationManager.removeParameterListener("Library.ShowTagButtons", this);
 			    COConfigurationManager.removeParameterListener("Library.ShowTagButtons.CompOnly", this);
 			    COConfigurationManager.removeParameterListener("Library.ShowTagButtons.FiltersOnly", this);
+			    COConfigurationManager.removeParameterListener("Library.ShowTagButtons.ImageOverride", this );
 			    COConfigurationManager.removeParameterListener("Library.ShowTagButtons.Inclusive", this);
        		}));
   }
@@ -1106,7 +1152,7 @@ public class MyTorrentsView
 
 			TagCanvas button = new TagCanvas(cCategoriesAndTags, tag, false, true);
 			button.setTrigger(buttonListener);
-			button.setCompact(true);
+			button.setCompact(true,COConfigurationManager.getBooleanParameter( "Library.ShowTagButtons.ImageOverride" ));
 
 			if (isCat) {
   			if (spacer == null) {
@@ -1802,7 +1848,8 @@ public class MyTorrentsView
   					menu,
   					new DownloadManager[]{ firstFileDM },
   					new DiskManagerFileInfo[][]{ listFileInfos.toArray(new DiskManagerFileInfo[0])},
-  					false );
+  					false,
+  					false);
   			return;
   		}
 		}
@@ -2455,6 +2502,7 @@ public class MyTorrentsView
 			if ( 	parameterName.equals("Library.ShowCatButtons") ||
 					parameterName.equals("Library.ShowTagButtons" ) ||
 					parameterName.equals("Library.ShowTagButtons.FiltersOnly" ) ||
+					parameterName.equals("Library.ShowTagButtons.ImageOverride" ) ||
 					parameterName.equals("Library.ShowTagButtons.CompOnly" )){
 
 				createTabs();
@@ -3007,7 +3055,7 @@ public class MyTorrentsView
 	{
 		// request to fully resync this tag
 
-		List<DownloadManager> dataSources = tv.getDataSources();
+		Collection<DownloadManager> dataSources = tv.getDataSources();
 
 		for ( DownloadManager dm : dataSources ){
 
@@ -3105,6 +3153,31 @@ public class MyTorrentsView
     				removed_while_selected.put( dm.getInternalName(), "" );
     			}
     		}
+    		
+    		TableRowCore[] selected_rows = tv.getSelectedRows();
+    		
+    		if ( selected_rows.length == 1 && selected_rows[0] == row ){
+    			
+    			// this is going to remove the selection completely so select existing appropriate row
+    			// logic should probably be in the table itself but for the moment fix here
+    			
+    			TableRowCore[] rows = tv.getRows();
+    			
+    			for ( int i=0;i<rows.length;i++){
+    				
+    				if ( rows[i] == row ){
+    					
+    					if ( i < rows.length - 1 ){
+    						
+    						tv.setSelectedRows( new TableRowCore[]{ rows[i+1 ] });
+    						
+    					}else if ( i > 0 ){
+    						
+    						tv.setSelectedRows( new TableRowCore[]{ rows[i-1 ] });
+    					}
+    				}
+    			}
+    		}
     	}
     }
     
