@@ -16,6 +16,7 @@ import org.eclipse.swt.widgets.*;
 
 import com.biglybt.core.CoreFactory;
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.global.GlobalManager;
 import com.biglybt.core.internat.MessageText;
@@ -129,11 +130,13 @@ PeersViewBase
 	protected Shell shell;
 
 	private boolean				swarm_view_enable;
+	private boolean				local_peer_enable = true;
 	
 	private PeersGraphicView 	swarm_view;
 	private Set<PEPeer>			swarm_peers = new HashSet<>();
 	private volatile boolean	peers_changed;
 
+	private volatile boolean	show_local_peer;
 
 	protected
 	PeersViewBase(
@@ -224,12 +227,86 @@ PeersViewBase
 			
 			return tableParent;
 			
+		}else if ( local_peer_enable ){
+			
+			Composite parent = new Composite(composite, SWT.NONE);
+			GridLayout layout = new GridLayout(1,true);
+			layout.marginHeight = layout.marginWidth = 0;
+			layout.horizontalSpacing = layout.verticalSpacing = 0;
+			parent.setLayout(layout);
+	
+			Layout compositeLayout = composite.getLayout();
+			if (compositeLayout instanceof GridLayout) {
+				parent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
+			} else if (compositeLayout instanceof FormLayout) {
+				parent.setLayoutData(Utils.getFilledFormData());
+			}
+			
+			Composite header = new Composite(parent, SWT.NONE);
+			layout = new GridLayout(1,true);
+			layout.marginHeight = layout.marginWidth = 0;
+			header.setLayout(layout);
+			
+			header.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
+
+			Button lp_enable = new Button( header, SWT.CHECK );
+			lp_enable.setLayoutData(new GridData( GridData.FILL_HORIZONTAL ));
+			
+			Messages.setLanguageText( lp_enable, "label.local.peer.show" );
+			lp_enable.addListener( SWT.Selection, (ev)->{
+				COConfigurationManager.setParameter( "Peers View Show Local Peer", lp_enable.getSelection());
+			});
+			
+			COConfigurationManager.addAndFireParameterListener(
+				"Peers View Show Local Peer",
+				new ParameterListener(){
+					public void
+					parameterChanged(
+						String n )
+					{
+						if ( lp_enable.isDisposed()){
+							
+							COConfigurationManager.removeParameterListener( n, this );
+							
+							return;
+						}
+						
+						boolean enabled = COConfigurationManager.getBooleanParameter( n );
+						
+						lp_enable.setSelection( enabled );
+						
+						setShowLocalPeer( enabled );
+					}
+				});
+			
+			Composite tableParent = new Composite(parent, SWT.NONE);
+			
+			tableParent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
+			layout = new GridLayout();
+			layout.horizontalSpacing = layout.verticalSpacing = 0;
+			layout.marginHeight = layout.marginWidth = 0;
+			tableParent.setLayout(layout);
+
+			return( tableParent );
+			
 		}else {
 			
 			return( super.initComposite(composite));
 		}
 	}
 
+	protected boolean
+	getShowLocalPeer()
+	{
+		return( show_local_peer );
+	}
+	
+	protected void
+	setShowLocalPeer(
+		boolean		b )
+	{	
+		show_local_peer = b;
+	}
 	
 	protected TableViewSWT<PEPeer> 
 	initYourTableView(
@@ -383,15 +460,13 @@ PeersViewBase
 						
 						GlobalManager gm = CoreFactory.getSingleton().getGlobalManager();
 						
-						List<PEPeer> peers = tv.getDataSources();
-						
-						swarm_peers = new HashSet<>( peers );
+						swarm_peers = tv.getDataSources();
 						
 						final Map<PEPeerManager,int[]> done_pms = new HashMap<>();
 						
 						List<DownloadManager>	dms = new ArrayList<>();
 						
-						for ( PEPeer peer: peers ){
+						for ( PEPeer peer: swarm_peers ){
 							
 							PEPeerManager pm = peer.getManager();
 							
@@ -527,12 +602,17 @@ PeersViewBase
 		long	upSpeedSetMax		= 0;
 		long	maxUp				= 0;
 
+		boolean onlyMyPeer = true;
+		
 		if ( hasSelection ){
 			GlobalManager gm = CoreFactory.getSingleton().getGlobalManager();
 
 			for (int i = 0; i < peers.length; i++) {
 				PEPeer peer = peers[i];
 
+				if ( !peer.isMyPeer()){
+					onlyMyPeer = false;
+				}
 				PEPeerManager m = peer.getManager();
 
 				if ( m != null ){
@@ -595,7 +675,7 @@ PeersViewBase
 			final MenuItem block_item = new MenuItem(menu, SWT.CHECK);
 			PEPeer peer = peers.length==0?null:peers[0];
 
-			if ( peer == null || peer.getManager().getDiskManager().getRemainingExcludingDND() > 0 ){
+			if ( onlyMyPeer || peer == null || peer.getManager().getDiskManager().getRemainingExcludingDND() > 0 ){
 				// disallow peer upload blocking when downloading
 				block_item.setSelection(false);
 				block_item.setEnabled(false);
@@ -653,7 +733,12 @@ PeersViewBase
 			boolean has_pb_potential = false;
 			
 			for ( PEPeer peer: peers ){
-														
+						
+				if ( peer.isMyPeer()){
+					
+					continue;
+				}
+				
 				DownloadManager dm = peer_dm_map.get( peer );
 				
 				Peer p_peer = PluginCoreUtils.wrap( peer );
@@ -767,9 +852,13 @@ PeersViewBase
 		kick_item.addListener(SWT.Selection, new PeersRunner(peers) {
 			@Override
 			public void run(PEPeer peer) {
-				peer.getManager().removePeer(peer,"Peer kicked" );
+				if ( !peer.isMyPeer()){
+					peer.getManager().removePeer(peer,"Peer kicked" );
+				}
 			}
 		});
+		
+		kick_item.setEnabled( !onlyMyPeer );
 
 		final MenuItem ban_item = new MenuItem(menu, SWT.PUSH);
 
@@ -777,13 +866,17 @@ PeersViewBase
 		ban_item.addListener(SWT.Selection, new PeersRunner(peers) {
 			@Override
 			public void run(PEPeer peer) {
-				String msg = MessageText.getString("PeersView.menu.kickandban.reason");
-				IpFilterManagerFactory.getSingleton().getIPFilter().ban(peer.getIp(),
-						msg, true );
-				peer.getManager().removePeer(peer, "Peer kicked and banned");
+				if ( !peer.isMyPeer()){
+					String msg = MessageText.getString("PeersView.menu.kickandban.reason");
+					IpFilterManagerFactory.getSingleton().getIPFilter().ban(peer.getIp(),
+							msg, true );
+					peer.getManager().removePeer(peer, "Peer kicked and banned");
+				}
 			}
 		});
 
+		ban_item.setEnabled( !onlyMyPeer );
+
 		final MenuItem ban_for_item = new MenuItem(menu, SWT.PUSH);
 
 		Messages.setLanguageText(ban_for_item, "PeersView.menu.kickandbanfor");
@@ -845,11 +938,14 @@ PeersViewBase
 
 						for ( PEPeer peer: peers ){
 
-							String msg = MessageText.getString("PeersView.menu.kickandbanfor.reason", new String[]{ String.valueOf( mins )});
-
-							filter.ban( peer.getIp(), msg, true, mins );
-
-							peer.getManager().removePeer(peer, "Peer kicked and banned");
+							if ( !peer.isMyPeer()){
+								
+								String msg = MessageText.getString("PeersView.menu.kickandbanfor.reason", new String[]{ String.valueOf( mins )});
+	
+								filter.ban( peer.getIp(), msg, true, mins );
+	
+								peer.getManager().removePeer(peer, "Peer kicked and banned");
+							}
 						}
 					}
 				});
@@ -858,6 +954,8 @@ PeersViewBase
 			}
 		});
 		
+		ban_for_item.setEnabled( !onlyMyPeer );
+
 		addPeersMenu( download_specific, "", menu );
 	}
 	
