@@ -42,7 +42,6 @@ import com.biglybt.core.util.AERunnable;
 import com.biglybt.core.util.Debug;
 import com.biglybt.core.util.DisplayFormatters;
 import com.biglybt.core.util.GeneralUtils;
-import com.biglybt.ui.config.ConfigSectionSecurity;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.config.*;
@@ -62,6 +61,8 @@ import com.biglybt.ui.swt.utils.FontUtils;
 public class TagSettingsView
 	implements UISWTViewCoreEventListener, TagTypeListener
 {
+	public static final String VIEW_ID = "TagSettingsView";
+	
 	private static final String CM_ADD_REMOVE 	= "am=0;";
 	private static final String CM_ADD_ONLY	 	= "am=1;";
 	private static final String CM_REMOVE_ONLY	= "am=2;";
@@ -73,7 +74,7 @@ public class TagSettingsView
 
 	private Composite cMainComposite;
 
-	private ScrolledComposite sc;
+	//private ScrolledComposite sc;
 
 	private Tag[] tags;
 
@@ -127,6 +128,11 @@ public class TagSettingsView
 		public BooleanSwtParameter		moveOnRemoveData;
 		public BooleanSwtParameter		moveOnRemoveTorrent;
 
+		public folderOption 		moveOnAssignFolder;
+		public BooleanSwtParameter	moveOnAssignData;
+		public BooleanSwtParameter	moveOnAssignTorrent;
+
+
 		
 		public Text		 	constraints;
 		public Label		constraintError;
@@ -239,28 +245,13 @@ public class TagSettingsView
 			return;
 		}
 
-		if (cMainComposite == null || cMainComposite.isDisposed()) {
-			if (parent == null || parent.isDisposed()) {
-				return;
-			}
-			sc = new ScrolledComposite(parent, SWT.V_SCROLL);
-			sc.setExpandHorizontal(true);
-			sc.setExpandVertical(true);
-			sc.getVerticalBar().setIncrement(16);
-			Layout parentLayout = parent.getLayout();
-			if (parentLayout instanceof GridLayout) {
-				GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
-				sc.setLayoutData(gd);
-			} else if (parentLayout instanceof FormLayout) {
-				sc.setLayoutData(Utils.getFilledFormData());
-			}
-
-			cMainComposite = new Composite(sc, SWT.NONE);
-
-			sc.setContent(cMainComposite);
-		} else {
-			Utils.disposeComposite(cMainComposite, false);
+		if (parent == null || parent.isDisposed()) {
+			return;
 		}
+			
+		Utils.disposeComposite(parent, false);
+
+		cMainComposite = Utils.createScrolledComposite( parent, SWT.V_SCROLL | SWT.H_SCROLL );
 
 		if (tags == null) {
 			params = null;
@@ -947,7 +938,8 @@ public class TagSettingsView
 				if (	fl.supportsTagCopyOnComplete() || 
 						fl.supportsTagInitialSaveFolder() || 
 						fl.supportsTagMoveOnComplete() ||
-						fl.supportsTagMoveOnRemove()) {
+						fl.supportsTagMoveOnRemove() ||
+						fl.supportsTagMoveOnAssign()) {
 
 					Group gFiles = new Group(cMainComposite, SWT.NONE);
 					gFiles.setText(MessageText.getString( "label.file.settings"));
@@ -1253,6 +1245,79 @@ public class TagSettingsView
 									}
 								});
 					}
+					
+					if ( fl.supportsTagMoveOnAssign()){
+
+						params.moveOnAssignFolder =
+							new folderOption(gFiles,
+								"label.move.on.assign")
+							{
+								@Override
+								public void setFolder(File folder)
+								{
+									params.moveOnAssignData.setEnabled( folder != null );
+									params.moveOnAssignTorrent.setEnabled( folder != null );
+															fl.setTagMoveOnAssignFolder(folder);
+								}
+
+								@Override
+								public File getFolder() {
+									File result = fl.getTagMoveOnAssignFolder();
+									params.moveOnAssignData.setEnabled( result != null );
+									params.moveOnAssignTorrent.setEnabled( result != null );
+															return( result );
+								}
+							};
+
+						params.moveOnAssignData = new BooleanSwtParameter(gFiles,
+								"tag.moveOnAssignData", "label.move.data", null,
+								new BooleanSwtParameter.ValueProcessor() {
+									@Override
+									public Boolean getValue(BooleanSwtParameter p) {
+										return ((fl.getTagMoveOnAssignOptions()
+												& TagFeatureFileLocation.FL_DATA) != 0);
+									}
+
+									@Override
+									public boolean setValue(BooleanSwtParameter p, Boolean value) {
+										long flags = fl.getTagMoveOnAssignOptions();
+										if ( value ){
+											flags |= TagFeatureFileLocation.FL_DATA;
+										}else{
+											flags &= ~TagFeatureFileLocation.FL_DATA;
+										}
+										if (fl.getTagMoveOnAssignOptions() != flags) {
+											fl.setTagMoveOnAssignOptions(flags);
+											return true;
+										}
+										return false;
+									}
+								});
+
+						params.moveOnAssignTorrent = new BooleanSwtParameter(gFiles,
+								"tag.moveOnAssignTorrent", "label.move.torrent", null,
+								new BooleanSwtParameter.ValueProcessor() {
+									@Override
+									public Boolean getValue(BooleanSwtParameter p) {
+										return(( fl.getTagMoveOnAssignOptions() & TagFeatureFileLocation.FL_TORRENT ) != 0);
+									}
+
+									@Override
+									public boolean setValue(BooleanSwtParameter p, Boolean value) {
+										long flags = fl.getTagMoveOnAssignOptions();
+										if ( value ){
+											flags |= TagFeatureFileLocation.FL_TORRENT;
+										}else{
+											flags &= ~TagFeatureFileLocation.FL_TORRENT;
+										}
+										if (fl.getTagMoveOnAssignOptions() != flags) {
+											fl.setTagMoveOnAssignOptions(flags);
+											return true;
+										}
+										return false;
+									}
+								});
+					}
 				}
 			}
 
@@ -1609,9 +1674,7 @@ public class TagSettingsView
 
 			swt_updateFields();
 		}
-		cMainComposite.layout();
-		Rectangle r = sc.getClientArea();
-		sc.setMinSize(cMainComposite.computeSize(r.width - 16, SWT.DEFAULT));
+		parent.layout( true, true );
 	}
 
 	private void
@@ -1791,6 +1854,9 @@ public class TagSettingsView
 		if (params.moveOnRemoveFolder != null) {
 			params.moveOnRemoveFolder.update();
 		}
+		if (params.moveOnAssignFolder != null) {
+			params.moveOnAssignFolder.update();
+		}
 		if (params.constraints != null ) {
 			Tag tag = tags[0];
 
