@@ -19,6 +19,7 @@
 package com.biglybt.ui.swt.views.skin;
 
 
+import java.io.File;
 import java.net.URL;
 import java.util.List;
 import java.util.*;
@@ -40,14 +41,19 @@ import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.common.ToolBarItem;
 import com.biglybt.ui.common.table.*;
 import com.biglybt.ui.common.table.impl.TableColumnManager;
+import com.biglybt.ui.common.table.impl.TableViewImpl;
 import com.biglybt.ui.common.updater.UIUpdatable;
+import com.biglybt.ui.selectedcontent.SelectedContentManager;
+import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.Messages;
 import com.biglybt.ui.swt.SimpleTextEntryWindow;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.columns.alltrackers.*;
 import com.biglybt.ui.swt.maketorrent.MultiTrackerEditor;
 import com.biglybt.ui.swt.maketorrent.TrackerEditorListener;
+import com.biglybt.ui.swt.mdi.MdiEntrySWT;
 import com.biglybt.ui.swt.pifimpl.UISWTViewBuilderCore;
+import com.biglybt.ui.swt.pifimpl.UISWTViewCore;
 import com.biglybt.ui.swt.skin.SWTSkinObject;
 import com.biglybt.ui.swt.skin.SWTSkinObjectTextbox;
 import com.biglybt.ui.swt.views.MyTorrentsSubView;
@@ -55,6 +61,7 @@ import com.biglybt.ui.swt.views.ViewManagerSWT;
 import com.biglybt.ui.swt.views.table.TableViewSWT;
 import com.biglybt.ui.swt.views.table.TableViewSWTMenuFillListener;
 import com.biglybt.ui.swt.views.table.impl.TableViewFactory;
+import com.biglybt.ui.swt.views.table.impl.TableViewSWT_TabsCommon;
 import com.biglybt.ui.swt.views.table.utils.TableColumnCreator;
 import com.biglybt.ui.swt.views.tableitems.ColumnDateSizer;
 import com.biglybt.ui.swt.views.utils.TagUIUtils;
@@ -64,6 +71,7 @@ import com.biglybt.pif.ui.UIInputReceiverListener;
 import com.biglybt.pif.ui.UIPluginViewToolBarListener;
 import com.biglybt.pif.ui.tables.TableColumn;
 import com.biglybt.pif.ui.tables.TableColumnCreationListener;
+import com.biglybt.plugin.net.buddy.BuddyPluginUtils;
 
 
 public class SBC_AllTrackersView
@@ -224,6 +232,39 @@ public class SBC_AllTrackersView
 					}
 				});
 
+		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersReportedUp.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnAllTrackersReportedUp(column);
+					}
+				});
+		
+		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersReportedDown.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnAllTrackersReportedDown(column);
+					}
+				});
+
+		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersResponseTime.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnAllTrackersResponseTime(column);
+					}
+				});
+
+		
+		tableManager.registerColumn(AllTrackersViewEntry.class, ColumnAllTrackersHasPrivate.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnAllTrackersHasPrivate(column);
+					}
+				});
+
 		
 		tableManager.setDefaultColumnNames(TABLE_NAME,
 				new String[] {
@@ -459,11 +500,27 @@ public class SBC_AllTrackersView
 		long activationType,
 		Object datasource)
 	{
+		boolean isTableSelected = false;
+		if (tv instanceof TableViewImpl) {
+			isTableSelected = ((TableViewImpl) tv).isTableSelected();
+		}
+		if (!isTableSelected) {
+			UISWTViewCore active_view = getActiveView();
+			if (active_view != null) {
+				UIPluginViewToolBarListener l = active_view.getToolBarListener();
+				if (l != null && l.toolBarItemActivated(item, activationType, datasource)) {
+					return true;
+				}
+			}
+			return false;
+		}
+		  
 		if ( tv == null || !tv.isVisible()){
 
 			return( false );
 		}
 
+		/*
 		List<Object> datasources = tv.getSelectedDataSources();
 
 		if ( datasources.size() > 0 ){
@@ -484,7 +541,8 @@ public class SBC_AllTrackersView
 
 			return true;
 		}
-
+		*/
+		
 		return false;
 	}
 
@@ -513,6 +571,14 @@ public class SBC_AllTrackersView
 
 		//list.put( "remove", canEnable ? UIToolBarItem.STATE_ENABLED : 0);
 	}
+	
+	private MdiEntrySWT getActiveView() {
+		TableViewSWT_TabsCommon tabsCommon = tv.getTabsCommon();
+		if (tabsCommon != null) {
+			return tabsCommon.getActiveSubView();
+		}
+		return null;
+	}
 
 	@Override
 	public void
@@ -854,14 +920,104 @@ public class SBC_AllTrackersView
 		
 		assoc_item.setEnabled( hasSelection && !assoc_templates.isEmpty());
 		
+		new MenuItem( menu, SWT.SEPARATOR );
+				
+		String short_key = null;
+		
+		for ( AllTrackersTracker t: trackers ){
+			
+			String sk = t.getShortKey();
+			
+			if ( short_key == null ){
+				
+				short_key = sk;
+				
+			}else if ( !short_key.equals( sk )){
+				
+				short_key = null;
+				
+				break;
+			}
+		}
+		
+		String f_short_key = short_key;
+		
+			// enable logging
+		
+		MenuItem itemEnableLogging = new MenuItem(menu, SWT.CHECK );
+
+		Messages.setLanguageText( itemEnableLogging, "alltorrents.logging.enable", (short_key==null?"":("- *." + short_key )));
+
+		if ( short_key != null ){
+			
+			itemEnableLogging.setSelection( all_trackers.getLoggingEnabled( short_key ));
+
+			itemEnableLogging.addListener(SWT.Selection, (ev)->{
+				
+				all_trackers.setLoggingEnabled( f_short_key, itemEnableLogging.getSelection());
+			});
+		}
+		
+		itemEnableLogging.setEnabled( short_key != null );
+		
+		// view logs
+
+		MenuItem itemViewLogs = new MenuItem(menu, SWT.PUSH );
+
+		Messages.setLanguageText( itemViewLogs, "alltorrents.logging.view" );
+
+		itemViewLogs.addListener(SWT.Selection, (ev)->{
+			
+			File f = all_trackers.getLogFile( f_short_key );
+			
+			UIFunctionsManager.getUIFunctions().showInExplorer(f); 
+		});
+		
+		itemViewLogs.setEnabled( short_key != null && all_trackers.getLoggingEnabled( short_key ));
+		
+		// clear reported stats
+		
+		MenuItem itemClearStats = new MenuItem(menu, SWT.PUSH);
+	
+		Messages.setLanguageText( itemClearStats, "alltorrents.reset.reported.stats" );
+	
+		itemClearStats.addListener(SWT.Selection, new Listener() {
+			@Override
+			public void
+			handleEvent(
+				Event event)
+			{
+				for ( AllTrackersTracker t: trackers ){
+					
+					t.resetReportedStats();
+				}
+			}});	
+		
+		itemClearStats.setEnabled( hasSelection );
+		
+		new MenuItem( menu, SWT.SEPARATOR );
+		
+			// discuss
+		
+		String tracker_key;
+		
+		if ( trackers.size() == 1 ){
+			
+			tracker_key = BuddyPluginUtils.getTrackerChatKey( trackers.get(0).getTrackerName());
+			
+		}else{
+			
+			tracker_key = null;
+		}
+		
+		MenuBuildUtils.addChatMenu( menu, "menu.discuss.tracker", tracker_key );
+		
 			// options
 		
 		Menu options_menu = new Menu( menu.getShell(), SWT.DROP_DOWN);
 
 		boolean opt_enabled = trackers.size() == 1;
-		
-		options_menu.setEnabled( opt_enabled );
-		
+				
 		MenuItem options_item = new MenuItem( menu, SWT.CASCADE);
 
 		Messages.setLanguageText( options_item, "ConfigView.title.full" );
@@ -872,6 +1028,8 @@ public class SBC_AllTrackersView
 
 		Messages.setLanguageText( itemOptCryptoPort, "menu.tracker.cryptoport.disabled" );
 
+		itemOptCryptoPort.setEnabled( opt_enabled );
+		
 		if ( opt_enabled ){
 			
 			AllTrackersTracker tracker = trackers.get(0);
@@ -1214,6 +1372,8 @@ public class SBC_AllTrackersView
 	selected(
 		TableRowCore[] row )
 	{
+		updateSelectedContent();
+		
 		UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 
 		if ( uiFunctions != null ){
@@ -1227,6 +1387,8 @@ public class SBC_AllTrackersView
 	deselected(
 		TableRowCore[] rows )
 	{
+		updateSelectedContent();
+		
 	  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 
 	  	if ( uiFunctions != null ){
@@ -1234,6 +1396,31 @@ public class SBC_AllTrackersView
 	  		uiFunctions.refreshIconBar();
 	  	}
 	}
+	
+	public void updateSelectedContent() {
+		updateSelectedContent( false );
+	}
+
+	public void updateSelectedContent( boolean force ) {
+		if (table_parent == null || table_parent.isDisposed()) {
+			return;
+		}
+			// if we're not active then ignore this update as we don't want invisible components
+			// updating the toolbar with their invisible selection. Note that unfortunately the
+			// call we get here when activating a view does't yet have focus
+
+		if ( !isVisible()){
+			if ( !force ){
+				return;
+			}
+		}
+		
+		SelectedContentManager.clearCurrentlySelectedContent();
+
+		if ( tv != null ){
+			SelectedContentManager.changeCurrentlySelectedContent(tv.getTableID(), null, tv);
+		}
+	}
 
 	@Override
 	public void
@@ -1448,6 +1635,7 @@ public class SBC_AllTrackersView
 		{
 			tracker	= _tracker;
 		}
+		
 		public AllTrackersTracker
 		getTracker()
 		{
@@ -1460,6 +1648,12 @@ public class SBC_AllTrackersView
 			return( tracker.getTrackerName());
 		}
 		
+		public String
+		getShortKey()
+		{
+			return( tracker.getShortKey());
+		}
+		
 		public String
 		getStatusString()
 		{
@@ -1506,6 +1700,39 @@ public class SBC_AllTrackersView
 			tracker.setOptions( options );
 		}
 		
+		@Override
+		public void resetReportedStats(){
+			tracker.resetReportedStats();
+		}
+		
+		@Override
+		public long 
+		getTotalReportedDown()
+		{
+			return( tracker.getTotalReportedDown());
+		}
+		
+		@Override
+		public long 
+		getTotalReportedUp()
+		{
+			return( tracker.getTotalReportedUp());
+		}
+		
+		@Override
+		public long 
+		getAverageRequestDuration()
+		{
+			return( tracker.getAverageRequestDuration());
+		}
+		
+		@Override
+		public int 
+		getPrivatePercentage()
+		{
+			return(tracker.getPrivatePercentage());
+		}
+		
 		public Tag
 		getTag()
 		{
