@@ -32,12 +32,16 @@ import org.eclipse.swt.events.KeyListener;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
 
+import com.biglybt.core.category.Category;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.tag.*;
+import com.biglybt.core.util.AEThread2;
+import com.biglybt.core.util.Base32;
 import com.biglybt.core.util.Constants;
 import com.biglybt.core.util.Debug;
 import com.biglybt.core.util.RegExUtil;
+import com.biglybt.core.util.SystemTime;
 import com.biglybt.ui.UIFunctions;
 import com.biglybt.ui.UIFunctionsManager;
 import com.biglybt.ui.common.ToolBarItem;
@@ -423,6 +427,14 @@ public class SBC_TagsOverview
 						new ColumnTagMoveOnRemove(column);
 					}
 				});
+		
+		tableManager.registerColumn(Tag.class, ColumnTagMoveOnAssign.COLUMN_ID,
+				new TableColumnCreationListener() {
+					@Override
+					public void tableColumnCreated(TableColumn column) {
+						new ColumnTagMoveOnAssign(column);
+					}
+				});
 
 		tableManager.registerColumn(Tag.class, ColumnTagProperties.COLUMN_ID,
 				new TableColumnCreationListener() {
@@ -821,7 +833,7 @@ public class SBC_TagsOverview
 		}
 
 		vm.registerView(PLUGIN_DS_TYPE, new UISWTViewBuilderCore(
-			"TagSettingsView", null, TagSettingsView.class));
+			TagSettingsView.VIEW_ID, null, TagSettingsView.class));
 
 		vm.registerView(PLUGIN_DS_TYPE, new UISWTViewBuilderCore(
 			MyTorrentsSubView.MSGID_PREFIX, null, MyTorrentsSubView.class));
@@ -1042,8 +1054,20 @@ public class SBC_TagsOverview
 						tag.setVisible( true );
 					}
 
-					String id = "Tag." + tag.getTagType().getTagType() + "." + tag.getTagID();
-					uiFunctions.getMDI().showEntryByID(id, tag);
+					if ( tag instanceof Category ){
+						
+						String name = ((Category)tag).getName();
+						
+						String id = "Cat." + Base32.encode(name.getBytes());
+						
+						uiFunctions.getMDI().showEntryByID(id, tag);
+						
+					}else{
+						
+						String id = "Tag." + tag.getTagType().getTagType() + "." + tag.getTagID();
+						
+						uiFunctions.getMDI().showEntryByID(id, tag);
+					}
 				}
 			}
 		}
@@ -1179,6 +1203,31 @@ public class SBC_TagsOverview
 			}else{
 
 				tv.setSelectedRows(new TableRowCore[] { row });
+				
+				AEThread2.createAndStartDaemon(
+					"feh",
+					()->{
+						long start = SystemTime.getMonotonousTime();
+						
+						while( tv.hasChangesPending()){
+							
+							if ( SystemTime.getMonotonousTime() - start > 20*1000 ){
+								
+								break;
+							}
+							
+							try{
+								Thread.sleep(100);
+							}catch( Throwable e ){
+								
+							}
+						}
+							
+						Utils.execSWTThread(()->{
+							tv.getTabsCommon().getMDI().showEntryByID( TagSettingsView.VIEW_ID );
+							tv.showRow( row ); 
+						});
+					});
 			}
 		}
 	}
