@@ -263,15 +263,16 @@ public class SB_Transfers
 		statsWithLowNoise = new stats();
 		statsWithLowNoise.includeLowNoise = true;
 		
+		refresh_limiter = new FrequencyLimitedDispatcher(
+				new AERunnable() {
+					@Override
+					public void runSupport() {
+						refreshAllLibrariesSupport( vuze_ui );
+					}
+				}, 250);
+		refresh_limiter.setSingleThreaded();
+		
 		if ( vuze_ui ){
-			refresh_limiter = new FrequencyLimitedDispatcher(
-					new AERunnable() {
-						@Override
-						public void runSupport() {
-							refreshAllLibrariesSupport();
-						}
-					}, 250);
-			refresh_limiter.setSingleThreaded();
 	
 			MdiEntryCreationListener libraryCreator = new MyMdiEntryCreationListener(mdi);
 			mdi.registerEntry(SideBar.SIDEBAR_SECTION_LIBRARY, libraryCreator);
@@ -1292,10 +1293,10 @@ public class SB_Transfers
 		ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
 	}
 
-	private void setupCategory(final Category category) {
+	public MdiEntry setupCategory(final Category category) {
 		MultipleDocumentInterface mdi = UIFunctionsManager.getUIFunctions().getMDI();
 		if (mdi == null) {
-			return;
+			return(null);
 		}
 
 		String name = category.getName();
@@ -1304,7 +1305,7 @@ public class SB_Transfers
 		MdiEntry existing = mdi.getEntry( id );
 
 		if ( existing != null ){
-			return;
+			return( existing );
 		}
 		
 		category.addCategoryListener( categoryListener );
@@ -1320,8 +1321,24 @@ public class SB_Transfers
 			@Override
 			public Object getTitleInfoProperty(int propertyID) {
 				if (propertyID == TITLE_INDICATOR_TEXT) {
-					if (statsNoLowNoise.numIncomplete > 0) {
+					if ( category.getType() == Category.TYPE_ALL ){
+						
+						int all = core.getGlobalManager().getDownloadManagers().size();
+						
+						return( String.valueOf( all ));
+						
+					}else if ( category.getType() == Category.TYPE_UNCATEGORIZED ){
+						
+						int all = core.getGlobalManager().getDownloadManagers().size();
+
+						int	num_cat = CategoryManager.getCategorisedDownloadCount();
+						
+						return( String.valueOf( all - num_cat ));
+
+					}else{
+						
 						List<?> dms = category.getDownloadManagers(null);
+						
 						if (dms != null) {
 							return "" + dms.size();
 						}
@@ -1392,6 +1409,7 @@ public class SB_Transfers
 			});
 		}
 
+		return( entry );
 	}
 
 	private void removeCategory(Category category) {
@@ -1621,6 +1639,8 @@ public class SB_Transfers
 										
 										addMenuCollapseAll( mdi, menuTree, entry.getViewID());
 										
+										new org.eclipse.swt.widgets.MenuItem( menuTree, SWT.SEPARATOR );
+										
 										TagUIUtils.createSideBarMenuItems(menuTree, tg );
 									}
 								});
@@ -2444,99 +2464,105 @@ public class SB_Transfers
 		}
 	}
 
-	void refreshAllLibrariesSupport() {
+	void refreshAllLibrariesSupport( boolean vuze_ui ) {
 		for (countRefreshListener l : listeners) {
 			l.countRefreshed(statsWithLowNoise, statsNoLowNoise);
 		}
-		MultipleDocumentInterface mdi = UIFunctionsManager.getUIFunctions().getMDI();
-		if (mdi == null) {
-			return;
-		}
-
-			// don't mess with things until MDI initialized as it might be auto-opening views
 		
-		if ( mdi.isInitialized()){
-			
-			boolean showDownloading = COConfigurationManager.getBooleanParameter( "Show Downloading In Side Bar" );
+		if ( vuze_ui ){
+			MultipleDocumentInterface mdi = UIFunctionsManager.getUIFunctions().getMDI();
+			if (mdi == null) {
+				return;
+			}
 	
-			if ( showDownloading && statsNoLowNoise.numIncomplete > 0){
+				// don't mess with things until MDI initialized as it might be auto-opening views
+			
+			if ( mdi.isInitialized()){
 				
-				MdiEntry entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL);
-				
-				if (entry == null) {
-					long now = SystemTime.getMonotonousTime();
+				boolean showDownloading = COConfigurationManager.getBooleanParameter( "Show Downloading In Side Bar" );
+		
+				if ( showDownloading && statsNoLowNoise.numIncomplete > 0){
 					
-						// prevent rapid show/hides and also trt and work around bug where 2 Downloading entries have
-						// been seen to get created (lame fix I know)
+					MdiEntry entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL);
 					
-					if ( now - last_dl_entry_load > 5000 ){
+					if (entry == null) {
+						long now = SystemTime.getMonotonousTime();
 						
-						last_dl_entry_load = now;
-											
-						mdi.loadEntryByID(SideBar.SIDEBAR_SECTION_LIBRARY_DL, false);
+							// prevent rapid show/hides and also trt and work around bug where 2 Downloading entries have
+							// been seen to get created (lame fix I know)
+						
+						if ( now - last_dl_entry_load > 5000 ){
+							
+							last_dl_entry_load = now;
+												
+							mdi.loadEntryByID(SideBar.SIDEBAR_SECTION_LIBRARY_DL, false);
+						}
+					}
+				} else {
+					mdi.closeEntry(mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL), false );
+				}
+			}else{
+				refreshAllLibraries();
+			}
+			
+			MdiEntry entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL);
+			if (entry != null) {
+				List<? extends MdiEntryVitalityImage> vitalityImages = entry.getVitalityImages();
+				for (MdiEntryVitalityImage vitalityImage : vitalityImages) {
+					String imageID = vitalityImage.getImageID();
+					if (imageID == null) {
+						continue;
+					}
+					if (imageID.equals(ID_VITALITY_ACTIVE)) {
+						//replace this annoying spinner with color change
+						//vitalityImage.setVisible(statsNoLowNoise.numDownloading > 0);
+	
+					} else if (imageID.equals(ID_VITALITY_ALERT)) {
+						vitalityImage.setVisible(statsNoLowNoise.numErrorInComplete > 0);
+						if (statsNoLowNoise.numErrorInComplete > 0) {
+							vitalityImage.setToolTip(statsNoLowNoise.errorInCompleteTooltip);
+						}
 					}
 				}
-			} else {
-				mdi.closeEntry(mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL), false );
+				ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
+				
+				requestRedraw( entry );
+			}
+	
+			entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_CD);
+			if (entry != null) {
+				List<? extends MdiEntryVitalityImage> vitalityImages = entry.getVitalityImages();
+				for (MdiEntryVitalityImage vitalityImage : vitalityImages) {
+					String imageID = vitalityImage.getImageID();
+					if (imageID == null) {
+						continue;
+					}
+					if (imageID.equals(ID_VITALITY_ALERT)) {
+						vitalityImage.setVisible(statsNoLowNoise.numErrorComplete > 0);
+						if (statsNoLowNoise.numErrorComplete > 0) {
+							vitalityImage.setToolTip(statsNoLowNoise.errorCompleteTooltip);
+						}
+					}
+				}
+			}
+	
+			entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_UNOPENED);
+			if (entry != null) {
+				ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
+				
+				requestRedraw( entry );
+			}
+			
+			entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY);
+			if (entry != null) {
+				ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
+				
+				requestRedraw( entry );
 			}
-		}else{
-			refreshAllLibraries();
 		}
 		
-		MdiEntry entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_DL);
-		if (entry != null) {
-			List<? extends MdiEntryVitalityImage> vitalityImages = entry.getVitalityImages();
-			for (MdiEntryVitalityImage vitalityImage : vitalityImages) {
-				String imageID = vitalityImage.getImageID();
-				if (imageID == null) {
-					continue;
-				}
-				if (imageID.equals(ID_VITALITY_ACTIVE)) {
-					//replace this annoying spinner with color change
-					//vitalityImage.setVisible(statsNoLowNoise.numDownloading > 0);
-
-				} else if (imageID.equals(ID_VITALITY_ALERT)) {
-					vitalityImage.setVisible(statsNoLowNoise.numErrorInComplete > 0);
-					if (statsNoLowNoise.numErrorInComplete > 0) {
-						vitalityImage.setToolTip(statsNoLowNoise.errorInCompleteTooltip);
-					}
-				}
-			}
-			ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
-			
-			requestRedraw( entry );
-		}
-
-		entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_CD);
-		if (entry != null) {
-			List<? extends MdiEntryVitalityImage> vitalityImages = entry.getVitalityImages();
-			for (MdiEntryVitalityImage vitalityImage : vitalityImages) {
-				String imageID = vitalityImage.getImageID();
-				if (imageID == null) {
-					continue;
-				}
-				if (imageID.equals(ID_VITALITY_ALERT)) {
-					vitalityImage.setVisible(statsNoLowNoise.numErrorComplete > 0);
-					if (statsNoLowNoise.numErrorComplete > 0) {
-						vitalityImage.setToolTip(statsNoLowNoise.errorCompleteTooltip);
-					}
-				}
-			}
-		}
-
-		entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY_UNOPENED);
-		if (entry != null) {
-			ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
-			
-			requestRedraw( entry );
-		}
-		
-		entry = mdi.getEntry(SideBar.SIDEBAR_SECTION_LIBRARY);
-		if (entry != null) {
-			ViewTitleInfoManager.refreshTitleInfo(entry.getViewTitleInfo());
-			
-			requestRedraw( entry );
-		}
+		RefreshCategorySideBar( CategoryManager.getCategory( Category.TYPE_ALL ));
+		RefreshCategorySideBar( CategoryManager.getCategory( Category.TYPE_UNCATEGORIZED ));
 	}
 
 	private static boolean	TABLE_SUBCONFIG_ENABLE = COConfigurationManager.getBooleanParameter( "Library.EnableSepColConfig" );
