@@ -40,7 +40,6 @@ import com.biglybt.ui.common.updater.UIUpdater;
 import com.biglybt.ui.common.viewtitleinfo.ViewTitleInfo;
 import com.biglybt.ui.mdi.MdiEntry;
 import com.biglybt.ui.mdi.MultipleDocumentInterface;
-import com.biglybt.ui.swt.DelayedListenerMultiCombiner;
 import com.biglybt.ui.swt.FixedHTMLTransfer;
 import com.biglybt.ui.swt.FixedURLTransfer;
 import com.biglybt.ui.swt.Utils;
@@ -57,6 +56,7 @@ import com.biglybt.ui.swt.skin.*;
 import com.biglybt.ui.swt.uiupdater.UIUpdaterSWT;
 import com.biglybt.ui.swt.utils.FontUtils;
 import com.biglybt.ui.swt.views.IViewAlwaysInitialize;
+import com.biglybt.ui.swt.views.QuickLinksView;
 import com.biglybt.ui.swt.views.ViewManagerSWT;
 import com.biglybt.ui.swt.views.skin.SkinnedDialog;
 
@@ -215,6 +215,25 @@ public class SideBar
 			}
 		});
 
+		
+		Display.getDefault().addFilter(SWT.Traverse,(ev)->{
+			
+			if ( 	ev.character == '\t' &&
+					(ev.stateMask & (SWT.MOD1 + SWT.SHIFT)) == SWT.MOD1 ) {
+
+				showNext();
+				
+				ev.doit = false;
+				
+			}else  if ( 	ev.character == '\t' &&
+					(ev.stateMask & (SWT.MOD1 + SWT.SHIFT)) == SWT.MOD1 + SWT.SHIFT) {
+
+				showPrevious();
+				
+				ev.doit = false;
+			}
+		});
+		
 		return null;
 	}
 
@@ -264,11 +283,35 @@ public class SideBar
 	
 			menuParentItem.setStyle( MenuItem.STYLE_MENU );
 						
+			String ql_res = "v3.MainWindow.menu.view.quick-links";
+			
 			menuParentItem.addFillListener((menu, data) -> {
+				
 				SideBarEntrySWT entry = getCurrentEntry();
-				menu.setVisible(entry != null && entry.canBuildStandAlone()
-						&& !entry.getViewID().equals(
-								MultipleDocumentInterface.SIDEBAR_HEADER_DASHBOARD));
+				
+				boolean visible = 
+					entry != null && 
+					entry.canBuildStandAlone() &&
+					!entry.getViewID().equals( MultipleDocumentInterface.SIDEBAR_HEADER_DASHBOARD);
+			
+				menu.setVisible( visible );
+				
+				if ( visible ){
+					
+					boolean ql_ok = QuickLinksView.canAddItem( this, entry );
+					
+					MenuItem[] items = menu.getItems();
+						
+					for ( MenuItem item: items ){
+						
+						String res = item.getResourceKey();
+							
+						if ( res.equals( ql_res )){
+							
+							item.setVisible( ql_ok );
+						}
+					}
+				}
 			});
 	
 			MenuItem menuItemDashBoard = menuManager.addMenuItem( menuParentItem, "label.dashboard");
@@ -298,6 +341,20 @@ public class SideBar
 					}
 				}
 			});
+			
+			MenuItem menuItemQuickLinks = menuManager.addMenuItem( menuParentItem, ql_res );
+
+			menuItemQuickLinks.addListener(new MenuItemListener() {
+				@Override
+				public void selected(MenuItem menu, Object target) {
+					SideBarEntrySWT sbe = getCurrentEntry();
+	
+					if ( sbe != null ){
+						
+						QuickLinksView.addItem( SideBar.this, sbe );
+					}
+				}
+			});
 		}
 		
 		{
@@ -2047,38 +2104,27 @@ public class SideBar
 	
 		// track entry additions and selection so we can switch to previous entry when one is closed
 
-	private final Stack<String> entryViewHistory = new Stack<>();
+	private final Stack<String> entryViewHistory 	= new Stack<>();
+	private final Stack<String> entryViewFuture 	= new Stack<>();
 
 	@Override
 	public void addItem(BaseMdiEntry entry) {
 		super.addItem( entry );
-		synchronized(entryViewHistory){
-			String id = entry.getViewID();
-			entryViewHistory.remove(id);
-			if ( entry.isSelectable()){
-				entryViewHistory.push( id );
-			}
-		}
+		// addHistory( entry ); don't want this simply on addition, wait for selection
 	}
 
 	@Override
 	protected void
 	itemSelected(MdiEntry entry ){
 		super.itemSelected( entry );
-		synchronized(entryViewHistory){
-			String id = entry.getViewID();
-			entryViewHistory.remove(id);
-			if ( entry.isSelectable()){
-				entryViewHistory.push(id);
-			}
-		}
+		addHistory( entry );
 	}
 
 	@Override
-	public BaseMdiEntry closeEntryByID(String id) {
+	public BaseMdiEntry closeEntryByID(String id, boolean user_initiated ) {
 		MdiEntry currentBeforeClose = getCurrentEntry();
 
-		BaseMdiEntry entry = super.closeEntryByID(id);
+		BaseMdiEntry entry = super.closeEntryByID(id, user_initiated);
 		if (entry == null || Utils.isDisplayDisposed()) {
 			return entry;
 		}
@@ -2108,16 +2154,133 @@ public class SideBar
 			}
 		}
 		
-		if (next == null) {
-			// OSX doesn't select a treeitem after closing an existing one
-			// Force selection
+		if ( next == null ){
+			
+				// OSX doesn't select a treeitem after closing an existing one
+				// Force selection
+			
 			next = SideBar.SIDEBAR_SECTION_LIBRARY;
 		}
 
 		showEntryByID( next );
+		
 		return entry;
 	}
 
+	private void
+	addHistory(
+		MdiEntry	entry )
+	{
+		synchronized(entryViewHistory){
+			String id = entry.getViewID();
+			if ( !entryViewHistory.remove(id)){
+				if ( !entryViewFuture.contains( id )){
+					entryViewFuture.clear();
+				}
+			}
+			if ( entry.isSelectable()){
+				if ( !isInitialized()){
+					entryViewHistory.clear();	// don't build up a load of history cruff during startup
+				}
+				entryViewHistory.push( id );
+			}
+		}
+	}
+	
+	private void
+	showNext()
+	{
+		MdiEntry current = getCurrentEntry();
+
+		String next = null;
+		
+		synchronized( entryViewHistory ){
+			
+			while( !entryViewFuture.isEmpty()){
+				
+				String maybe_next = entryViewFuture.pop();
+												
+				if ( entryExists( maybe_next )){
+				
+					if ( current != null && current.getViewID().equals( maybe_next )){
+						
+						continue;
+					}
+					
+					next = maybe_next;
+					
+					entryViewHistory.push( next );  // prepare for show-next so we keep future history
+					
+					break;
+				}
+			}
+		}
+		
+		if ( next != null ){
+
+			showEntryByID( next );
+		}
+	}
+	
+	private void
+	showPrevious()
+	{
+		MdiEntry current = getCurrentEntry();
+		
+		String next = null;
+
+		synchronized( entryViewHistory ){
+			
+			boolean	future_added = false;
+			
+			if ( current != null && entryExists( current.getViewID()) && current.isSelectable()){
+				
+				String cid = current.getViewID();
+				
+				if ( entryViewFuture.isEmpty() || !entryViewFuture.peek().equals( cid )){
+				
+					entryViewFuture.push( cid );
+				}
+				
+				future_added = true;
+			}
+						
+			while( !entryViewHistory.isEmpty()){
+				
+				String maybe_next = entryViewHistory.pop();
+					
+				if ( current != null && current.getViewID().equals( maybe_next) ){
+										
+					continue;
+				}
+				
+				if ( entryExists( maybe_next )){
+
+					next = maybe_next;
+					
+					if ( !future_added ){
+					
+						if ( entryViewFuture.isEmpty() || !entryViewFuture.peek().equals( next )){
+
+							entryViewFuture.push( next );
+						}
+					}
+
+					entryViewHistory.push( next );	// prepare for show-next so we keep future history
+					
+					break;
+				}
+			}
+		}
+				
+		if ( next == null ){
+			
+			next = SideBar.SIDEBAR_SECTION_LIBRARY;
+		}
+
+		showEntryByID( next );
+	}
+	
 	@Override
 	public void generate(IndentWriter writer) {
 		MdiEntrySWT[] entries = getEntries();
