@@ -31,6 +31,7 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.widgets.*;
 
+import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.util.*;
 import com.biglybt.pifimpl.local.ui.menus.MenuItemImpl;
@@ -64,7 +65,18 @@ public class TableViewSWT_Common
 	private static final int ASYOUTYPE_MODE = ASYOUTYPE_MODE_FILTER;
 	private static final int ASYOUTYPE_UPDATEDELAY = 300;
 
-	private static final Color COLOR_FILTER_REGEX	= Colors.fadedYellow;
+	private static Color COLOR_FILTER_REGEX;
+	private static Color COLOR_FILTER_NO_REGEX;
+	
+	static{
+		COConfigurationManager.addWeakParameterListener((n)->{
+			
+			COLOR_FILTER_NO_REGEX 	= Utils.getConfigColor( "table.filter.active.colour", Colors.fadedBlue );
+			COLOR_FILTER_REGEX 		= Utils.getConfigColor( "table.filter.regex.colour", Colors.fadedYellow );
+			
+		}, true, "table.filter.active.colour", "table.filter.regex.colour" );	
+	}
+	
 	
 	private Font FONT_NO_REGEX;
 	private Font FONT_REGEX;
@@ -184,6 +196,18 @@ public class TableViewSWT_Common
 						&& lastMouseUpPos.x == e.x && lastMouseUpPos.y == e.y) {
 					// Fake double click because Cocoa SWT 3650 doesn't always trigger
 					// DefaultSelection listener on a Tree on dblclick (works find in Table)
+					
+					// parg, 07/02/20 - not sure if this is still an issue but we're sometimes running the default action even though it
+					// has been cancelled - reset the cancel indicator if we've had a recent real double click and cancel. obviously crap...
+					
+					long now = System.currentTimeMillis();
+					
+					if (	time  - lastMouseDblClkEventTime < 1000 &&
+							now - lCancelSelectionTriggeredOn < 1000 ){
+						
+						lCancelSelectionTriggeredOn = now;
+					}
+					
 					runDefaultAction(e.stateMask, 0 );
 					return;
 				}
@@ -551,8 +575,14 @@ public class TableViewSWT_Common
 		if (event.stateMask == 0) {
 			if (filter != null && filter.widget == event.widget) {
 				if (event.keyCode == SWT.ARROW_DOWN) {
-					tv.setFocus();
-					event.doit = false;
+					tv.requestFocus( 2 );
+						// for detailed library views we want to give the seeding section a chance to grab
+						// focus if the incomplete section is empty - code below allows the 'other' view a chance
+						// to do this
+					int rows = tv.getRowCount();
+					if ( rows > 0 ){
+						event.doit = false;
+					}
 				} else if (event.character == 13) {
 					tv.refilter();
 				}
@@ -652,20 +682,35 @@ public class TableViewSWT_Common
 	validateFilterRegex()
 	{
 		TableViewSWTFilter<?> filter = tv.getSWTFilter();
+		
 		Text widget = filter.widget;
+				
 		Color old_bg = (Color)widget.getData( "TVSWTC:filter.bg" );
 		if ( old_bg == null ){
 			old_bg = widget.getBackground();
 			widget.setData( "TVSWTC:filter.bg", old_bg);
 		}
+		Color old_fg = (Color)widget.getData( "TVSWTC:filter.fg" );
+		if ( old_fg == null ){
+			old_fg = widget.getForeground();
+			widget.setData( "TVSWTC:filter.fg", old_fg);
+		}
 		if (filter.regex) {
 			if ( FONT_NO_REGEX == null ){
-				FONT_NO_REGEX = widget.getFont();
-				FontData[] fd = FONT_NO_REGEX.getFontData();
+				Font font = widget.getFont();
+				
+				FontData[] fd = font.getFontData();
+				for (int i = 0; i < fd.length; i++) {
+					fd[i].setStyle(SWT.NORMAL);
+				}
+				FONT_NO_REGEX = new Font( widget.getDisplay(), fd );
+				
+				fd = FONT_NO_REGEX.getFontData();
 				for (int i = 0; i < fd.length; i++) {
 					fd[i].setStyle(SWT.BOLD);
 				}
 				FONT_REGEX = new Font( widget.getDisplay(), fd );
+				
 				for (int i = 0; i < fd.length; i++) {
 					fd[i].setStyle(SWT.ITALIC);
 				}
@@ -673,7 +718,7 @@ public class TableViewSWT_Common
 				
 				widget.addDisposeListener(
 					(e)->{
-						FONT_NO_REGEX = null;
+						FONT_NO_REGEX.dispose();
 						FONT_REGEX.dispose();
 						FONT_REGEX_ERROR.dispose();
 					});
@@ -684,17 +729,31 @@ public class TableViewSWT_Common
 				Pattern.compile(filter.nextText, Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE );
 				
 				widget.setBackground(COLOR_FILTER_REGEX);
+				widget.setForeground( old_fg );
+				
 				widget.setFont( FONT_REGEX );
 
-				Messages.setLanguageTooltip(widget,"MyTorrentsView.filter.tooltip");
+				if ( filter.nextText.isEmpty()){
+					Messages.setLanguageTooltip(widget,"MyTorrentsView.filter.tooltip");
+				}else{
+					Utils.setTT(widget, null); // prevent TT from appearing as you type into field
+				}
 			} catch (Exception e) {
 				widget.setBackground(Colors.colorErrorBG);
+				widget.setForeground( old_fg );
 				Utils.setTT(widget,e.getMessage());
 				widget.setFont( FONT_REGEX_ERROR );
 			}
 		} else {
-			widget.setBackground(old_bg);
-			Messages.setLanguageTooltip(widget,"MyTorrentsView.filter.tooltip");
+			Color bg = filter.nextText==null||filter.nextText.isEmpty()?old_bg:COLOR_FILTER_NO_REGEX;
+			Color fg = bg==COLOR_FILTER_NO_REGEX?Colors.getInstance().getReadableColor(bg):old_bg;
+			widget.setBackground( bg );
+			widget.setForeground( fg );
+			if ( filter.nextText.isEmpty()){
+				Messages.setLanguageTooltip(widget,"MyTorrentsView.filter.tooltip");
+			}else{
+				Utils.setTT(widget, null); // prevent TT from appearing as you type into field
+			}
 			if ( FONT_NO_REGEX != null ){
 				widget.setFont( FONT_NO_REGEX );
 			}
@@ -1052,6 +1111,10 @@ public class TableViewSWT_Common
 									}
 								}
 							}
+							
+							@Override
+							public void buildComplete(Menu menu){
+							}
 						});
 			}
 		}
