@@ -45,6 +45,7 @@ import com.biglybt.ui.selectedcontent.SelectedContentListener;
 import com.biglybt.ui.selectedcontent.SelectedContentManager;
 import com.biglybt.ui.swt.MenuBuildUtils;
 import com.biglybt.ui.swt.SimpleTextEntryWindow;
+import com.biglybt.ui.swt.TextWithHistory;
 import com.biglybt.ui.swt.Utils;
 import com.biglybt.ui.swt.debug.ObfuscateImage;
 import com.biglybt.ui.swt.debug.UIDebugGenerator;
@@ -58,7 +59,9 @@ import com.biglybt.ui.swt.views.table.*;
 import com.biglybt.ui.swt.views.table.impl.TableTooltips;
 import com.biglybt.ui.swt.views.table.impl.TableViewSWT_Common;
 import com.biglybt.ui.swt.views.table.impl.TableViewSWT_TabsCommon;
-
+import com.biglybt.pif.download.Download;
+import com.biglybt.pif.download.DownloadTypeComplete;
+import com.biglybt.pif.download.DownloadTypeIncomplete;
 import com.biglybt.pif.ui.UIInputReceiver;
 import com.biglybt.pif.ui.UIInputReceiverListener;
 import com.biglybt.pif.ui.tables.TableRowMouseEvent;
@@ -356,7 +359,13 @@ public class TableViewPainted
 					}
 				}
 				if (getComposite() != event.widget) {
-					super.keyPressed(event);
+					if ( getComposite().isVisible()){
+							// we only want super to get the opportunity to consume the event if it is visible
+							// the way search works we come through here multiple times for various table views (up to three
+							// if simple/detailed library views have been visited) and we don't want an invisible one
+							// to mark the event as 'doit=false' and prevent a visible one from processing it...
+						super.keyPressed(event);
+					}
 					return;
 				}
 				boolean updateTable = false;
@@ -896,14 +905,51 @@ public class TableViewPainted
 	 * @see TableView#setFocus()
 	 */
 	@Override
-	public void setFocus() {
+	public void requestFocus( int reason ) {
 		Utils.execSWTThread(new AERunnable() {
 			@Override
 			public void runSupport() {
 				if (isDisposed()) {
 					return;
 				}
-				cTable.setFocus();
+				
+					// if we have an active search then we don't want to grab focus
+					// away from the text box due to a selected-content change
+				
+				if ( reason == 1 ){
+					
+					TableViewSWTFilter<?> filter = getSWTFilter();
+					
+					if ( filter != null ){ // && filter.widget.isFocusControl()){
+						
+						String text = filter.text;
+						
+						if ( text != null && !text.isEmpty()){
+							
+							return;
+						}
+					}
+					
+					cTable.setFocus();
+					
+				}else if ( reason == 2 ){
+				
+					if ( getSelectedRowsSize() == 0 ){
+						
+						TableRowCore[] rows = getRows();
+						
+						if ( rows.length > 0 ){
+							
+							setSelectedRows( new TableRowCore[]{ rows[0] });
+						}
+					}
+					
+					cTable.setFocus();
+					
+				}else{
+					
+					cTable.setFocus();
+				}
 			}
 		});
 	}
@@ -1559,6 +1605,9 @@ public class TableViewPainted
 			FrequencyLimitedDispatcher(
 				AERunnable.create(()->{
 					Utils.execSWTThread(()->{
+						if ( cTable.isDisposed()){
+							return;
+						}
 						if (DEBUG_SELECTION) {
 							debug("vBar changed " + vBar.getSelection() + " via " + Debug.getCompressedStackTrace());
 						}
@@ -1892,6 +1941,34 @@ public class TableViewPainted
 		}
 		filter.widget = txtFilter;
 		if (txtFilter != null) {
+			
+			Class<?> cla = getDataSourceType();
+			
+			String historyKey = "";
+			
+			if ( cla != null ){
+				if ( cla == DownloadTypeComplete.class || cla == DownloadTypeIncomplete.class || cla == Download.class ){
+					// default, leave blank
+				}else{
+					historyKey = "." + cla.getName();	// different history for different table types
+				}
+			}
+			
+				// must create this before adding other listeners so it gets priority over key events
+			
+			TextWithHistory twh = new TextWithHistory( "tableviewpainted.search" + historyKey, "table.filter.history", txtFilter );
+			
+				// disable as interferes with key-down into search results feature 
+			
+			twh.setKeDownShowsHistory( false );
+			
+			txtFilter.addListener( SWT.FocusOut, (ev)->{
+				String text = txtFilter.getText().trim();
+				if ( !text.isEmpty()){
+					twh.addHistory( text );
+				}
+			});
+			
 			txtFilter.addKeyListener(tvSWTCommon);
 
 			filter.widgetModifyListener = new ModifyListener() {
@@ -1931,6 +2008,9 @@ public class TableViewPainted
 			} else {
 				filter.text = filter.nextText = txtFilter.getText();
 			}
+			
+
+
 		} else {
 			filter.text = filter.nextText = "";
 		}
